Package: XYomics
Title: Analysis of Sex Differences in Omics Data for Complex Diseases
Version: 0.1.4
Authors@R: 
  c(
    person("Enrico", "Glaab", email = "enrico.glaab@uni.lu", role = c("aut", "cre")),
    person("Sophie", "Le Bars", email = "sophie.lebars@uni.lu", role = "aut"),
    person("Mohamed", "Soudy", email = "mohamed.soudy@uni.lu", role = "aut"),
    person("Murodzhon", "Akhmedov", role = c("cph"))
  )
Maintainer: Enrico Glaab <enrico.glaab@uni.lu>
Description: Tools to analyze sex differences in omics data for complex diseases. It includes functions for differential expression analysis using the 'limma' method <doi:10.1093/nar/gkv007>, interaction testing between sex and disease, pathway enrichment with 'clusterProfiler' <doi:10.1089/omi.2011.0118>, and gene regulatory network (GRN) construction and analysis using 'igraph'. The package enables a reproducible workflow from raw data processing to biological interpretation. 
Depends: R (>= 3.6)
Imports: limma, igraph, edgeR, SeuratObject, Seurat, data.table,
        ggplot2, tidyr, grid, ggraph, dplyr, AnnotationDbi, ggrepel,
        Rcpp, cowplot, patchwork, methods, DESeq2, S4Vectors,
        clusterProfiler
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: R.utils, DT, gridExtra, knitr, htmltools, kableExtra,
        rmarkdown, stringr, ReactomePA, org.Hs.eg.db
LinkingTo: Rcpp
VignetteBuilder: knitr, rmarkdown
NeedsCompilation: yes
Packaged: 2026-05-11 12:25:32 UTC; sophie
Author: Enrico Glaab [aut, cre],
  Sophie Le Bars [aut],
  Mohamed Soudy [aut],
  Murodzhon Akhmedov [cph]
Repository: CRAN
Date/Publication: 2026-05-11 16:40:03 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-11 19:08:31 UTC; unix
Archs: XYomics.so.dSYM

# Generated by roxygen2: do not edit by hand

export(PCSF)
export(categorize_sex)
export(categorized_enrich)
export(construct_ppi_pcsf)
export(convert_gene_ids)
export(generate_cat_report)
export(generate_dotplot_sc)
export(generate_violinplot_bulk)
export(get_string_network)
export(get_top_hubs)
export(plot_enrichment_dotplots)
export(plot_network)
export(plot_network_pipeline)
export(plot_volcano_deg)
export(ppi_pipeline)
export(sex_interaction_analysis_bulk)
export(sex_interaction_analysis_sc)
export(sex_interaction_pipeline_sc)
export(sex_stratified_analysis_bulk)
export(sex_stratified_analysis_sc)
export(sex_stratified_pipeline_sc)
export(validate_input_bulk)
export(validate_input_sc)
exportPattern("^[[:alpha:]]+")
import(dplyr)
import(ggplot2)
import(ggraph)
import(ggrepel)
import(grid)
import(limma)
import(patchwork)
import(tidyr)
importFrom(AnnotationDbi,keytypes)
importFrom(Rcpp,evalCpp)
importFrom(Seurat,"Idents<-")
importFrom(Seurat,DefaultAssay)
importFrom(Seurat,DotPlot)
importFrom(Seurat,FindMarkers)
importFrom(Seurat,GetAssayData)
importFrom(Seurat,Idents)
importFrom(SeuratObject,"DefaultAssay<-")
importFrom(SeuratObject,"Idents<-")
importFrom(SeuratObject,DefaultAssay)
importFrom(SeuratObject,Idents)
importFrom(clusterProfiler,dotplot)
importFrom(cowplot,draw_grob)
importFrom(cowplot,draw_plot)
importFrom(cowplot,get_legend)
importFrom(cowplot,ggdraw)
importFrom(data.table,fread)
importFrom(dplyr,"%>%")
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,slice_min)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,filterByExpr)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggrepel,geom_text_repel)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,betweenness)
importFrom(igraph,components)
importFrom(igraph,degree)
importFrom(igraph,delete_vertices)
importFrom(igraph,ecount)
importFrom(igraph,edge_attr_names)
importFrom(igraph,ends)
importFrom(igraph,graph.data.frame)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,induced_subgraph)
importFrom(igraph,is_igraph)
importFrom(igraph,neighbors)
importFrom(igraph,simplify)
importFrom(igraph,vcount)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(methods,slotNames)
importFrom(patchwork,wrap_plots)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(tidyr,pivot_longer)
importFrom(utils,download.file)
useDynLib(XYomics, .registration = TRUE)

# XYomics - R package for the analysis of condition-specific sex differences in omics data

## Table of contents
- [Introduction](#introduction)
- [Content](#content)
- [Data](#data)
- [Requirements](#requirements)
- [License](#license)
- [Instructions](#instructions)

## Introduction

This R package provides tools for analyzing condition-specific molecular sex differences in omics data. It enables identification of sex-specific, sex-dimorphic, and sex-modulated genes through differential expression and interaction analyses, accompanied by pathway enrichment and gene regulatory network analysis capabilities.

## Downloads

### Package File

You can download the latest version of the XYomics package here:

- [XYomics_0.1.1.tar.gz](https://gitlab.com/uniluxembourg/lcsb/bds/xyomics/-/raw/main/XYomics_0.1.1.tar.gz)

### Checksum

MD5 Checksum: `2c1bba2aa8291e4edfbb0e50c86b52a4`

## Content

Five main analysis modules (+ dedicated implementations for single-cell data analysis):

1.  **Differential Expression Analysis**: Identifies sex-specific and sex-dimorphic genes across conditions in bulk and single-cell data.

2.  **Sex Interaction Analysis**: Detects sex-modulated genes through interaction analysis in bulk and single-cell data.

3.  **Pathway Enrichment Analysis**: Performs pathway analysis on sex-biased gene sets for bulk and single-cell data.

4.  **Gene Regulatory Network (GRN) Analysis**: Constructs and analyzes condition-specific gene regulatory networks for single-cell data.

5.  **Plotting and Reporting Functions**: Generates visualizations and comprehensive analysis reports.

## Data

Includes example datasets and supports various omics data types (RNA-seq, microarray, proteomics). Input data should be formatted as described in the vignettes.

## Example output
 Please find below the vignettes for the XYomics R-packages:

- Vignette for sex-dependent analysis of bulk RNA-seq data: [XYomics_bulk_example](https://gitlab.com/uniluxembourg/lcsb/bds/xyomics/-/blob/main/xyomics/vignettes/XYomics_bulk_example.Rmd)
- Vignette for sex-dependent analysis of single-cell RNA-seq data: [XYomics_sc_example](https://gitlab.com/uniluxembourg/lcsb/bds/xyomics/-/blob/main/xyomics/vignettes/XYomics_sc_example.Rmd)

## Requirements

Required R version: ≥ 4.2.0

Key dependencies:

- **Bioconductor:** DESeq2, limma, edgeR, clusterProfiler, org.Hs.eg.db, ReactomePA  
- **CRAN:** ggplot2, dplyr, igraph  

## License

MIT License

## Instructions

### Install from GitLab

- For Ubuntu users, please install the following dependencies before running the installation script:
```
apt-get install libcurl-dev libcurl4-openssl-dev libudunits2-dev libgdal-dev libharfbuzz-dev libfribidi-dev libharfbuzz-dev libfribidi-dev
```

#### 1. Install the dependencies:

```{r}
install_XYpackages <- function() {
  # Helper to check which packages are not installed
  not_installed <- function(pkgs) {
    setdiff(pkgs, rownames(installed.packages()))
  }
  # Ensure BiocManager is installed
  if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
  }
  # Install Bioconductor packages
  bioc_pkgs <- c("clusterProfiler", "org.Hs.eg.db", "ReactomePA", "topGO", "edgeR", "DESeq2")
  to_install <- not_installed(bioc_pkgs)
  if (length(to_install) > 0) {
    BiocManager::install(to_install, update = FALSE, ask = FALSE)
    message(paste(to_install, collapse = ", "), " packages added...")
  }
  # Install CRAN helper packages
  cran_pkgs <- c("devtools", "remotes", "DT", "kableExtra")
  to_install_cran <- not_installed(cran_pkgs)
  if (length(to_install_cran) > 0) {
    install.packages(to_install_cran)
  }
  if (!requireNamespace("sf", quietly = TRUE)) {
    os_type <- Sys.info()[["sysname"]]
    message("Installing 'sf' package...")

    if (os_type == "Darwin") {
      # macOS
      install.packages("sf", type = "binary")
    } else if (os_type == "Linux" || os_type == "Windows") {
      # Linux or Windows
      install.packages("sf", type = "source")
    } else {
      warning("Unknown OS. Please install 'sf' manually.")
    }
  }
  # Install GitHub packages if missing
  github_pkgs <- list(
    PCSF = "IOR-Bioinformatics/PCSF",
    multienrichjam = "jmw86069/multienrichjam"
  )
  for (pkg in names(github_pkgs)) {
    if (pkg %in% not_installed(pkg)) {
      remotes::install_github(github_pkgs[[pkg]], upgrade = "never")
    }
  }
  # Message if nothing was added
  if (length(to_install) == 0 && length(to_install_cran) == 0 &&
      all(!names(github_pkgs) %in% not_installed(names(github_pkgs)))) {
    message("No new packages added...")
  }
}

install_XYpackages()
```
Install from GitLab:

    devtools::install_url("https://gitlab.com/uniluxembourg/lcsb/bds/xyomics/-/raw/main/XYomics_0.1.1.tar.gz")

Install from Local Download:
1. Download the package file: [XYomics_0.1.1.tar.gz](https://gitlab.com/uniluxembourg/lcsb/bds/xyomics/-/raw/main/XYomics_0.1.1.tar.gz)
2. Install using R:
   
   install.packages("path/to/XYomics_0.1.1.tar.gz", repos = NULL, type = "source")

The package installation is tested on the following operating systems 
- Windows 11
- MAC OS 
- Ubuntu 20.4

params <-
list(report_date = structure(20584, class = "Date"))

## ----simulate-data, message=FALSE, warning=FALSE------------------------------
library(XYomics)
library(dplyr)

# Load precomputed bulk dataset
expression_data <- readRDS(
  system.file("extdata", "bulk_expression.rds", package = "XYomics")
)

sex <- readRDS(
  system.file("extdata", "bulk_sex.rds", package = "XYomics")
)

phenotype <- readRDS(
  system.file("extdata", "bulk_phenotype.rds", package = "XYomics")
)

## ----stratified-analysis, eval=FALSE------------------------------------------
# res <- sex_stratified_analysis_bulk(expression_data, sex, phenotype)
# # The `min_samples` parameter defines the minimum number of cells required
# # to perform the interaction analysis. The default value is 3.
# 
# 

## ----load-stratified-results--------------------------------------------------
res <- readRDS(
  system.file("extdata", "bulk_results_degs.rds", package = "XYomics")
)
# Internal QC results from XYomics can be accessed through:
# res$validation
#
# This includes validation of sex and phenotype balance, group sample sizes,
# detection of design imbalances, and identification of groups below the
# minimum sample threshold required for reliable downstream analysis.
res$validation

## ----sex-specific-analysis, eval=FALSE----------------------------------------
# res_cat <- categorize_sex(res$male_DEGs, res$female_DEGs)

## ----load-categorized-results-------------------------------------------------
res_cat <- readRDS(
  system.file("extdata", "bulk_results_cat.rds", package = "XYomics")
)

cat("Top sex-stratified differentially expressed genes:\n")
head(res_cat)

table(res_cat$DEG_Type)

## ----interaction-term-analysis-bulk, eval=FALSE-------------------------------
# interaction_term_results_bulk <- sex_interaction_analysis_bulk(expression_data, phenotype, sex)
# # The `min_samples` parameter defines the minimum number of cells required
# # to perform the interaction analysis. The default value ofr interaction analysis is 20.
# 

## ----load-interaction-results-------------------------------------------------
interaction_term_results_bulk <- readRDS(
  system.file("extdata", "bulk_interaction_results.rds", package = "XYomics")
)

head(interaction_term_results_bulk$summary_stats)# signficant sex-modulated DEGs: interaction_term_results_bulk$sig_results

## ----visualization------------------------------------------------------------
top_male_gene <- res_cat %>%
  filter(DEG_Type == "male-specific") %>%
  arrange(Male_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)

top_female_gene <- res_cat %>%
  filter(DEG_Type == "female-specific") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)
top_dimorphic_gene <- res_cat %>%
  filter(DEG_Type == "sex-dimorphic") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)
top_neutral_gene <- res_cat %>%
  filter(DEG_Type == "sex-neutral") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)

top_genes = c(top_male_gene, top_female_gene, top_dimorphic_gene, top_neutral_gene )

generate_violinplot_bulk(expression_data,sex,phenotype , top_genes)

## ----pathway-analysis, eval=FALSE---------------------------------------------
# pathway_results <- categorized_enrich(res_cat, enrichment_db = "GO")
# 
# # The `enrichment_db` parameter can be set to "GO", "KEGG", "REACTOME".
# # If the user wants to use a custom database, a TERM2GENE data frame must be
# # provided via the `custom_db` parameter.
# #
# # Gene identifiers can be either any Gene identifier type supported by OrgDb
# #' (e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT")
# # using the `gene_type` parameter.
# #
# # The `return_df` parameter controls the output format:
# # if set to TRUE, results are returned as data frames;
# # by default (FALSE), enrichResult objects are returned

## ----load-pathway-results-----------------------------------------------------
pathway_results <- readRDS(
  system.file("extdata", "bulk_pathway_results.rds", package = "XYomics")
)

head(pathway_results)

## ----celltype1_plot , results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE----
# visualize via dotplot
plot <- plot_enrichment_dotplots(pathway_results)
print(plot)

## ----get-string-network, message=FALSE----------------------------------------
# Fetch STRING network (can be replaced with a custom network)
# g <- get_string_network(organism = "9606", score_threshold = 900)

# Load a pre-existing network from a file
g <- readRDS(system.file("extdata", "string_example_network.rds", package = "XYomics")) # You can also download the hormonal network "hormonal_network_metacore.rds" instead of "string_example_network.rds".

## ----construct-pcsf-network, eval=FALSE---------------------------------------
# # Use dimorphic DE results to define prizes
# dimorphic_specific <- res_cat[res_cat$DEG_Type == "sex-dimorphic", ]
# dimorphic_prizes <- -log10((dimorphic_specific$Male_FDR + dimorphic_specific$Female_FDR) / 2)
# names(dimorphic_prizes) <-dimorphic_specific$Gene_Symbols
# 
# # Construct the PCSF subnetwork
# 
# dimorphic_network <- construct_ppi_pcsf(g = g, prizes = dimorphic_prizes)
# network <- ppi_pipeline(res_cat, g)

## ----load-network-results-----------------------------------------------------
network <- readRDS(
  system.file("extdata", "bulk_network_results.rds", package = "XYomics")
)

neutral_network <- readRDS(
  system.file("extdata", "bulk_neutral_network.rds", package = "XYomics")
)

neutral_specific <- res_cat[res_cat$DEG_Type == "sex-neutral", ]

## ----visualize-network--------------------------------------------------------


plot_network(neutral_network, "sex-neutral", neutral_specific, show_barplot = T)


#Generate plots for all categories

all_plots <- plot_network_pipeline(network, res_cat)


## ----report, eval=FALSE-------------------------------------------------------
# # This command generates a comprehensive HTML report
# 
# template_path = system.file("extdata", "Template_report_bulk.Rmd", package = "XYomics") # fetch the template for bulk from the package
# 
# generate_cat_report(res_cat, pathway_results, network, template_path = template_path)
# 
# 

params <-
list(report_date = structure(20584, class = "Date"))

## ----simulate-data, message=FALSE, warning=FALSE------------------------------
library(Seurat)
library(XYomics)
library(dplyr)

sim.seurat <- readRDS(
  system.file("extdata", "sc_toy_seurat.rds", package = "XYomics")
)

## ----preprocessing, message=FALSE, warning=FALSE, results='hide'--------------
# Precomputed during data generation

# sim.seurat <- NormalizeData(sim.seurat)
# sim.seurat <- FindVariableFeatures(sim.seurat)
# sim.seurat <- ScaleData(sim.seurat)
# sim.seurat <- RunPCA(sim.seurat)
# sim.seurat <- FindNeighbors(sim.seurat)
# sim.seurat <- FindClusters(sim.seurat)
# sim.seurat <- RunUMAP(sim.seurat)
DimPlot(sim.seurat)

## ----annotate-data, warning=FALSE---------------------------------------------
table(sim.seurat$cell_type)
table(sim.seurat$sex)
table(sim.seurat$status)


## ----sex-stratified-analysis, eval=FALSE--------------------------------------
# # Run for all cell types
# # if you want to have the female and male degs
# 
# results_degs <- sex_stratified_analysis_sc(sim.seurat)
# # The differential expression method can be specified using the `test.use` parameter.
# # Available options include "MAST", "bimod", "poisson", "LR", and any other
# # method supported by Seurat::FindMarkers().
# #
# # The `min_samples` parameter defines the minimum number of cells required
# # to perform the analysis. The default value is 3.
# 
# #
# # Internal QC results generated by XYomics can be accessed via:
# # results_degs$validation
# #
# # This QC output includes assessment of sex and phenotype balance, per-group cell
# # counts, detection of under-represented groups below the minimum cell threshold,
# # and overall design imbalance warnings, as well as dataset sparsity metrics.
# 
# # if you want the categorized DEGs by cell type more lenient than the default thesholds as it is simulated data
# 
# results <- sex_stratified_pipeline_sc(sim.seurat, min_abs_logfc = 0.1,  alpha = 0.1)
# # By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# # You can also run on one or multiple cell types, for example:
# # target_cell_type_flag = "cell type 1", "cell type 2"
# 

## ----load-stratified-results--------------------------------------------------
results_degs <- readRDS(
  system.file("extdata", "sc_results_degs_small.rds", package = "XYomics")
)

results <- readRDS(
  system.file("extdata", "sc_results_small.rds", package = "XYomics")
)

head(results$`cell type 1`)

## ----sex-interaction-analysis, eval=FALSE-------------------------------------
# # Example for one cell type (e.g., "cell type 1")
# 
# interaction_results <- sex_interaction_pipeline_sc(sim.seurat)
# # By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# # You can also specify one or multiple cell types, for example:
# # target_cell_type_flag = c("cell type 1", "cell type 2")
# #
# # The `min_samples` parameter defines the minimum number of cells required
# # to perform the interaction analysis. The default value is 20.

## ----load-interaction-results-------------------------------------------------
interaction_results <- readRDS(
  system.file("extdata", "sc_interaction_results_small.rds", package = "XYomics")
)

head(interaction_results$`cell type 1`$summary_stats)# signficant sex-modulated DEGs: interaction_results$`cell type 1`$sig_results 

## ----dot-plots, message=FALSE, warning=FALSE, fig.width=10, fig.height=8------
# Get top gene from each category for 'cell type 1'
top_male_gene <- results$`cell type 1` %>% filter(DEG_Type == "male-specific") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)
top_female_gene <- results$`cell type 1`  %>% filter(DEG_Type == "female-specific") %>% top_n(1, -Female_FDR) %>% pull(Gene_Symbols)
top_dimorphic_gene <- results$`cell type 1`  %>% filter(DEG_Type == "sex-dimorphic") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)
top_neutral_gene <- results$`cell type 1`  %>% filter(DEG_Type == "sex-neutral") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)

top_genes <- c(top_male_gene, top_female_gene, top_dimorphic_gene, top_neutral_gene)

generate_dotplot_sc(sim.seurat, top_genes, target_cell_type_flag = "cell type 1")
# By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# You can also specify one or multiple cell types, for example:
# target_cell_type_flag = c("cell type 1", "cell type 2")

## ----pathway-analysis, eval=FALSE---------------------------------------------
# # Run for all cell types
# pathway_category <- lapply(
#   results,
#   function(cell) categorized_enrich(cell, enrichment_db = "GO")
# )
# # The `enrichment_db` parameter can be set to "GO", "KEGG", "REACTOME".
# # If the user wants to use a custom database, a TERM2GENE data frame must be
# # provided via the `custom_db` parameter.
# #
# # Gene identifiers can be either any Gene identifier type supported by OrgDb
# #' (e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT")
# # using the `gene_type` parameter.
# #
# # The `return_df` parameter controls the output format:
# # if set to TRUE, results are returned as data frames;
# # by default (FALSE), enrichResult objects are returned.
# # Run for one specific cell type
# # pathway_category_one <- categorized_enrich(results$`cell type 1`, return_df =T, enrichment_db = "GO")

## ----load-pathway-results-----------------------------------------------------
pathway_category <- readRDS(
  system.file("extdata", "sc_pathway_single_small.rds", package = "XYomics")
)

pathway_category_one <- readRDS(
  system.file("extdata", "sc_pathway_single_small_df.rds", package = "XYomics")
)

head(pathway_category_one$`female-specific`)

## ----celltype1_plot ,  results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE----
# visualize via dotplot
plot <- plot_enrichment_dotplots(pathway_category)
print(plot)

## ----build-network, message=FALSE, warning=FALSE------------------------------
# Fetch STRING network (can be replaced with a custom network)
# g <- get_string_network(organism = "9606", score_threshold = 900)

# Load a pre-existing network from a file
g <- readRDS(system.file("extdata", "string_example_network.rds", package = "XYomics")) # You can also download the hormonal network "hormonal_network_metacore.rds" instead of "string_example_network.rds".


## ----network-analysis, eval=FALSE---------------------------------------------
# 
# ### Run for all cell type
# network_results <- ppi_pipeline(results, g)
# 
# # By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# # You can also specify one or multiple cell types, for example:
# # target_cell_type_flag = c("cell type 1", "cell type 2")
# 
# # Example for one specific cell type
# 
# 
# network_results_one <- ppi_pipeline(results,g , target_cell_type_flag = "cell type 2")

## ----load-network-results-----------------------------------------------------
network_results <- readRDS(
  system.file("extdata", "sc_network_results_small.rds", package = "XYomics")
)

## ----visualize-network, warning=FALSE, message=FALSE--------------------------
female_network <- network_results$`cell type 2`$female_network

#Generate network visualization
plot_network(female_network, "female-specific", results$`cell type 2`, "Cell type 2", show_barplot = T) 

#Generate all_plots for all categories and all cell types

all_plots <- plot_network_pipeline(network_results, results)


## ----report, eval=FALSE-------------------------------------------------------
# # This command generates a comprehensive HTML report for single-cell dataset
# generate_cat_report(results, pathway_category, network_results)
# 
# 

#' Perform Sex-Phenotype Interaction Analysis for Bulk Data
#'
#' @description Identifies genes whose expression is modulated by the interaction
#' between sex and phenotype using a differential difference contrast 
#'
#' @param expression_data Numeric matrix of expression data (features x samples).
#' @param phenotype Character or factor vector of sample phenotypes.
#' @param sex Character or factor vector of sample sexes.
#' @param phenotype_labels_vector Character vector of phenotype levels (default c("WT","TG")).
#' @param sex_labels_vector Character vector of sex levels (default c("F","M")).
#' @param min_logfc Numeric. Minimum absolute log fold change for reporting significant genes (default 0.25).
#' @param alpha Numeric. FDR threshold for significance (default 0.05).
#' @param min_samples Integer. Minimum number of samples per group (default 20).
#'
#' @return A list with all DE results, filtered significant DEGs, and summary statistics.
#'
#' @import limma
#' @importFrom stats model.matrix
#' @export
sex_interaction_analysis_bulk <- function(expression_data,
                                          phenotype,
                                          sex,
                                          phenotype_labels_vector = c("WT","TG"),
                                          sex_labels_vector = c("F","M"),
                                          min_logfc = 0.25,
                                          alpha = 0.05,
                                          min_samples = 20) {
  
  # validation
  validation <- validate_input_bulk(
    expression_data = expression_data,
    sex = sex,
    phenotype = phenotype,
    sex_labels_vector = sex_labels_vector,
    phenotype_labels_vector = phenotype_labels_vector,
    min_samples = min_samples
  )
  
  # factors
  phenotype <- factor(phenotype, levels = phenotype_labels_vector)
  sex <- factor(sex, levels = sex_labels_vector)
  
  # combined group
  group <- factor(paste(phenotype, sex, sep = "_"))
  
  # design matrix (USED EVERYWHERE)
  design <- model.matrix(~ 0 + group)
  colnames(design) <- levels(group)
  
  # voom flag
  used_voom <- FALSE
  fit <- NULL
  
  # check if raw counts
  is_likely_counts <- mean(expression_data %% 1 == 0, na.rm = TRUE) > 0.99
  
  if (is_likely_counts) {
    
    warning(
      "Expression data appear to be raw counts. Applying limma-voom."
    )
    
    v <- limma::voom(expression_data, design, plot = FALSE)
    
    # IMPORTANT: keep full voom object (E + weights)
    fit <- limma::lmFit(v, design)
    used_voom <- TRUE
    
  } else {
    
    # already normalized/log-expression
    fit <- limma::lmFit(expression_data, design)
  }
  
  # contrasts
  group_wt_f <- paste0(phenotype_labels_vector[1], "_", sex_labels_vector[1])
  group_tg_f <- paste0(phenotype_labels_vector[2], "_", sex_labels_vector[1])
  group_wt_m <- paste0(phenotype_labels_vector[1], "_", sex_labels_vector[2])
  group_tg_m <- paste0(phenotype_labels_vector[2], "_", sex_labels_vector[2])
  
  contrast_formula <- paste0(
    "(", group_tg_m, " - ", group_wt_m, ") - (",
    group_tg_f, " - ", group_wt_f, ")"
  )
  
  contrast_matrix <- limma::makeContrasts(
    contrasts = contrast_formula,
    levels = design
  )
  
  # fit model
  fit2 <- limma::contrasts.fit(fit, contrast_matrix)
  fit2 <- limma::eBayes(fit2)
  
  # results
  all_res <- limma::topTable(fit2, number = Inf, sort.by = "P")
  all_res$gene <- rownames(all_res)
  
  # prevent NOTE in R CMD check
  adj.P.Val <- logFC <- NULL
  
  # significant genes
  sig_res <- subset(all_res, adj.P.Val < alpha & abs(logFC) > min_logfc)
  
  return(list(
    all_results = all_res,
    sig_results = sig_res,
    summary_stats = data.frame(
      n_total_genes = nrow(all_res),
      n_sig_genes = nrow(sig_res),
      voom_used = used_voom,
      stringsAsFactors = FALSE
    ),
    validation = validation
  ))
}

#' Generate Violin Plots for Bulk Expression Data
#'
#' Creates violin plots displaying the distribution of gene expression values
#' across Sex and Phenotype groups for a set of selected genes. Multiple genes
#' (up to 10) are visualized using faceting.
#'
#' @param expression_data A matrix or data.frame of bulk expression values
#'   with genes in rows and samples in columns.
#' @param sex A vector indicating the sex of each sample.
#' @param phenotype A vector indicating the phenotype or condition of each sample.
#' @param genes Character vector of genes to visualize (maximum 10 recommended).
#' @param sex_labels_vector Character vector specifying sex labels.
#' @param phenotype_labels_vector Character vector specifying phenotype labels.
#'
#' @return A `ggplot` object containing the violin plots.
#'
#' @import ggplot2
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr mutate
#'
#' @export

generate_violinplot_bulk <- function(expression_data, 
                                     sex, 
                                     phenotype,
                                     genes,
                                     sex_labels_vector = c("F", "M"),
                                     phenotype_labels_vector = c("WT", "TG")) {
  
  #  Validate genes
  if (length(genes) == 0) {
    stop("No genes provided")
  }
  
  genes_present <- genes[genes %in% rownames(expression_data)]
  
  if (length(genes_present) == 0) {
    stop("None of the provided genes found in expression_data")
  }
  
  if (length(genes_present) < length(genes)) {
    warning("Some genes were not found and will be ignored")
  }
  
  # Limit number of genes 
  if (length(genes_present) > 10) {
    warning("More than 10 genes provided - plotting first 10 only")
    genes_present <- genes_present[1:10]
  }
  
  #  Transpose expression (samples as rows) 
  expr_df <- as.data.frame(t(expression_data[genes_present, , drop = FALSE]))
  
  expr_df$sex <- factor(sex, levels = sex_labels_vector)
  expr_df$phenotype <- factor(phenotype, levels = phenotype_labels_vector)
  
  expr_df$group_plot <- paste(expr_df$sex, expr_df$phenotype, sep = "_")
  
  # Long format 
  expr_long <- tidyr::pivot_longer(
    expr_df,
    cols = all_of(genes_present),
    names_to = "Gene",
    values_to = "Expression"
  )
  
  # Plot
  group_plot <- Expression <- NULL
  p <- ggplot(expr_long, aes(x = group_plot, y = Expression, fill = sex)) +
    
    geom_violin(trim = FALSE, alpha = 0.6, color = NA) +
    

    geom_jitter(
     color = "black",
      width = 0.08,
      size = 1,
      alpha = 0.8,
      show.legend = FALSE
    ) +
    
    stat_summary(
      fun = mean,
      geom = "point",
      shape = 21,
      size = 2,
      fill = "red",
      color = "black"
    ) +
    
    facet_wrap(~ Gene, scales = "free_y") +
    labs(
      title = "Expression of Selected Genes Across Sex-Phenotype Groups",
      x = "Sex-Phenotype Groups",
      y = "Expression"
    )  +
    
    theme_classic() +
    
    theme(
      panel.background = element_blank(),  
      plot.background  = element_blank(),
      panel.grid       = element_blank(),   
      strip.background = element_blank(),  
      strip.text       = element_text(size = 11, face = "bold"),
      axis.text.x      = element_text(angle = 45, hjust = 1),
      legend.position  = "none"
    )
  
  return(p)
}


#' Generate dotplot of selected genes across groups
#'
#' Visualizes gene expression across Sex / Phenotype / Cell type combinations.
#'
#' @param seurat_obj A Seurat object
#' @param genes Character vector of genes to plot
#' @param sex Metadata column for sex
#' @param phenotype Metadata column for phenotype/status
#' @param celltype_col Metadata column for cell types
#' @param target_cell_type_flag Either "all" or a specific cell type
#'
#' @return A ggplot object (DotPlot)
#' 
#' @importFrom Seurat Idents "Idents<-"
#' @importFrom Seurat  DotPlot
#' @export
generate_dotplot_sc <- function(seurat_obj,
                             genes,
                             sex = "sex",
                             phenotype = "status",
                             celltype_col = "cell_type",
                             target_cell_type_flag = "all") {
  
  # Validate metadata columns
  required_cols <- c(sex, phenotype, celltype_col)
  
  if (!all(required_cols %in% colnames(seurat_obj@meta.data))) {
    stop("Required metadata columns not found in Seurat object")
  }
  # Validate genes
  if (length(genes) == 0) {
    stop("No genes provided")
  }
  
  genes_present <- genes[genes %in% rownames(seurat_obj)]
  
  if (length(genes_present) == 0) {
    stop("None of the provided genes found in Seurat object")
  }
  
  if (length(genes_present) < length(genes)) {
    warning("Some genes were not found and will be ignored")
  }
  
  # Optional cell type subset
  if (!("all" %in% target_cell_type_flag)) {
    
    missing_ct <- setdiff(
      target_cell_type_flag,
      unique(seurat_obj@meta.data[[celltype_col]])
    )
    
    if (length(missing_ct) > 0) {
      stop(paste(
        "Cell type(s) not found:",
        paste(missing_ct, collapse = ", ")
      ))
    }
    cell_meta <- seurat_obj@meta.data[
      seurat_obj@meta.data[[celltype_col]] %in% target_cell_type_flag, ,
      drop = FALSE
    ]
    
    seurat_obj <- subset(seurat_obj, cells = rownames(cell_meta))
  }
  
  # Create grouping variable
  seurat_obj$group_plot <- paste(
    seurat_obj@meta.data[[celltype_col]],
    seurat_obj@meta.data[[sex]],
    seurat_obj@meta.data[[phenotype]],
    sep = "_"
  )
  
  Idents(seurat_obj) <- "group_plot"
  # Generate dotplot
  p <- Seurat::DotPlot(
    seurat_obj,
    features = genes_present,
    cols = c("blue", "red"),
    assay = "RNA"
  ) +
    coord_flip() +
    labs(
      title =paste0("Dotplot of selected gene expression in ", target_cell_type_flag, " cells"),
      x = "Genes",
      y = "Groups"
    ) +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1)
    )
  
  return(p)
}



#' Create enrichment dotplots from enrichResult objects
#'
#' @description
#' Generates dotplots from enrichment results. Input must contain enrichResult
#' objects (clusterProfiler / ReactomePA). Invalid entries are skipped with messages.
#' Works for single enrichResult, flat list (DEG types), or nested list (cell type → DEG type).
#'
#' @param enrichment_results A list or nested list of enrichResult objects.
#' @param showCategory Number of categories to display (default = 10).
#' @param ncol Number of columns for arranged plots (default = 2).
#'
#' @return Named list of combined ggplots. For nested input, names correspond to cell types;
#' for flat input, a single `"all"` entry is returned.
#'
#' @importFrom clusterProfiler dotplot
#' @importFrom patchwork wrap_plots
#' @export
plot_enrichment_dotplots <- function(enrichment_results,
                                     showCategory = 10,
                                     ncol = 2) {
  
  if (is.null(enrichment_results) || length(enrichment_results) == 0) {
    message("No enrichment results provided.")
    return(NULL)
  }
  
  # Convert single enrichResult to list
  if (inherits(enrichment_results, "enrichResult")) {
    enrichment_results <- list(Result = enrichment_results)
  }
  
  create_dotplot <- function(res, label, cell_type="") {
    if (!inherits(res, "enrichResult")) {
      message("Skipping ", label, ": not an enrichResult object.")
      return(NULL)
    }
    
    df <- as.data.frame(res)
    if (nrow(df) == 0) {
      message("Skipping ", label, ": enrichResult is empty.")
      return(NULL)
    }
    
    suppressMessages(
      clusterProfiler::dotplot(res, showCategory = showCategory) +
        ggplot2::labs(title = paste("Top enriched pathways -", label, cell_type)) +
        ggplot2::scale_y_discrete(expand = ggplot2::expansion(mult = c(0.1, 0.1))) +
        ggplot2::theme(plot.margin = ggplot2::margin(10, 10, 15, 10)) +
        ggplot2::scale_size_continuous(labels = function(x) as.integer(round(x)))
    )
  }
  
  all_plots <- list()
  
  # Detect nested (cell type → DEG type) or flat
  is_nested <- any(sapply(enrichment_results, is.list))
  
  if (is_nested) {
    for (cell_type in names(enrichment_results)) {
      cell_res <- enrichment_results[[cell_type]]
      if (!is.list(cell_res)) next
      
      cell_plots <- lapply(names(cell_res), function(deg_type) {
        create_dotplot(cell_res[[deg_type]], deg_type, cell_type)
      })
      names(cell_plots) <- names(cell_res)
      
      # Remove NULLs
      cell_plots <- cell_plots[!sapply(cell_plots, is.null)]
      if (length(cell_plots) == 0) next
      
      # Combine plots side by side using patchwork
      all_plots[[cell_type]] <- patchwork::wrap_plots(cell_plots, ncol = ncol)
    }
    
  } else {
    # Flat list (bulk) or single cell
    plots <- lapply(names(enrichment_results), function(deg_type) {
      create_dotplot(enrichment_results[[deg_type]], deg_type)
    })
    plots <- plots[!sapply(plots, is.null)]
    
    if (length(plots) > 0) {
      all_plots[["all"]] <- patchwork::wrap_plots(plots, ncol = ncol)
    }
  }
  
  if (length(all_plots) == 0) {
    message("No valid enrichment plots generated.")
    return(NULL)
  }
  
  return(all_plots)
}


#' Generate volcano plots for categorized DEGs
#'
#' @description
#' Creates volcano plots for all DEG categories (male-specific, female-specific,
#' sex-dimorphic, sex-neutral). Accepts either a single data.frame (bulk) or a
#' list of data.frames per cell type.
#'
#' @param de_results A data.frame or named list of data.frames containing categorized DEGs.
#'                   Required columns: DEG_Type, Gene_Symbols, Male_avg_logFC, Male_FDR,
#'                   Female_avg_logFC, Female_FDR
#' @param top_n Number of top genes to highlight per sex (default = 5).
#' @param logfc_thresh Minimum absolute logFC to consider a gene for highlighting (default = 1.5).
#' @param ncol Number of columns for arranging plots (default = 2).
#'
#' @return Named list of combined volcano plots per cell type or "all" for bulk.
#' @importFrom ggplot2 ggplot geom_point scale_color_manual theme_minimal labs theme element_text
#' @importFrom ggrepel geom_text_repel
#' @importFrom dplyr filter mutate slice_min
#' @importFrom patchwork wrap_plots
#' @export
plot_volcano_deg <- function(de_results,
                             top_n = 5,
                             logfc_thresh = 1.5,
                             ncol = 2) {
  
  create_volcano_plot <- function(df, deg_type, x_col, y_col, title_suffix) {
    
    # prevent warning when checking package
    DEG_Type <- GeneName <- Gene_Symbols <- Male_avg_logFC <- Male_FDR <- Female_avg_logFC <- Female_FDR <- logFC <- Sex <- . <- Significant <- NULL 
    
    df_filtered <- df %>% dplyr::filter(DEG_Type == deg_type) %>%
      dplyr::mutate(GeneName = Gene_Symbols)  # ensure gene names column
    
    # Identify top significant genes per sex
    df_filtered <- df_filtered %>%
      dplyr::mutate(Significant = GeneName %in% c(
        df_filtered %>%
          dplyr::filter(Male_FDR < 0.05, abs(Male_avg_logFC) > logfc_thresh) %>%
          dplyr::slice_min(Male_FDR, n = top_n, with_ties = FALSE) %>%
          .$Gene_Symbols,
        df_filtered %>%
          dplyr::filter(Female_FDR < 0.05, abs(Female_avg_logFC) > logfc_thresh) %>%
          dplyr::slice_min(Female_FDR, n = top_n, with_ties = FALSE) %>%
          .$Gene_Symbols
      ))
    
    # Create mapped columns 
    plot_x <- plot_y <- NULL
    df_filtered$plot_x <- eval(parse(text = x_col), envir = df_filtered)
    
    df_filtered$plot_y <- eval(parse(text = y_col), envir = df_filtered)
    
    
    
    # Create volcano plot
    
    ggplot(df_filtered, aes(x = plot_x, y = plot_y, color = Significant)) +
      geom_point(alpha = 0.6) +
      scale_color_manual(values = c("grey70", "firebrick")) +
      theme_minimal() +
      labs(
        x = "log2 Fold Change",
        y = "-log10(p-value)",
        title = paste("Volcano Plot -", title_suffix, "-", deg_type)
      ) +
      theme(
        legend.position = "bottom",
        plot.title = element_text(size = 10, hjust = 0.5, margin = ggplot2::margin(b = 10))
      ) +
      ggrepel::geom_text_repel(
        data = df_filtered %>% dplyr::filter(Significant),
        aes(label = Gene_Symbols),
        size = 3,
        max.overlaps = 30
      )
  }
  
  all_plots <- list()
  
  # Detect nested input (list of cell types) or bulk (single data.frame)
  is_nested <- is.list(de_results) && !inherits(de_results, "data.frame")
  
  if (is_nested) {
    for (cell_type in names(de_results)) {
      df <- de_results[[cell_type]]
      
      plots <- list(
        create_volcano_plot(df, "female-specific", "Female_avg_logFC", "-log10(Female_FDR)", cell_type),
        create_volcano_plot(df, "male-specific", "Male_avg_logFC", "-log10(Male_FDR)", cell_type),
        create_volcano_plot(df, "sex-dimorphic", "Male_avg_logFC - Female_avg_logFC","-log10(pmin(Male_FDR, Female_FDR))", cell_type),
        create_volcano_plot(df, "sex-neutral","(Male_avg_logFC + Female_avg_logFC)/2", "-log10(pmin(Male_FDR, Female_FDR))", cell_type)
      )
      
      # Remove NULLs
      plots <- plots[!sapply(plots, is.null)]
      if (length(plots) == 0) next
      
      all_plots[[cell_type]] <- patchwork::wrap_plots(plots, ncol = ncol)
    }
    
  } else {
    # Bulk input
    plots <- list(
      create_volcano_plot(de_results, "female-specific", "Female_avg_logFC", "-log10(Female_FDR)", "all"),
      create_volcano_plot(de_results, "male-specific", "Male_avg_logFC", "-log10(Male_FDR)", "all"),
      create_volcano_plot(de_results, "sex-dimorphic", "Male_avg_logFC - Female_avg_logFC","-log10(pmin(Male_FDR, Female_FDR))", "all"),
      create_volcano_plot(de_results, "sex-neutral","(Male_avg_logFC + Female_avg_logFC)/2", "-log10(pmin(Male_FDR, Female_FDR))",  "all")
    )
    
    plots <- plots[!sapply(plots, is.null)]
    if (length(plots) > 0) {
      all_plots[["all"]] <- patchwork::wrap_plots(plots, ncol = ncol)
    }
  }
  
  if (length(all_plots) == 0) {
    message("No valid volcano plots generated.")
    return(NULL)
  }
  
  return(all_plots)
}

#' XYomics: Analysis of Sex Differences in Omics Data
#'
#' The **XYomics** package provides functions for performing differential expression analysis,
#' pathway enrichment, gene regulatory network analysis, and comprehensive report generation for omics data.
#'
#' This package is designed to integrate various omics analyses (e.g., functional omics and single-cell data)
#' with advanced visualization and reporting tools.
#'
#' @name XYomics-package
#' @keywords internal
#' @useDynLib XYomics, .registration = TRUE
#' @exportPattern "^[[:alpha:]]+"
"_PACKAGE"

#' Generate a Comprehensive Analysis Report
#'
#' This function creates an integrated report that combines key analysis outputs,
#' @param results_cat A data frame or list containing differential expression results.
#' @param enrichment_cat A list with enrichment objects (e.g., BP, MF, KEGG, and optionally GSEA results).
#' @param grn_cat An igraph object representing the gene regulatory network (e.g., from PCSF analysis).
#' @param output_file Character. The desired name (and optionally path) for the rendered report (default: "analysis_report.html").
#' @param output_dir Character. Output directory to save the report to. 
#' @param template_path Character. Path to the R Markdown template file. If \code{NULL}, the function uses the built-in template located in \code{inst/rmd/template_report.Rmd}.
#' @param quiet Logical. If \code{TRUE} (default), rendering will be quiet.
#'
#' @return A character string with the path to the rendered report.
#'
#'
#' @export
generate_cat_report <- function(results_cat,
                                enrichment_cat,
                                grn_cat,
                                output_file = "cat_analysis_report.html",
                                output_dir = tempdir(), 
                                template_path = NULL,
                                quiet = TRUE) {

  # Load the RDS files (lists of data frames)
  de_results <- results_cat
  enrichment_results <- enrichment_cat
  grn_object <-  grn_cat 
  

  output_file <- file.path(output_dir, output_file) 
  
  message("Generating report: ", output_file)

  if (is.null(template_path)) {
    template_path <- system.file("extdata", "Template_report.Rmd", package = "XYomics")
    # Fallback to dev path
    if (template_path == "") {
      template_path <- "Template_report.Rmd"
    }
  }
  # Create parameter list
  report_params <- list(
    de_results = de_results,
    enrichment_results = enrichment_results,
    grn_object = grn_object,
    report_date = Sys.Date()
  )
  
  # Render report
  rmarkdown::render(
    input = template_path,
    output_file = output_file,
    params = report_params,
    envir = new.env(parent = globalenv()),
    quiet = quiet
  )
  
  return(invisible(output_file))
}

#' Perform differential expression analysis within each sex
#'
#' This function identifies differentially expressed genes between phenotype groups
#' separately for each sex using limma (default), DESeq2, or edgeR.
#'
#' @param expression_data A numeric matrix of expression data with features in rows
#'   and samples in columns. For \code{method = "limma"}, this can be a normalized
#'   log-expression matrix or a raw count matrix. For \code{method = "deseq2"} or
#'   \code{method = "edger"}, this must be a raw (non-negative integer) count matrix.
#' @param sex A vector indicating sex for each sample.
#' @param phenotype A vector indicating phenotype labels for each sample.
#' @param sex_labels_vector A character vector of length two defining the labels for sex (e.g., c("F", "M")). The first element must correspond to "female" and the second to "male".
#' @param phenotype_labels_vector Character vector specifying phenotype levels. The
#'   first element is the reference, the second the comparison; contrast = 2 - 1
#'   (e.g. c("WT","TG")).
#' @param min_samples Integer. Minimum number of samples per group (default 3).
#' @param method Character. Differential expression method to use. One of
#'   \code{"limma"} (default), \code{"deseq2"}, or \code{"edger"}.
#'
#' @return A list with three elements:
#'   \describe{
#'     \item{male_DEGs}{A data.frame of DE results for males.}
#'     \item{female_DEGs}{A data.frame of DE results for females.}
#'     \item{validation}{Output from \code{validate_input_bulk()}.}
#'   }
#'   Regardless of the method, each DEG data.frame contains the standardized
#'   columns \code{avg_log2FC}, \code{p_val}, \code{p_val_adj}, and \code{gene},
#'   with gene names as rownames. Additional method-specific columns (e.g.
#'   \code{baseMean} for DESeq2, \code{logCPM} for edgeR, \code{AveExpr} for limma)
#'   are retained for reference.
#'
#' @details For each sex, the function subsets the data and fits the appropriate
#'   model:
#'   \itemize{
#'    \item \strong{limma}: Fits a linear model via \code{lmFit} + \code{eBayes}
#'       on the input matrix directly. Suitable for microarray, proteomics, or
#'       pre-normalized RNA-seq data. If raw RNA-seq counts are provided, the
#'       function automatically detects integer matrices and applies
#'       \code{limma-voom} internally with a warning. For full control, consider
#'       using \code{"edger"} or \code{"deseq2"} instead, or apply \code{voom}
#'       externally and pass the transformed data with
#'       \code{method = "limma"}.
#'     \item \strong{DESeq2}: Fits a negative binomial GLM via
#'       \code{DESeq2::DESeq()}. Requires raw integer counts. Performs internal
#'       normalization (median-of-ratios) and dispersion estimation.
#'     \item \strong{edgeR}: Fits a negative binomial GLM via
#'       \code{edgeR::glmQLFit()} + \code{edgeR::glmQLFTest()}. Requires raw
#'       integer counts. Performs TMM normalization internally.
#'   }
#'   Positive logFC values indicate higher expression in the test phenotype
#'   (second element of \code{phenotype_labels_vector}) compared to the reference
#'   (first element).
#'
#' @importFrom stats model.matrix
#' @importFrom limma lmFit contrasts.fit eBayes topTable makeContrasts
#'
#' @export
sex_stratified_analysis_bulk <- function(expression_data,
                                         sex,
                                         phenotype,
                                         sex_labels_vector = c("F", "M"),
                                         phenotype_labels_vector = c("WT", "TG"),
                                         min_samples = 3,
                                         method = c("limma", "deseq2", "edger")) {
  
  method <- match.arg(method)
  
  # Validate input
  validation <- validate_input_bulk(
    expression_data = expression_data,
    sex = sex,
    phenotype = phenotype,
    sex_labels_vector = sex_labels_vector,
    phenotype_labels_vector = phenotype_labels_vector,
    min_samples = min_samples
  )
  
  # Check count matrix requirement for DESeq2 and edgeR
  if (method %in% c("deseq2", "edger")) {
    if (any(expression_data < 0, na.rm = TRUE)) {
      stop(
        "method = '", method, "' requires non-negative count data. ",
        "The input matrix contains negative values. ",
        "Use method = 'limma' for log-transformed or normalized data."
      )
    }
    if (!all(expression_data == floor(expression_data), na.rm = TRUE)) {
      warning(
        "method = '", method, "' expects raw integer counts, but non-integer ",
        "values were detected. This usually means the data has already been ",
        "normalized (e.g. log-transformed, TPM, FPKM). Rounding to nearest ",
        "integer, but results may be unreliable. Consider using ",
        "method = 'limma' for pre-normalized data instead."
      )
      expression_data <- round(expression_data)
    }
    storage.mode(expression_data) <- "integer"
  }
  
  # Convert to factors
  sex <- factor(sex, levels = sex_labels_vector)
  phenotype <- factor(phenotype, levels = phenotype_labels_vector)
  
  male_results <- NULL
  female_results <- NULL
  
  for (sex_label in sex_labels_vector) {
    
    selected_samples <- which(as.character(sex) == sex_label)
    
    if (length(selected_samples) == 0) {
      message("No samples for sex: ", sex_label)
      next
    }
    
    sex_subset <- expression_data[, selected_samples, drop = FALSE]
    phenotype_subset <- phenotype[selected_samples]
    
    if (length(unique(phenotype_subset)) < 2) {
      message("Only one phenotype level found for sex: ", sex_label)
      next
    }
    
    # Dispatch to method-specific backend
    results <- switch(method,
                      limma  = .de_limma(sex_subset, phenotype_subset, phenotype_labels_vector),
                      deseq2 = .de_deseq2(sex_subset, phenotype_subset, phenotype_labels_vector),
                      edger  = .de_edger(sex_subset, phenotype_subset, phenotype_labels_vector)
    )
    

    
    # Assign to sex
    if (sex_label == sex_labels_vector[2]) {
      male_results <- results
    } else {
      female_results <- results
    }
  }
  
  return(list(
    male_DEGs = male_results,
    female_DEGs = female_results,
    validation = validation
  ))
}


# --- Internal DE backends ------------------------------------------------

#' limma backend for sex-stratified DE
#' @noRd
#' @keywords internal
.de_limma <- function(counts, phenotype, phenotype_labels_vector) {
  
  design <- model.matrix(~ 0 + phenotype)
  colnames(design) <- levels(phenotype)
  
  # --- Detect raw counts and apply voom if needed ---
  is_likely_counts <- mean(counts %% 1 == 0, na.rm = TRUE) > 0.99
  
  if (is_likely_counts) {
    warning("Expression data appear to be raw counts. Applying limma-voom.")
    v <- limma::voom(counts, design, plot = FALSE)
    fit <- limma::lmFit(v, design)
  } else {
    fit <- limma::lmFit(counts, design)
  }
  
  # --- Contrast setup ---
  contrast_formula <- paste0(
    phenotype_labels_vector[2], " - ", phenotype_labels_vector[1]
  )
  
  contrast_matrix <- limma::makeContrasts(
    contrasts = contrast_formula,
    levels = design
  )
  
  fit <- limma::contrasts.fit(fit, contrast_matrix)
  fit <- limma::eBayes(fit)
  
  res <- limma::topTable(fit, coef = 1, number = Inf)
  
  # --- Standardize column names ---
  res$avg_log2FC <- res$logFC
  res$p_val      <- res$P.Value
  res$p_val_adj  <- res$adj.P.Val
  res$gene       <- rownames(res)
  
  return(res)
}

#' DESeq2 backend for sex-stratified DE
#' 
#' @noRd
#' @keywords internal
.de_deseq2 <- function(counts, phenotype, phenotype_labels_vector) {
  
  if (!requireNamespace("DESeq2", quietly = TRUE)) {
    stop("Package 'DESeq2' is required for method = 'deseq2'. ",
         "Install it from Bioconductor: BiocManager::install('DESeq2')")
  }
  
  col_data <- data.frame(
    condition = phenotype,
    row.names = colnames(counts)
  )
  
  dds <- DESeq2::DESeqDataSetFromMatrix(
    countData = counts,
    colData   = col_data,
    design    = ~ condition
  )
  
  # Ensure the reference level is set correctly
  dds$condition <- stats::relevel(dds$condition, ref = phenotype_labels_vector[1])
  
  # Run DESeq2 pipeline with fallback for dispersion estimation failures.
  # Small or atypical datasets can cause the parametric dispersion curve fit
  # to fail. In that case we fall back to gene-wise estimates, which is the
  # approach recommended by the DESeq2 authors in the error message itself.
  dds <- tryCatch(
    suppressMessages(DESeq2::DESeq(dds)),
    error = function(e) {
      if (grepl("dispersion", e$message, ignore.case = TRUE)) {
        message(
          "DESeq2: parametric dispersion fit failed. ",
          "Falling back to gene-wise dispersion estimates. ",
          "This is expected for small or atypical datasets."
        )
        dds <- suppressMessages(DESeq2::estimateSizeFactors(dds))
        dds <- suppressMessages(DESeq2::estimateDispersionsGeneEst(dds))
        DESeq2::dispersions(dds) <- S4Vectors::mcols(dds)$dispGeneEst
        dds <- suppressMessages(DESeq2::nbinomWaldTest(dds))
        return(dds)
      } else {
        stop(e)
      }
    }
  )
  
  res <- DESeq2::results(
    dds,
    contrast = c("condition",
                 phenotype_labels_vector[2],
                 phenotype_labels_vector[1])
  )
  
  res <- as.data.frame(res)
  
  # Remove genes with NA p-values (e.g. low count outliers filtered by DESeq2)
  res <- res[!is.na(res$pvalue), ]
  
  # Standardize column names
  res$avg_log2FC <- res$log2FoldChange
  res$p_val      <- res$pvalue
  res$p_val_adj  <- res$padj
  # Genes with extreme Cook's distance get NA padj; fill with 1
  res$p_val_adj[is.na(res$p_val_adj)] <- 1
  res$gene       <- rownames(res)
  
  return(res)
}


#' edgeR backend for sex-stratified DE
#' @noRd
#' @keywords internal
.de_edger <- function(counts, phenotype, phenotype_labels_vector) {
  
  if (!requireNamespace("edgeR", quietly = TRUE)) {
    stop("Package 'edgeR' is required for method = 'edger'. ",
         "Install it from Bioconductor: BiocManager::install('edgeR')")
  }
  
  dge <- edgeR::DGEList(counts = counts, group = phenotype)
  
  # Filter lowly expressed genes
  keep <- edgeR::filterByExpr(dge, group = phenotype)
  dge <- dge[keep, , keep.lib.sizes = FALSE]
  
  # TMM normalization
  dge <- edgeR::calcNormFactors(dge, method = "TMM")
  
  # Design matrix
  design <- model.matrix(~ 0 + phenotype)
  colnames(design) <- levels(phenotype)
  
  # Estimate dispersions and fit quasi-likelihood model
  dge <- edgeR::estimateDisp(dge, design)
  fit <- edgeR::glmQLFit(dge, design)
  
  # Contrast: test vs reference
  contrast_vector <- rep(0, ncol(design))
  names(contrast_vector) <- colnames(design)
  contrast_vector[phenotype_labels_vector[2]] <-  1
  contrast_vector[phenotype_labels_vector[1]] <- -1
  
  qlf <- edgeR::glmQLFTest(fit, contrast = contrast_vector)
  
  res <- edgeR::topTags(qlf, n = Inf, sort.by = "PValue")$table
  
  # Standardize column names
  res$avg_log2FC <- res$logFC
  res$p_val      <- res$PValue
  res$p_val_adj  <- res$FDR
  res$gene       <- rownames(res)
  
  return(res)
}

#' Validate bulk input data for sex-stratified analysis
#'
#' @param expression_data Numeric matrix (features x samples)
#' @param sex Vector of sex labels
#' @param phenotype Vector of phenotype labels
#' @param sex_labels_vector Expected sex labels (length 2)
#' @param phenotype_labels_vector Expected phenotype labels (length 2)
#' @param min_samples Minimum samples per group
#'
#' @return A list with validation status, ratios, imbalance flag, and group counts
#' @export
validate_input_bulk <- function(expression_data,
                                sex,
                                phenotype,
                                sex_labels_vector = c("F", "M"),
                                phenotype_labels_vector = c("WT", "TG"),
                                min_samples = 3) {
  
  # -------------------------------
  # Basic checks
  # -------------------------------
  if (!is.matrix(expression_data)) {
    stop("expression_data must be a matrix.")
  }
  
  if (ncol(expression_data) != length(phenotype)) {
    stop("Number of columns in expression_data must match length of phenotype.")
  }
  
  if (length(sex) != length(phenotype)) {
    stop("Length of sex and phenotype must match.")
  }
  
  if (length(sex_labels_vector) != 2) {
    stop("sex_labels_vector must contain exactly two labels.")
  }
  
  if (length(phenotype_labels_vector) != 2) {
    stop("phenotype_labels_vector must contain exactly two labels.")
  }
  
  # -------------------------------
  # Factor conversion (no shadowing)
  # -------------------------------
  sex_vec <- factor(sex, levels = sex_labels_vector)
  phenotype_vec <- factor(phenotype, levels = phenotype_labels_vector)
  
  if (any(is.na(sex_vec))) {
    stop("Some sex values are not in sex_labels_vector.")
  }
  
  if (any(is.na(phenotype_vec))) {
    stop("Some phenotype values are not in phenotype_labels_vector.")
  }
  
  # -------------------------------
  # Sex ratio
  # -------------------------------
  sex_counts <- table(sex_vec)
  total_samples <- sum(sex_counts)
  sex_ratio <- sex_counts / total_samples
  
  ratio_diff <- if (length(sex_ratio) == 2) abs(diff(sex_ratio)) else 0
  is_imbalanced <- ratio_diff > 0.30
  
  # -------------------------------
  # Group counts (robust)
  # -------------------------------
  group <- interaction(phenotype_vec, sex_vec, drop = FALSE)
  group_counts <- table(group)
  
  low_sample_groups <- group_counts[group_counts < min_samples]
  
  # -------------------------------
  # Warnings
  # -------------------------------
  if (is_imbalanced) {
    warning(
      paste(
        "Sex imbalance detected (>30%). Ratios:",
        paste(names(sex_ratio), round(sex_ratio, 2), collapse = ", "),
        "Proceeding despite sex imbalance..."
      )
    )
  }
  
  if (length(low_sample_groups) > 0) {
    warning(
      paste(
        "Some groups have fewer than", min_samples, "samples:",
        paste(names(low_sample_groups), low_sample_groups, sep = ":", collapse = ", "),
        "Sex-analysis may be unreliable."
      )
    )
  }
  
  # -------------------------------
  # Return
  # -------------------------------
  return(list(
    valid = TRUE,
    sex_counts = sex_counts,
    sex_ratio = round(sex_ratio, 3),
    imbalance = is_imbalanced,
    ratio_difference = round(ratio_diff, 3),
    group_counts = group_counts,
    low_sample_groups = low_sample_groups
  ))
}

#' Perform Pathway Enrichment Analysis for Pre-Categorized DEGs
#'
#' @description
#' Performs pathway enrichment analysis for categorized DEGs using GO, KEGG, Reactome, or a custom TERM2GENE database.
#'
#' @param DEGs_category Data frame with columns 'DEG_Type' and 'Gene_Symbols'.
#' @param enrichment_db "KEGG", "GO", "REACTOME", or "CUSTOM" (default: "KEGG").
#' @param organism Organism code for KEGG (default: "hsa").
#' @param gene_type Gene identifier type supported by OrgDb
#' (e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT").
#' @param org_db OrgDb object for gene annotation (default: org.Hs.eg.db).
#' @param custom_db TERM2GENE data frame, required if enrichment_db = "CUSTOM".
#' @param pvalueCutoff Numeric p-value cutoff (default: 0.05).
#' @param qvalueCutoff Numeric q-value cutoff (default: 0.2).
#' @param return_df Logical, if TRUE returns data.frames, else enrichResult objects (default: FALSE).
#'
#' @return Named list of enrichment results per DEG_Type.
#' @details Gene IDs converted only when needed; KEGG/Reactome require ENTREZID; GO/CUSTOM use provided gene_type.
#' @importFrom dplyr %>%
#' @importFrom AnnotationDbi keytypes
#' @export

categorized_enrich <- function(
    DEGs_category,
    enrichment_db = "KEGG",
    organism = "hsa",
    gene_type = "SYMBOL",
    org_db = NULL,
    custom_db = NULL,
    pvalueCutoff = 0.05,
    qvalueCutoff = 0.2,
    return_df = FALSE
) {
  if (is.null(org_db)) {
    if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
      stop(
        "Please provide an OrgDb or install 'org.Hs.eg.db' via:\n",
        "BiocManager::install('org.Hs.eg.db')"
      )
    }
    org_db <- org.Hs.eg.db::org.Hs.eg.db
  }
  db <- toupper(enrichment_db)
  gene_type <- toupper(gene_type)
  
  # Input validation
  if (!all(c("DEG_Type", "Gene_Symbols") %in% colnames(DEGs_category))) {
    stop("DEGs_category must contain columns 'DEG_Type' and 'Gene_Symbols'.")
  }
  
  if (!gene_type %in% AnnotationDbi::keytypes(org_db)) {
    stop(
      paste(
        "Unsupported gene_type. Supported types include:",
        paste(AnnotationDbi::keytypes(org_db), collapse = ", ")
      )
    )
  }
  
  if (!is.null(custom_db)) {
    if (!is.data.frame(custom_db) ||
        !all(c("term", "gene") %in% colnames(custom_db))) {
      stop("custom_db must be a data frame with columns: term, gene.")
    }
  }
  
  # Standardize DEG_Type
  all_categories <- unique(DEGs_category$DEG_Type)
  
  results <- list()
  
  # ---- NEW: helper-based conversion ----
  
  # For KEGG / Reactome → convert once to ENTREZ
  if (db %in% c("KEGG", "REACTOME") && gene_type != "ENTREZID") {
    DEGs_category <- convert_gene_ids(
      DEGs_category,
      fromType = gene_type,
      toType = "ENTREZID",
      org_db = org_db,
      gene_col = "Gene_Symbols"
    )
    gene_type <- "ENTREZID"
  }
  
  # -------------------------------------
  
  for (type in all_categories) {
    
    genes <- DEGs_category[
      DEGs_category$DEG_Type == type,
      "Gene_Symbols"
    ] %>% na.omit() %>% unique()
    
    if (length(genes) == 0) next
    
    # GO
    if (db == "GO") {
      
      enrichment_result <- clusterProfiler::enrichGO(
        gene = genes,
        OrgDb = org_db,
        keyType = gene_type,
        ont = "BP",   # lighter than ALL
        pvalueCutoff = pvalueCutoff,
        qvalueCutoff = qvalueCutoff
      )
    }
    
    # KEGG
    else if (db == "KEGG") {
      
      enrichment_result <- clusterProfiler::enrichKEGG(
        gene = genes,
        organism = organism,
        pvalueCutoff = pvalueCutoff,
        qvalueCutoff = qvalueCutoff
      )
    }
    
    # Reactome
    else if (db == "REACTOME") {
      
      enrichment_result <- ReactomePA::enrichPathway(
        gene = genes,
        organism = "human",
        pvalueCutoff = pvalueCutoff,
        qvalueCutoff = qvalueCutoff
      )
    }
    
    # Custom
    else if (db == "CUSTOM") {
      
      if (is.null(custom_db)) {
        stop("custom_db must be provided when enrichment_db = 'CUSTOM'.")
      }
      
      enrichment_result <- clusterProfiler::enricher(
        gene = genes,
        TERM2GENE = custom_db,
        pvalueCutoff = pvalueCutoff,
        qvalueCutoff = qvalueCutoff
      )
    }
    
    else {
      stop("Unsupported enrichment_db.")
    }
    
    # Store
    results[[type]] <- if (return_df) {
      as.data.frame(enrichment_result)
    } else {
      enrichment_result
    }
  }
  
  return(results)
}


#' Convert gene identifiers in a DEG table
#'
#' @param df Data frame containing a gene column
#' @param fromType Original ID type (e.g. "UNIPROT", "ENSEMBL")
#' @param toType Target ID type (e.g. "SYMBOL", "ENTREZID")
#' @param org_db OrgDb object (default: org.Hs.eg.db)
#' @param gene_col Column name containing gene IDs (default: "Gene_Symbols")
#' @param remove_na Remove unmapped genes (default: TRUE)
#' @param remove_duplicates Remove duplicated mapped IDs (default: TRUE)
#'
#' @return Updated data frame with converted gene IDs
#' @export
convert_gene_ids <- function(
    df,
    fromType,
    toType = "SYMBOL",
    org_db = NULL,
    gene_col = "Gene_Symbols",
    remove_na = TRUE,
    remove_duplicates = TRUE
) {
  
  if (!gene_col %in% colnames(df)) {
    stop(paste("Column", gene_col, "not found in data frame"))
  }
  
  supported <- AnnotationDbi::keytypes(org_db)
  
  if (is.null(org_db)) {
    if (!requireNamespace("org.Hs.eg.db", quietly = TRUE)) {
      stop(
        "Please provide an OrgDb or install 'org.Hs.eg.db' via:\n",
        "BiocManager::install('org.Hs.eg.db')"
      )
    }
    org_db <- org.Hs.eg.db::org.Hs.eg.db
  }
  if (!fromType %in% supported) {
    stop(paste("fromType not supported. Choose from:", paste(supported, collapse = ", ")))
  }
  
  if (!toType %in% supported) {
    stop(paste("toType not supported. Choose from:", paste(supported, collapse = ", ")))
  }
  
  genes <- unique(df[[gene_col]])
  n_input <- length(genes)
  
  mapping <- clusterProfiler::bitr(
    genes,
    fromType = fromType,
    toType = toType,
    OrgDb = org_db
  )
  
  n_mapped <- length(unique(mapping[[fromType]]))
  unmapped <- setdiff(genes, mapping[[fromType]])
  
  message("Gene ID conversion summary:")
  message(" - Input genes: ", n_input)
  message(" - Mapped genes: ", n_mapped)
  message(" - Unmapped genes: ", length(unmapped))
  message(" - Mapping rate: ", round(n_mapped / n_input * 100, 2), "%")
  
  df <- merge(
    df,
    mapping,
    by.x = gene_col,
    by.y = fromType,
    all.x = TRUE
  )
  
  df[[gene_col]] <- df[[toType]]
  
  if (remove_na) {
    df <- df[!is.na(df[[gene_col]]), ]
  }
  
  if (remove_duplicates) {
    df <- df[!duplicated(df[[gene_col]]), ]
  }
  
  df[[toType]] <- NULL
  
  return(df)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Internal function \code{call_sr}
#'
#' This function is internally used to solve the PCST.
#'
#' @keywords internal
#'
#' @param from  A \code{CharacterVector} that corresponds to \code{head} nodes of the edges.
#' @param to A \code{CharacterVector} that corresponds the \code{tail} nodes of the edges.
#' @param cost A \code{NumericVector} which represents the edge weights.
#' @param node_names A \code{CharacterVector} demonstrates the names of the nodes.
#' @param node_prizes A \code{NumericVector} which corresponds to the node prizes.
#' @author Murodzhon Akhmedov
#'
call_sr <- function(from, to, cost, node_names, node_prizes) {
    .Call(`_XYomics_call_sr`, from, to, cost, node_names, node_prizes)
}

#' Perform sex-stratified differential expression analysis for single-cell Seurat object
#'
#' @param seurat_obj A Seurat object containing single-cell RNA-seq data.
#' @param sex Character. Name of the column in metadata indicating sex.
#' @param phenotype Character. Name of the column indicating phenotype/condition.
#' @param celltype_col Character. Name of the column indicating cell type.
#' @param sex_labels_vector A character vector of length two defining the labels for sex (e.g., c("F", "M")). The first element must correspond to "female" and the second to "male". 
#' @param phenotype_labels_vector Character vector of length 2 specifying the reference and test phenotypes (e.g., c("WT", "TG")).
#' @param test.use Statistical test to use in FindMarkers.
#' @param min_samples Integer. Minimum number of cells per group (default 3).
#' @return A list with two elements: `male_DEGs` and `female_DEGs`, each containing a list of DEGs per cell type.
#' 
#' @importFrom Seurat GetAssayData FindMarkers 
#' @importFrom SeuratObject DefaultAssay DefaultAssay<- Idents Idents<-
#' @export
sex_stratified_analysis_sc <- function(seurat_obj,
                                       sex = "sex",
                                       phenotype = "status",
                                       celltype_col = "cell_type",
                                       sex_labels_vector = c("F", "M"),
                                       phenotype_labels_vector = c("WT", "TG"),
                                       test.use = "wilcox",
                                       min_samples = 3){
  
  # Use SC validation function 
  validation <- validate_input_sc(
    seurat_obj = seurat_obj,
    sex = sex,
    phenotype = phenotype,
    celltype_col = celltype_col,
    sex_labels_vector = sex_labels_vector,
    phenotype_labels_vector = phenotype_labels_vector,
    min_cells = min_samples
  )
  
  #Prepare metadata factors
  metadata <- seurat_obj@meta.data
  metadata[[sex]] <- factor(metadata[[sex]], levels = sex_labels_vector)
  metadata[[phenotype]] <- factor(metadata[[phenotype]], levels = phenotype_labels_vector)
  metadata[[celltype_col]] <- as.factor(metadata[[celltype_col]])
  
  # Create combined group
  metadata$Group <- factor(paste(metadata[[phenotype]], metadata[[sex]], sep="_"))
  
  # Initialize result lists
  male_results <- list()
  female_results <- list()
  
  # Loop over cell types
  cell_types <- unique(metadata[[celltype_col]])
  for (cell in cell_types) {
    message("Processing cell type: ", cell)
    
    # Subset by cell type
    Idents(seurat_obj) <- celltype_col
    cell_subset <- subset(seurat_obj, idents = cell)

    for (sex_value in sex_labels_vector) {
      # Subset by sex
      Idents(cell_subset) <- sex
      sex_subset <- subset(cell_subset, idents = sex_value)
      
      # Skip if not enough samples per phenotype
      if (sum(sex_subset@meta.data[[phenotype]] == phenotype_labels_vector[1]) < min_samples ||
          sum(sex_subset@meta.data[[phenotype]] == phenotype_labels_vector[2]) < min_samples) {
        next
      }
      
      # Try-catch for FindMarkers
      tryCatch({
        markers <- FindMarkers(
          object = sex_subset,
          group.by = phenotype,
          ident.1 = phenotype_labels_vector[2],
          ident.2 = phenotype_labels_vector[1],
          only.pos = FALSE,
          logfc.threshold = 0,
          test.use = test.use
        )
        markers[[celltype_col]] <- cell
        markers[[sex]] <- sex_value
        markers$gene <- rownames(markers)
        markers$direction <- ifelse(markers$avg_log2FC < 0, "down", "up")
        
        # Store results
        if (sex_value == sex_labels_vector[2]) {
          male_results[[cell]] <- markers
        } else {
          female_results[[cell]] <- markers
        }
      }, error = function(e) {
        message("Error processing cell type ", cell, " and sex ", sex_value, ": ", e$message)
      })
    }
  }
  
  return(list(
    male_DEGs = male_results,
    female_DEGs = female_results,
    validation = validation 
  ))
}

#' Run sex stratified differential expression pipeline for single cell data
#'
#' @description Performs sex specific differential expression analysis per cell type
#' and categorizes genes into male specific, female specific, sex dimorphic,
#' and sex neutral groups.
#'
#' @param seurat_obj Seurat object containing single cell data.
#' @param target_cell_type_flag Character. Cell type to analyze. 
#' @param sex Character. Metadata column for sex.
#' @param phenotype Character. Metadata column for phenotype.
#' @param celltype_col Character. Metadata column for cell type.
#' @param sex_labels_vector Character vector specifying sex levels.
#' @param phenotype_labels_vector Character vector specifying phenotype levels.
#' @param test.use Character. Statistical test used in FindMarkers.
#' @param alpha Numeric. FDR threshold for significance.
#' @param beta Numeric. P value threshold for exclusion in opposite sex.
#' @param min_abs_logfc Numeric. Minimum absolute log2 fold change for categorization.
#' @param min_samples Integer. Minimum number of cells per group (default 3).
#'
#' @return A named list of categorized differential expression results per cell type.
#'
#' @export

sex_stratified_pipeline_sc <- function(seurat_obj,
                                       target_cell_type_flag = "all",
                                       sex = "sex",
                                       phenotype = "status",
                                       celltype_col = "cell_type",
                                       sex_labels_vector = c("F", "M"),
                                       phenotype_labels_vector = c("WT", "TG"),
                                       test.use = "wilcox", 
                                       alpha = 0.05,
                                       beta = 0.5,
                                       min_abs_logfc = 0.25,
                                       min_samples = 3) {
  
  # Validate metadata columns
  required_columns <- c(sex, phenotype, celltype_col)
  if (!all(required_columns %in% colnames(seurat_obj@meta.data))) {
    missing_cols <- required_columns[!required_columns %in% colnames(seurat_obj@meta.data)]
    stop(paste("Missing metadata columns:", paste(missing_cols, collapse = ", ")))
  }
  
  # Optional cell type subsetting
  if (!("all" %in% target_cell_type_flag)) {
    
    valid_celltypes <- unique(seurat_obj@meta.data[[celltype_col]])
    
    if (!all(target_cell_type_flag %in% valid_celltypes)) {
      missing_ct <- target_cell_type_flag[!target_cell_type_flag %in% valid_celltypes]
      stop(paste("Cell type(s) not found:", paste(missing_ct, collapse = ", ")))
    }
    
    # Subset metadata
    cell_meta <- seurat_obj@meta.data[
      seurat_obj@meta.data[[celltype_col]] %in% target_cell_type_flag, ,
      drop = FALSE
    ]
    
    # Subset Seurat object
    seurat_obj <- subset(seurat_obj, cells = rownames(cell_meta))
  }
  
  # Run stratified DE analysis
  results <- sex_stratified_analysis_sc(
    seurat_obj = seurat_obj,
    sex = sex,
    phenotype = phenotype,
    celltype_col = celltype_col,
    sex_labels_vector = sex_labels_vector,
    phenotype_labels_vector = phenotype_labels_vector,
    test.use = test.use,
    min_samples = min_samples
  )
  
  # Extract cell types AFTER subsetting
  cell_types <- unique(seurat_obj@meta.data[[celltype_col]])
  
  # Build DEG list per cell type
  sex_degs <- lapply(cell_types, function(cell) {
    list(
      male   = results$male_DEGs[[cell]],
      female = results$female_DEGs[[cell]]
    )
  })
  
  names(sex_degs) <- cell_types
  
  # Categorize DEGs
  result_categories <- lapply(sex_degs, function(degs) {
    
    if (is.null(degs$male) || is.null(degs$female)) {
      message("Skipping cell type due to missing DEG results")
      return(NULL)
    }
    
    categorize_sex(
      degs$male,
      degs$female,
      alpha = alpha,
      beta = beta,
      min_abs_logfc = min_abs_logfc
    )
  })
  
  # Remove NULL entries
  result_categories <- Filter(Negate(is.null), result_categories)
  
  return(result_categories)
}


#' Validate single-cell input data for sex-stratified analysis
#'
#' @param seurat_obj Seurat object
#' @param sex Column name for sex
#' @param phenotype Column name for phenotype
#' @param celltype_col Column name for cell type
#' @param sex_labels_vector Expected sex labels (length 2)
#' @param phenotype_labels_vector Expected phenotype labels (length 2)
#' @param min_cells Minimum number of cells per group
#'
#' @return A list with validation status, ratios, imbalance flag, and group counts
#' @export
validate_input_sc <- function(seurat_obj,
                              sex = "sex",
                              phenotype = "status",
                              celltype_col = "cell_type",
                              sex_labels_vector = c("F", "M"),
                              phenotype_labels_vector = c("WT", "TG"),
                              min_cells = 3) {
  
  meta <- seurat_obj@meta.data
  

  # Check required columns

  required_columns <- c(sex, phenotype, celltype_col)
  if (!all(required_columns %in% colnames(meta))) {
    missing_cols <- required_columns[!required_columns %in% colnames(meta)]
    stop(paste("Missing metadata columns:", paste(missing_cols, collapse = ", ")))
  }
  

  # Factor conversion 

  sex_vec <- factor(meta[[sex]], levels = sex_labels_vector)
  phenotype_vec <- factor(meta[[phenotype]], levels = phenotype_labels_vector)
  
  if (any(is.na(sex_vec))) {
    stop("Some sex values are not in sex_labels_vector.")
  }
  
  if (any(is.na(phenotype_vec))) {
    stop("Some phenotype values are not in phenotype_labels_vector.")
  }
  
  # Sex ratio

  sex_counts <- table(sex_vec)
  total_cells <- sum(sex_counts)
  sex_ratio <- sex_counts / total_cells
  
  ratio_diff <- if (length(sex_ratio) == 2) abs(diff(sex_ratio)) else 0
  is_imbalanced <- ratio_diff > 0.30
  
 
  # Group counts (robust to missing groups)

  # Ensures all phenotype-sex combinations exist (even with 0 cells)
  group <- interaction(phenotype_vec, sex_vec, drop = FALSE)
  group_counts <- table(group)
  
  low_cell_groups <- group_counts[group_counts < min_cells]
  
  # Sparsity calculation

  assay <- seurat_obj[["RNA"]]
  
  counts <- if (inherits(assay, "Assay5")) {
    # Seurat v5
    if ("counts" %in% names(assay@layers)) {
      assay@layers[["counts"]]
    } else {
      stop("No 'counts' layer found in Assay5 object.")
    }
  } else if (inherits(assay, "Assay")) {
    # Seurat v4
    assay@counts
  } else {
    stop("Unsupported assay type.")
  }
  
  if (inherits(counts, "dgCMatrix")) {
    zero_entries <- length(counts) - length(counts@x[counts@x != 0]) 
    total_entries <- prod(dim(counts)) 
    
  } else {
    
    zero_entries <- sum(counts == 0) 
    total_entries <- length(counts)
    
  }
  
  sparsity <- (zero_entries / total_entries) * 100

  # Warnings

  if (is_imbalanced) {
    warning(
      paste(
        "Sex imbalance detected (>30%). Ratios:",
        paste(names(sex_ratio), round(sex_ratio, 2), collapse = ", "),
        "Proceeding despite imbalance..."
      )
    )
  }
  
  if (length(low_cell_groups) > 0) {
    warning(
      paste(
        "Some groups have fewer than", min_cells, "cells:",
        paste(names(low_cell_groups), low_cell_groups, sep = ":", collapse = ", "),
        "Sex-stratified analysis may be unreliable."
      )
    )
  }
  
  # Return results
  return(list(
    valid = TRUE,
    sex_counts = sex_counts,
    sex_ratio = round(sex_ratio, 3),
    imbalance = is_imbalanced,
    ratio_difference = round(ratio_diff, 3),
    group_counts = group_counts,
    low_cell_groups = low_cell_groups,
    sparsity_percent = round(sparsity, 2)
  ))
}

#' Construct Protein-protein interaction Network using Prize-Collecting Steiner Forest
#'
#' Constructs a condition-specific gene regulatory network based on differential
#' expression results using the PCSF algorithm.
#'
#' @param g An igraph object representing the base network.
#' @param prizes A named numeric vector of gene scores (prizes). Names must match vertex names in g.
#' @param w Numeric. Edge cost scaling weight. Default is 2.
#' @param b Numeric. Balance between prizes and edge costs. Default is 1.
#' @param mu Numeric. Trade-off parameter for sparsity. Default is 5e-04.
#' @param seed Integer. Random seed. Default is 1.
#' @param min_nodes Integer. Minimum number of nodes in subnetwork. Default is 1.
#' @return An igraph object representing the extracted subnetwork. Returns NULL invisibly 
#'   if no prize genes are present, the subnetwork is too small, or the PCSF algorithm fails.
#' @importFrom igraph V E E<- V<- graph_from_data_frame simplify vcount ecount neighbors induced_subgraph edge_attr_names is_igraph
#'
#' @return An igraph object representing the extracted subnetwork. Returns NULL invisibly if no prize genes are present, the subnetwork is too small, or the PCSF algorithm fails
#'
#' @export
construct_ppi_pcsf <- function(g, prizes, w = 2, b = 1, mu = 5e-04, seed = 1, min_nodes = 1) {
  set.seed(seed)
  
  common_genes <- intersect(names(prizes), V(g)$name)
  if (length(common_genes) == 0) {
    warning("None of the prize genes are present in the network.")
    return(invisible())
  }
  
  prize_nodes <- which(V(g)$name %in% common_genes)
  neighbors <- unique(unlist(lapply(prize_nodes, function(x) neighbors(g, x, mode = "all"))))
  nodes_to_keep <- unique(c(prize_nodes, neighbors))
  
  sub_g <- induced_subgraph(g, nodes_to_keep)
  
  if (vcount(sub_g) < min_nodes) {
    warning("Subnetwork too small: ", vcount(sub_g), " node(s). Skipping PCSF.")
    return(invisible())
  }
  
  prizes_sub <- rep(0, vcount(sub_g))
  names(prizes_sub) <- V(sub_g)$name
  prizes_sub[common_genes] <- prizes[common_genes]
  
  if (!"weight" %in% edge_attr_names(sub_g)) {
    E(sub_g)$weight <- 1
  }
  
  result_network <- tryCatch({
      result <- PCSF(
        ppi = sub_g,
        terminals = prizes_sub,
        w = w,
        b = b,
        mu = mu
      )

   
    
    if (!is.null(result) && is_igraph(result)) {
      V(result)$prize <- prizes_sub[V(result)$name]
      return(result)
    } else {
      warning("PCSF did not return a valid network.")
      return(invisible())
    }
  }, error = function(e) {
    warning("PCSF failed: ", e$message)
    return(invisible())
  })
  
  return(result_network)
}


#' Download and Process STRING Protein-Protein Interaction Network
#'
#' Downloads and processes the STRING protein-protein interaction network,
#' converting it to a simplified igraph object. The function downloads the
#' network from STRING database, filters interactions by confidence score,
#' converts STRING IDs to ENTREZ IDs, and returns the largest connected
#' component as an undirected graph.
#'
#' @param organism Character string specifying the NCBI taxonomy identifier.
#'        Default is "9606" (Homo sapiens).
#' @param score_threshold Numeric value between 0 and 1000 specifying the minimum
#'        combined score threshold for including interactions. Default is 700.
#' @param use_default it will return the default network (9606 and score of 700)
#'
#' @return An igraph object representing the largest connected component of
#'         the filtered STRING network, with the following properties:
#'         \itemize{
#'           \item Undirected edges
#'           \item No self-loops
#'           \item No multiple edges
#'           \item Edge weights (1 - combined_score/1000)
#'           \item Vertex names as ENTREZ IDs
#'         }
#'
#' @details 
#' STRING combined scores range from 0 (low confidence) to 1000 (high confidence). 
#' Because PCSF treats edge weights as traversal costs to be minimized, 
#' scores are inverted via 1 - score/1000 so that higher-confidence interactions correspond to lower costs.
#' The function performs the following steps:
#' \enumerate{
#'   \item Downloads protein interactions from STRING database
#'   \item Filters interactions based on combined score
#'   \item Downloads and processes STRING ID to ENTREZ ID mappings
#'   \item Creates an igraph object with filtered interactions
#'   \item Removes self-loops and multiple edges
#'   \item Extracts the largest connected component
#' }
#'
#'
#' @importFrom data.table fread
#' @importFrom igraph V E E<- V<- graph_from_data_frame simplify vcount ecount neighbors induced_subgraph edge_attr_names is_igraph graph_from_data_frame components
#' @importFrom utils download.file
#' @importFrom stats na.omit
#'
#' @export
get_string_network <- function(
    organism = "9606",
    score_threshold = 700,
    use_default = TRUE
) {
  
  if (use_default) {
    return(readRDS(system.file(
      "extdata",
      "STRING_9606_700.rds",
      package = "XYomics"
    )))
  }
  
  temp_dir <- tempdir()
  
  tryCatch({
    
    # Download interaction file
    string_url <- paste0(
      "https://stringdb-static.org/download/protein.links.v11.5/",
      organism, ".protein.links.v11.5.txt.gz"
    )
    
    temp_links <- file.path(temp_dir, "string_links.txt.gz")
    download.file(string_url, temp_links, mode = "wb", method = "auto")
    
    interactions <- data.table::fread(temp_links)
    
    interactions <- interactions[
      interactions$combined_score >= score_threshold,
    ]

    # Download alias file
    alias_url <- paste0(
      "https://stringdb-static.org/download/protein.aliases.v11.5/",
      organism, ".protein.aliases.v11.5.txt.gz"
    )
    
    temp_alias <- file.path(temp_dir, "string_alias.txt.gz")
    download.file(alias_url, temp_alias, mode = "wb", method = "auto")
    
    string_to_entrez <- data.table::fread(
      temp_alias,
      header = TRUE,
      quote = "",
      sep = "\t"
    )
    
    colnames(string_to_entrez) <- c(
      "string_protein_id",
      "alias_name",
      "source"
    )
    
    # Keep only Entrez mappings
    entrez_sources <- "Ensembl_EntrezGene"
    
    string_to_entrez <- string_to_entrez[
      string_to_entrez$source %in% entrez_sources,
    ]
    
    # Build ID map
    id_map <- string_to_entrez$alias_name
    names(id_map) <- gsub(
      paste0(organism, "\\."),
      "",
      string_to_entrez$string_protein_id
    )
    
    protein1 <- gsub(paste0(organism, "\\."), "", interactions$protein1)
    protein2 <- gsub(paste0(organism, "\\."), "", interactions$protein2)
    
    edges <- data.frame(
      from = id_map[protein1],
      to = id_map[protein2],
      weight = 1 - interactions$combined_score / 1000,
      stringsAsFactors = FALSE
    )
    
    # Remove unmapped edges
    edges <- edges[stats::complete.cases(edges), ]
    
    # Build graph safely
    if (nrow(edges) < 1) {
      warning("No edges passed threshold or ID mapping failed.")
      return(NULL)
    }
    
    g <- igraph::graph_from_data_frame(edges, directed = FALSE)
    
    g <- igraph::simplify(
      g,
      remove.multiple = TRUE,
      remove.loops = TRUE,
      edge.attr.comb = "mean"
    )

    # Keep largest connected component
    components <- igraph::components(g)
    
    if (components$no > 1) {
      largest_component <- which.max(components$csize)
      g <- igraph::induced_subgraph(
        g,
        vids = V(g)[components$membership == largest_component]
      )
    }
    
    return(g)
    
  }, finally = {
    # cleanup
    unlink(file.path(temp_dir, "string_links.txt.gz"))
    unlink(file.path(temp_dir, "string_alias.txt.gz"))
  })
}



#' Plot a condition-specific protein–protein interaction network with DEG annotations
#'
#' Visualizes a gene or protein interaction network for a selected differential
#' expression category. Nodes are sized by degree centrality, and hub genes can be
#' optionally annotated with mini barplots showing sex-specific log fold changes.
#'
#' @param g An igraph object representing the interaction network.
#' @param DEG_type Character string specifying the DEG category to visualize.
#' @param result_categories A data.frame containing DEG results with at least
#'   the columns DEG_Type, Gene_Symbols, Male_avg_logFC, and Female_avg_logFC.
#' @param cell_type Optional character string used in the plot title.
#' @param top_hubs Integer specifying the number of hub genes to highlight.
#' @param show_barplot Logical indicating whether hub barplots are displayed.
#'
#' @return A ggplot object representing the network visualization.
#'
#' @details
#' Nodes represent genes or proteins and edges represent interactions. Node size
#' reflects degree centrality (number of connections). The top \code{top_hubs}
#' nodes by degree are labeled. When \code{show_barplot = TRUE}, mini barplots are
#' overlaid on hub nodes to display sex-specific log fold changes. The visualization
#' is restricted to the largest connected component of the network.
#'
#' @examples
#' # Minimal reproducible example (CRAN-safe)
#' library(igraph)
#'
#' # Create a small toy network
#' g <- make_ring(5)
#' V(g)$name <- paste0("Gene", 1:5)
#'
#' # Create minimal DEG table
#' result_categories <- data.frame(
#'   DEG_Type = rep("example", 5),
#'   Gene_Symbols = paste0("Gene", 1:5),
#'   Male_avg_logFC = runif(5, -1, 1),
#'   Female_avg_logFC = runif(5, -1, 1)
#' )
#'
#' # Run function
#' plot_network(g, DEG_type = "example", result_categories = result_categories)
#'
#' @importFrom igraph V degree components induced_subgraph vcount
#' @import ggraph
#' @import ggplot2
#' @import ggrepel
#' @import dplyr
#' @import tidyr
#' @import grid
#' @importFrom cowplot get_legend ggdraw draw_plot draw_grob
#' @export
plot_network <- function(g, DEG_type, result_categories, cell_type ="", top_hubs=20, show_barplot=FALSE) {
  
  # ---- Safety checks ----
  if (is.null(g)) {
    message("plot_network: Input graph is NULL.")
    return(NULL)
  }
  
  if (!igraph::is_igraph(g)) {
    stop("plot_network: g must be an igraph object.")
  }
  
  if (igraph::vcount(g) == 0) {
    message("plot_network: Graph contains no nodes.")
    return(NULL)
  }
  
  # Node size scaling
  min_size <- if (vcount(g) > 30) 1 else 3
  
  # Largest connected component
  comps <- igraph::components(g, mode = "weak")
  main_id <- which.max(comps$csize)
  g <- igraph::induced_subgraph(g, vids = V(g)[comps$membership == main_id])
  
  # Degree & top hubs
  degree_vals <- degree(g, mode = "all")
  V(g)$degree <- degree_vals
  top_n <- min(top_hubs, vcount(g))
  top_nodes <- names(sort(degree_vals, decreasing = TRUE))[1:top_n]
  
  deg_breaks <- sort(unique(degree_vals))
  
  if (length(deg_breaks) > 6) {
    deg_breaks <- pretty(deg_breaks, n = 5)
  }
  
  # Layout
  set.seed(123)
  layout <- create_layout(g, layout = "fr")
  layout$label <- if (vcount(g) < 20) layout$name else ifelse(layout$name %in% top_nodes, layout$name, "")
  
  # prevent warning when checking package
  x <- y <- label <- NULL 
  # Base network plot
  gplot_net <- ggraph(layout) +
    geom_edge_link(alpha = 0.5, color = "grey", show.legend = FALSE) +
    geom_node_point(aes(size = degree), fill = "lightblue", shape = 21, color = "black", stroke = 0.5, show.legend = TRUE) +
    scale_size(range = c(min_size, 10), breaks = deg_breaks, name = "Node Degree") +
    geom_label_repel(aes(x = x, y = y , label = label),
                     color = "black", size = 3, fill = "white",
                     box.padding = 0.5, point.padding = 0.5, max.overlaps = 1000) +
    theme_void() +
    labs(title = paste("Protein-protein interaction network", cell_type, DEG_type)) +
    theme(legend.position = "right", plot.title = element_text(hjust = 0.5, size = 14))
  
  # prevent warning when checking package
  DEG_Type <- Gene_Symbols <- Male_avg_logFC <- Female_avg_logFC <- logFC <- Sex <- NULL 
  if (show_barplot!=F){
    # Prepare DEG data
    df <- result_categories %>%
      filter(DEG_Type == DEG_type, Gene_Symbols %in% V(g)$name) %>%
      dplyr::select(Gene_Symbols, Male_avg_logFC, Female_avg_logFC) %>%
      pivot_longer(cols = ends_with("FC"), names_to = "Sex", values_to = "logFC")
    
    # Mini-barplots, no legend per barplot
    barplots <- lapply(V(g)$name, function(gene) {
      sub_df <- df[df$Gene_Symbols == gene, ]
      ggplot(sub_df, aes(Sex, logFC, fill = Sex)) +
        geom_col(show.legend = FALSE) +
        scale_fill_manual(values = c("Female_avg_logFC" = "salmon", "Male_avg_logFC" = "blue")) +
        geom_hline(yintercept = 0, color = "black", linewidth = 0.3) +
        theme_void() +
        theme(plot.margin = margin(0, 0, 0, 0),
              panel.background = element_rect(fill = "transparent", color = NA))
    })
    
    # Scale mini-plots
    n_nodes <- vcount(g)
    base_scale <- 0.015
    plot_scale <- base_scale * (n_nodes / 3)^2.0
    plot_scale <- min(plot_scale, 0.8)
    
 
    # Embed mini-barplots
    for (i in seq_len(nrow(layout))) {
      if (layout$name[i] %in% intersect(top_nodes, df$Gene_Symbols)) {
        gplot_net <- gplot_net + annotation_custom(
          grob = ggplotGrob(barplots[[i]]),
          xmin = layout$x[i] - plot_scale,
          xmax = layout$x[i] + plot_scale,
          ymin = layout$y[i] - plot_scale,
          ymax = layout$y[i] + plot_scale
        )
      }
    }
    
    # Global legend for barplot
    legend_df <- data.frame(Sex = c("Female_avg_logFC", "Male_avg_logFC"), logFC = c(1, 1))
    legend_plot <- ggplot(legend_df, aes(Sex, logFC, fill = Sex)) +
      geom_col() +
      scale_fill_manual(values = c("Female_avg_logFC" = "salmon", "Male_avg_logFC" = "blue"),
                        labels = c("Female", "Male"), name = "logFC") +
      theme_void() +
      theme(legend.position = "bottom",
            legend.title = element_text(size = 10),
            legend.text = element_text(size = 8))
    barplot_legend <- cowplot::get_legend(legend_plot)
    
    # Combine network + global legend
    final_plot <- cowplot::ggdraw() +
      cowplot::draw_plot(gplot_net) +
      cowplot::draw_grob(barplot_legend, x = 0.35, y = 0, width = 0.3, height = 0.05)
    
  } else{
    final_plot <- gplot_net
  }
 
  
  return(final_plot)
}



#' Get Top Hub Genes by Betweenness Centrality
#'
#' Computes normalized betweenness centrality for an igraph network and
#' returns the top N hub nodes ranked by centrality.
#'
#' @param g An `igraph` object representing the network.
#' @param top_n Integer. Number of top hub nodes to return. Default is 10.
#' @param directed Logical. Whether the network is directed. Default is FALSE.
#'
#' @return A named numeric vector of betweenness scores for the top hub nodes.
#'
#' @importFrom igraph betweenness is_igraph
#'
#' @export
get_top_hubs <- function(g, top_n = 10, directed = FALSE) {
  
  if (!igraph::is_igraph(g)) {
    stop("Input must be an igraph object")
  }
  
  if (top_n <= 0) {
    stop("top_n must be > 0")
  }
  
  bet_scores <- igraph::betweenness(g, directed = directed, normalized = T)
  sorted_bet <- sort(bet_scores, decreasing = TRUE)
  top_nodes <- utils::head(sorted_bet, n = top_n)
  
  return(top_nodes)
}


#' Run PPI PCSf pipeline for single cell DEG categories
#'
#' Builds protein protein interaction subnetworks for each cell type
#' based on DEG categories and FDR derived node prizes.
#'
#' @param result_categories A named list of data.frames containing DEG results per cell type.
#'   Each data.frame must include DEG_Type, Gene_Symbols, Male_FDR, and Female_FDR.
#' @param g An igraph object representing the global protein interaction network.
#' @param target_cell_type_flag Character. Specific cell type to analyze or "all".
#' @return A named list containing PCSf networks per cell type and DEG category.
#'
#' @importFrom igraph vcount
#'
#' @export
ppi_pipeline <- function(result_categories, g, target_cell_type_flag = "all") {
  
  # Internal safe log10 function to avoid Inf from zero FDRs
  safe_log10 <- function(x, min_val = 1e-300) {
    -log10(pmax(x, min_val))
  }
  if (inherits(result_categories, "data.frame")) {
    result_categories <- list(all = result_categories)
  }
  
  network_results <- list()
  
  if (!("all" %in% target_cell_type_flag)) {
    cells=target_cell_type_flag
  } else{
    cells=names(result_categories)
  }
  for (cell_type in cells) {
    
    cell_results <- result_categories[[cell_type]]
    
    male_specific   <- cell_results[cell_results$DEG_Type == "male-specific", ]
    female_specific <- cell_results[cell_results$DEG_Type == "female-specific", ]
    sex_dimorphic   <- cell_results[cell_results$DEG_Type == "sex-dimorphic", ]
    sex_neutral     <- cell_results[cell_results$DEG_Type == "sex-neutral", ]
    
    male_prizes <- safe_log10(male_specific$Male_FDR)
    names(male_prizes) <- male_specific$Gene_Symbols
    
    female_prizes <- safe_log10(female_specific$Female_FDR)
    names(female_prizes) <- female_specific$Gene_Symbols
    
    dimorphic_prizes <- safe_log10(pmin(sex_dimorphic$Male_FDR, sex_dimorphic$Female_FDR))
    names(dimorphic_prizes) <- sex_dimorphic$Gene_Symbols
    
    neutral_prizes   <- safe_log10(pmin(sex_neutral$Male_FDR, sex_neutral$Female_FDR))
    names(neutral_prizes) <- sex_neutral$Gene_Symbols
    
    male_network <- if (length(male_prizes) > 0)
      construct_ppi_pcsf(g, male_prizes) else NULL
    
    female_network <- if (length(female_prizes) > 0)
      construct_ppi_pcsf(g, female_prizes) else NULL
    
    dimorphic_network <- if (length(dimorphic_prizes) > 0)
      construct_ppi_pcsf(g, dimorphic_prizes) else NULL
    
    neutral_network <- if (length(neutral_prizes) > 0)
      construct_ppi_pcsf(g, neutral_prizes) else NULL
    
    network_results[[cell_type]] <- list(
      male_network      = male_network,
      female_network    = female_network,
      dimorphic_network = dimorphic_network,
      neutral_network   = neutral_network
    )
  }
  
  network_results <- lapply(network_results, function(cell) {
    Filter(Negate(is.null), cell)
  })
  
  return(network_results)
}





#' Generate protein-protein interaction network plots for single-cell DEGs
#'
#' @description
#' Accepts  a nested list of igraph networks (cell_type × DEG_type) object and produces annotated network visualizations.
#'
#' @param network_list A single igraph object or a nested list: cell_type → DEG_type → igraph
#' @param result_categories_list A data.frame or named list matching the networks, containing DEG results.
#' @param top_hubs Number of hub genes to annotate (default = 20)
#' @param ncol Number of columns when combining DEG type plots (default = 2)
#' @param show_barplot Logical indicating whether hub barplots are displayed.
#'
#' @return Named list of network plots per cell type (or `"all"` for single network)
#' @import patchwork
#' @export
plot_network_pipeline <- function(network_list, result_categories_list, top_hubs = 20, ncol = 2,  show_barplot = FALSE) {
  
  if (inherits(result_categories_list, "data.frame")) {
    result_categories_list<- list(all = result_categories_list)
  }
  
  if (inherits(network_list, "data.frame")) {
    network_list<- list(all = network_list)
  }
  
  
  all_plots <- list()
  
  for (cell_type in names(network_list)) {
    cell_networks <- network_list[[cell_type]]
    if (!is.list(cell_networks)) next
    
    cell_plots <- lapply(names(cell_networks), function(deg_type) {
      plot_network(
        g = cell_networks[[deg_type]],
        DEG_type = deg_type,
        result_categories = result_categories_list[[cell_type]],
        cell_type = cell_type,
        top_hubs = top_hubs,
        show_barplot = show_barplot  
      )
    })
    
    names(cell_plots) <- names(cell_networks)
    cell_plots <- cell_plots[!sapply(cell_plots, is.null)]
    
    if (length(cell_plots) > 0) {
      # Combine with patchwork
      all_plots[[cell_type]] <- patchwork::wrap_plots(cell_plots, ncol = ncol)
    }
  }
  
  return(all_plots)
}

#' Prize-collecting Steiner Forest (PCSF)
#'
#' \code{PCSF} returns a subnetwork obtained by solving the PCSF on the given interaction network.
#'
#' @param ppi An interaction network, an \pkg{igraph} object.
#' @param terminals  A list of terminal genes with prizes to be analyzed in the PCSF context.
#' A named \code{numeric} vector, where terminal genes are named same as in the interaction network
#' and numeric values correspond to the importance of the gene within the study.
#' @param w A \code{numeric} value for tuning the number of trees in the output. A default value is 2.
#' @param b A \code{numeric} value for tuning the node prizes. A default value is 1.
#' @param mu A \code{numeric} value for a hub penalization. A default value is 0.0005.
#' @param dummies A list of nodes that are to connected to the root of the tree. If missing the root will be connected to all terminals.
#' @return The final subnetwork obtained by the PCSF.
#' It return an \pkg{igraph} object with the node prize and edge cost attributes.
#' @importFrom igraph V E ends degree graph.data.frame delete_vertices
#' @importFrom Rcpp evalCpp
#' @export
#'
#' @details
#'
#' The PCSF is a well-know problem in graph theory.
#' Given an undirected graph \emph{G = (V, E)}, where the vertices are labeled with prizes
#' \eqn{p_{v}} and the edges are labeled with costs \eqn{c_{e} > 0}, the goal is to identify
#' a subnetwork \emph{G' = (V', E')} with a forest structure. The target is to minimize
#' the total edge costs in \emph{E'}, the total node prizes left out of \emph{V'}, and the
#' number of trees in \emph{G'}. This is equivalent to  minimization of the following
#' objective function:
#'
#' \deqn{F(G')= Minimize \sum_{ e \in E'} c_{e} + \beta*\sum_{v \not\in V'} p_v + \omega*k}
#' where, \emph{k} is the number of trees in the forest, and it is regulated by parameter \eqn{\omega}.
#' The parameter \eqn{\beta} is used to tune the prizes of nodes.
#'
#' This optimization problem nicely maps onto the problem of finding differentially
#' enriched subnetworks in the cell protein-protein interaction (PPI) network.
#' The vertices of interaction network correspond to genes or proteins, and edges
#' represent the interactions among them. We can assign prizes
#' to vertices based on measurements of differential expression, copy number, or
#' mutation, and costs to edges based on confidence scores for those intra-cellular
#' interactions from experimental observation, yielding a proper input to the PCSF
#' problem. Vertices that are assigned a prize are referred to \emph{terminal} nodes,
#' whereas the vertices which are not observed in patient data are not assigned a
#' prize and are called \emph{Steiner} nodes. After scoring the interactome, the
#' PCSF is used to detect a relevant subnetwork (forest), which corresponds to a
#' portion of the interactome, where many genes are highly correlated in terms of
#' their functions and may regulate the differentially active biological process
#' of interest. The PCSF aims to identify  neighborhoods in interaction networks
#' potentially belonging to the key dysregulated pathways of a disease.

#' In order to avoid a bias towards the hub nodes of PPI networks to appear in solution
#' of PCSF, we penalize the prizes of \emph{Steiner} nodes according to their degree
#' distribution in PPI, and it is regulated by parameter \eqn{\mu}:
#'
#'  \deqn{p'_{v} = p_{v} - \mu*degree(v)}
#'
#'  The parameter \eqn{\mu} also affects the total number of \emph{Steiner} nodes in the solution.
#'  Higher the value of \eqn{\mu} smaller the number of \emph{Steiners} in the subnetwork,
#'  and vice-versa. Based on our previous analysis the recommended range of \eqn{\mu}
#'  for biological networks is between 1e-4 and 5e-2, and users can choose the values
#'  resulting subnetworks with vertex sets that have desirable \emph{Steiner/terminal}
#'  node ratio and average \emph{Steiner/terminal} in-degree ratio
#'  in the template interaction network.
#'
#' @author Murodzhon Akhmedov
#'
#' @references
#' Akhmedov M., LeNail A., Bertoni F., Kwee I., Fraenkel E., and Montemanni R. (2017)
#' A Fast Prize-Collecting Steiner Forest Algorithm for Functional Analyses in Biological Networks.
#' \emph{Lecture Notes in Computer Science}, to appear.


PCSF <-
  function(ppi, terminals, w = 2, b = 1, mu = 0.0005, dummies){
    
    # Checking function arguments
    if (missing(ppi))
      stop("Need to specify an interaction network \"ppi\".")
    if (missing(terminals))
      stop("  Need to provide terminal nodes as a named numeric vector,
    where node names must be same as in the interaction network.")
    if(is.null(names(terminals)))
      stop("  The terminal nodes must be provided as a named numeric vector,
    where node names must be same as in the interaction network.")
    
    
    
    
    # Gather the terminal genes to be analyzed, and their scores
    terminal_names = names(terminals)
    terminal_values = as.numeric(terminals)
    
    # Incorporate the node prizes
    node_names = V(ppi)$name
    node_prz = vector(mode = "numeric", length = length(node_names))
    index = match(terminal_names, node_names)
    percent = signif((length(index) - sum(is.na(index)))/length(index)*100, 4)
    if (percent < 5)
      stop("  Less than 1% of your terminal nodes are matched in the interactome, check your terminals!")
    message(paste0("  ", percent, "% of your terminal nodes are included in the interactome\n"))
    terminal_names = terminal_names[!is.na(index)]
    terminal_values = terminal_values[!is.na(index)]
    index = index[!is.na(index)]
    node_prz[index] =  terminal_values
    
    if(missing(dummies)||is.null(dummies)||is.na(dummies))
      dummies = terminal_names #re-assign this to allow for input
    
    ## Prepare input file for MST-PCSF implementation in C++
    
    message("  Solving the PCSF...\n")
    
    # Calculate the hub penalization scores
    node_degrees = igraph::degree(ppi)
    hub_penalization = - mu*node_degrees
    
    # Update the node prizes
    node_prizes = b*node_prz
    index = which(node_prizes==0)
    node_prizes[index] = hub_penalization[index]
    
    
    # Construct the list of edges
    edges = ends(ppi,es = E(ppi))
    from = c(rep("DUMMY", length(dummies)), edges[,1])
    to = c(dummies, edges[,2])
    
    cost = c(rep(w, length(dummies)), E(ppi)$weight)
    
    #PCSF will faill if there are NAs in weights, this will check and fail gracefully
    if(any(is.na(E(ppi)$weight))){
      stop("PCSF requires all edge weights to be non-missing; found NA values in E(ppi)$weight.")
    }
    
    ## Feed the input into the PCSF algorithm
    output = call_sr(from,to,cost,node_names,node_prizes)
    
    # Check the size of output subnetwork and print a warning if it is 0
    if(length(output[[1]]) != 0){
      
      #names(output) = c("from", "to", "cost", "terminal_names", "terminal_prizes")
      
      # Contruct an igraph object from the MST-PCSF output
      e = data.frame(output[[1]], output[[2]], output[[3]])
      #e = e[which(e[,1]!="DUMMY"), ]
      Ee = e[which(e[,2]!="DUMMY"), ]
      names(e) = c("from", "to", "weight")
      
      
      # Differentiate the type of nodes
      type = rep("Steiner", length(output[[4]]))
      index = match(terminal_names, output[[4]])
      index = index[!is.na(index)]
      type[index] = "Terminal"
      
      v = data.frame(output[[4]], output[[5]], type)
      names(v) = c("terminals", "prize", "type")
      subnet = graph.data.frame(e,vertices=v,directed=F)
      E(subnet)$weight=as.numeric(output[[3]])
      subnet = delete_vertices(subnet, "DUMMY")
      subnet = delete_vertices(subnet, names(which(degree(subnet)==0)))
      
      
      class(subnet) <- c("PCSF", "igraph")
      
      return (subnet)
      
    } else{
      
      stop("  Subnetwork can not be identified for a given parameter set.
    Provide a compatible b or mu value with your terminal prize list...\n\n")
      
    }
    
  }

#' Perform Sex-Phenotype Interaction Analysis for Single-Cell Data
#'
#' @description Performs differential difference analysis for a given cell type
#' to identify genes modulated by sex-phenotype interactions using limma.
#'
#' @param seurat_obj A Seurat object.
#' @param target_cell_type Character. Cell type to analyze.
#' @param sex Character. Column name for sex (default "sex").
#' @param phenotype Character. Column name for phenotype (default "status").
#' @param celltype_col Character. Column name for cell type (default "cell_type").
#' @param min_logfc Numeric. Minimum absolute log fold change for reporting significant genes (default 0.25).
#' @param alpha Numeric. FDR threshold for significance (default 0.05).
#' @param sex_labels_vector Character vector of sex labels (default c("F","M")).
#' @param phenotype_labels_vector Character vector of phenotype groups (default c("WT","TG")).
#' @param min_samples Integer. Minimum number of cells per group (default 20).
#'
#' @return A list with all DE results, filtered significant DEGs, and summary statistics.
#'
#' @importFrom Seurat DefaultAssay 
#' @importFrom SeuratObject DefaultAssay DefaultAssay<- Idents Idents<-
#' @importFrom edgeR DGEList filterByExpr calcNormFactors
#' @importFrom limma voom lmFit makeContrasts contrasts.fit eBayes topTable
#' @importFrom stats model.matrix
#' @importFrom methods slotNames
#' @export
sex_interaction_analysis_sc <- function(seurat_obj,
                                        target_cell_type,
                                        sex = "sex",
                                        phenotype = "status",
                                        celltype_col = "cell_type",
                                        min_logfc = 0.25,
                                        alpha = 0.05,
                                        sex_labels_vector = c("F", "M"),
                                        phenotype_labels_vector = c("WT", "TG"),
                                        min_samples = 20) {
  
  # --- Check cell type existence ---
  if (!(target_cell_type %in% unique(seurat_obj@meta.data[[celltype_col]]))) {
    stop(paste0("Target cell type '", target_cell_type, "' not found in metadata"))
  }
  
  # --- Subset metadata ---
  cell_meta <- seurat_obj@meta.data[
    seurat_obj@meta.data[[celltype_col]] == target_cell_type, ,
    drop = FALSE
  ]
  
  # --- Subset Seurat object ---
  cell_subset <- subset(seurat_obj, cells = rownames(cell_meta))
  
  # --- Run validation on subset (FIX) ---
  validation <- validate_input_sc(
    seurat_obj = cell_subset,
    sex = sex,
    phenotype = phenotype,
    celltype_col = celltype_col,
    sex_labels_vector = sex_labels_vector,
    phenotype_labels_vector = phenotype_labels_vector,
    min_cells = min_samples
  )
  
  # Set factors 
  cell_meta[[sex]] <- factor(cell_meta[[sex]], levels = sex_labels_vector)
  cell_meta[[phenotype]] <- factor(cell_meta[[phenotype]], levels = phenotype_labels_vector)
  
  # Create combined group
  cell_meta$Group <- factor(paste(cell_meta[[phenotype]], cell_meta[[sex]], sep="_"))
  
  assay <- cell_subset[["RNA"]]
  
  counts <- if (inherits(assay, "Assay5")) {
    # Seurat v5
    if ("counts" %in% names(assay@layers)) {
      assay@layers[["counts"]]
    } else {
      stop("No 'counts' layer found in Assay5 object.")
    }
  } else if (inherits(assay, "Assay")) {
    # Seurat v4
    assay@counts
  } else {
    stop("Unsupported assay type.")
  }
  
  counts <- as.matrix(counts)
  
  # Design matrix
  design <- model.matrix(~ 0 + Group, data = cell_meta)
  colnames(design) <- gsub("^Group", "", colnames(design))
  
  # edgeR processing 
  dge <- DGEList(counts = counts)
  keep <- filterByExpr(dge, design)
  dge <- dge[keep, , keep.lib.sizes = FALSE]
  dge <- calcNormFactors(dge, method = "TMMwsp")
  
  # voom + limma 
  vm <- voom(dge, design, plot = FALSE)
  fit <- lmFit(vm, design)
  
  # Define interaction contrast
  group_wt_f <- paste0(phenotype_labels_vector[1], "_", sex_labels_vector[1])
  group_tg_f <- paste0(phenotype_labels_vector[2], "_", sex_labels_vector[1])
  group_wt_m <- paste0(phenotype_labels_vector[1], "_", sex_labels_vector[2])
  group_tg_m <- paste0(phenotype_labels_vector[2], "_", sex_labels_vector[2])
  
  contrast_formula <- paste0(
    "(", group_tg_m, " - ", group_wt_m, ") - (",
    group_tg_f, " - ", group_wt_f, ")"
  )
  
  contrast_matrix <- makeContrasts(
    contrasts = contrast_formula,
    levels = design
  )
  
  fit2 <- contrasts.fit(fit, contrast_matrix)
  fit2 <- eBayes(fit2)
  
  # Extract results 
  res <- topTable(fit2, number = Inf, adjust.method = "BH")
  res$cell_type <- target_cell_type
  res$gene <- rownames(res)
  
  # prevent warning when checking package
  adj.P.Val <- logFC <- NULL
  sig_res <- subset(res, adj.P.Val < alpha & abs(logFC) > min_logfc)
  
  return(list(
    all_results = res,
    sig_results = sig_res,
    summary_stats = data.frame(
      cell_type = target_cell_type,
      n_total_genes = nrow(res),
      n_sig_genes = nrow(sig_res),
      stringsAsFactors = FALSE
    ),
    validation = validation
  ))
}

#' Run sex phenotype interaction analysis pipeline for single cell data
#'
#' @description Performs sex phenotype interaction analysis per cell type
#' using a limma based difference in differences model.
#'
#' @param seurat_obj Seurat object containing single cell data.
#' @param target_cell_type_flag Character. Specific cell type to analyze or "all".
#' @param sex Character. Metadata column for sex.
#' @param phenotype Character. Metadata column for phenotype.
#' @param celltype_col Character. Metadata column for cell type.
#' @param min_logfc Numeric. Minimum absolute log fold change threshold.
#' @param alpha Numeric. FDR threshold for significance.
#' @param sex_labels_vector Character vector specifying sex levels.
#' @param phenotype_labels_vector Character vector specifying phenotype levels.
#' @param min_samples Integer. Minimum number of cells per group (default 20).
#' @return A named list containing interaction analysis results per cell type.
#'
#' @export

sex_interaction_pipeline_sc <- function(seurat_obj,
                                        target_cell_type_flag = "all",
                                        sex = "sex",
                                        phenotype = "status",
                                        celltype_col = "cell_type",
                                        min_logfc = 0.25,
                                        alpha = 0.05,
                                        sex_labels_vector = c("F", "M"),
                                        phenotype_labels_vector = c("WT", "TG"),
                                        min_samples = 20) {
  
  results <- list()
  

  # Determine cell types to analyze
  if ("all" %in% target_cell_type_flag) {
    
    cell_types <- unique(seurat_obj@meta.data[[celltype_col]])
    
  } else {
    
    available_ct <- unique(seurat_obj@meta.data[[celltype_col]])
    missing_ct <- setdiff(target_cell_type_flag, available_ct)
    
    if (length(missing_ct) > 0) {
      stop(paste(
        "Cell type(s) not found:",
        paste(missing_ct, collapse = ", ")
      ))
    }
    
    cell_types <- target_cell_type_flag
  }
  
  # Run analysis per cell type
  for (cell in cell_types) {
    
    message("Running analysis for cell type: ", cell)
    
    results[[cell]] <- sex_interaction_analysis_sc(
      seurat_obj = seurat_obj,
      target_cell_type = cell,
      sex = sex,
      phenotype = phenotype,
      celltype_col = celltype_col,
      min_logfc = min_logfc,
      alpha = alpha,
      sex_labels_vector = sex_labels_vector,
      phenotype_labels_vector = phenotype_labels_vector,
      min_samples = min_samples
    )
  }
  
  return(results)
}
  
  
  

#' Compute sex-specific differentially expressed genes (DEGs) per category
#' @description
#' Identifies male-specific, female-specific, sex-dimorphic, and sex-neutral DEGs from differential expression results. 
#' "sex-modulated" requires the interaction mode and is not identified by categorize_sex.
#'
#' @param male_degs Data frame containing male differential expression results from one specific cell-type or bulk dataset.
#' @param female_degs Data frame containing female differential expression results from one specific cell-type or bulk dataset.
#' @param alpha Numeric. FDR threshold for significance.
#' @param beta Numeric. P-value threshold for excluding genes in opposite sex.
#' @param min_abs_logfc Numeric. Minimum absolute log2 fold change threshold.
#'
#' @return Data frame containing categorized DEGs with associated statistics.
#'
#'
#' @export
#'

categorize_sex <- function(male_degs,
                           female_degs,
                           alpha = 0.05,
                           beta = 0.5,
                           min_abs_logfc = 0.25) {
  
  
  
  required_cols <- c("p_val_adj", "p_val", "avg_log2FC")
  
  missing_male <- setdiff(required_cols, colnames(male_degs))
  missing_female <- setdiff(required_cols, colnames(female_degs))
  
  if (length(missing_male) > 0) {
    stop(paste("Male DEG table is missing columns:",
               paste(missing_male, collapse = ", ")))
  }
  
  if (length(missing_female) > 0) {
    stop(paste("Female DEG table is missing columns:",
               paste(missing_female, collapse = ", ")))
  }
  
  
  # Helper function to extract logFC and FDR values
  get_deg_stats <- function(genes, deg_data) {
    matched_idx <- match(genes, rownames(deg_data))
    list(
      logFC = deg_data[matched_idx, "avg_log2FC"],
      fdr = deg_data[matched_idx, "p_val_adj"]
    )
  }
  
  # Identify male-specific genes
  male_candidates <- rownames(male_degs)[male_degs$p_val_adj < alpha]
  
  f_pval <- female_degs[match(male_candidates, rownames(female_degs)), "p_val"]
  
  male_specific <- male_candidates[is.na(f_pval) | f_pval > beta]
  
  male_specific <- male_specific[
    abs(male_degs[match(male_specific, rownames(male_degs)), "avg_log2FC"]) > min_abs_logfc
  ]
  
  
  # Identify female-specific genes
  female_candidates <- rownames(female_degs)[female_degs$p_val_adj < alpha]
  
  m_pval <- male_degs[match(female_candidates, rownames(male_degs)), "p_val"]
  
  female_specific <- female_candidates[is.na(m_pval) | m_pval > beta]
  
  female_specific <- female_specific[
    abs(female_degs[match(female_specific, rownames(female_degs)), "avg_log2FC"]) > min_abs_logfc
  ]
  
  # Identify shared and dimorphic genes
  dimorphic_genes <- NULL
  shared_genes <- NULL
  intersecting_genes <- intersect(male_candidates, female_candidates)
  
  if (length(intersecting_genes) > 0) {
    logfc_male <- male_degs[match(intersecting_genes, rownames(male_degs)),]$avg_log2FC
    logfc_female <- female_degs[match(intersecting_genes, rownames(female_degs)),]$avg_log2FC
    
    dimorphic_indices <- intersect(which(sign(logfc_male) != sign(logfc_female)),
                                   intersect(which(abs(logfc_male) > min_abs_logfc),
                                             which(abs(logfc_female) > min_abs_logfc)))
    dimorphic_genes <- intersecting_genes[dimorphic_indices]
    
    shared_indices <- intersect(which(sign(logfc_male) == sign(logfc_female)),
                                intersect(which(abs(logfc_male) > min_abs_logfc),
                                          which(abs(logfc_female) > min_abs_logfc)))
    shared_genes <- intersecting_genes[shared_indices]
  }
  
  # Compile results into a data frame
  all_categories <- list(
    "male-specific" = male_specific,
    "female-specific" = female_specific,
    "sex-dimorphic" = dimorphic_genes,
    "sex-neutral" = shared_genes
  )
  
  results <- data.frame(
    DEG_Type = rep(names(all_categories), sapply(all_categories, length)),
    Gene_Symbols = unlist(all_categories),
    stringsAsFactors = FALSE
  )
  
  if (nrow(results) == 0) {
    return(results)
  }
  
  # Add male statistics
  male_stats <- get_deg_stats(results$Gene_Symbols, male_degs)
  results$Male_avg_logFC <- male_stats$logFC
  results$Male_FDR <- male_stats$fdr
  
  # Add female statistics
  female_stats <- get_deg_stats(results$Gene_Symbols, female_degs)
  results$Female_avg_logFC <- female_stats$logFC
  results$Female_FDR <- female_stats$fdr
  
  return(results)
}

---
title: "Global Report Bulk RNA-Seq"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Template report bulk rna-seq}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
library(ggplot2)
library(DT)
library(dplyr)
library(igraph)
library(ggraph)
library(knitr)
library(kableExtra)
library(ggrepel)
library(gridExtra)
library(htmltools)

# fallback operator
`%||%` <- function(x, y) if (!is.null(x)) x else y
```

#  Overview

This is a global report built for **XYomics output on Bulk-RNA-Seq data**.

**Date:** `r params$report_date`\
**Includes:** - Differential expression analysis (DEGs) - Pathway enrichment analysis - Protein-protein interaction networks analysis

------------------------------------------------------------------------

#  Differential Expression Results of categorized DEGs 

```{r de-results, echo=F}

get_name <- function(df){

  names <- unique(df$gene_type)
  return(names)
}


render_de_table <-  function(df) {
    if (nrow(df) == 0) {
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p("No significant DEGs for this category.")
    ))
  }
  else
  {   name <- get_name(df) 
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p( datatable(df%>% mutate_if(is.numeric, ~format(., scientific=T, digits=4)),
            options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )

  }

 
  
  
}

tagList(
  lapply(params$de_results, render_de_table)
)
```

------------------------------------------------------------------------

# Enrichment / Pathway Analysis

```{r enrichment-tables, echo=FALSE}

render_enrichment_table <- function(df, name) {
  if (is.null(df)) {
    return(tags$div(
      tags$h4(paste("Pathway Enrichment -", name)),
      tags$p("No significant pathways for this category.")
    ))
  }
  else
  {  
    return(tags$div(
    tags$h4(paste("Pathway Enrichment -", name)),
    tags$p(datatable(
      df %>% mutate_if(is.numeric, ~format(., scientific=T, digits=4)),
      options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )
    
  }
  
  
}



tagList(lapply(seq_along(params$enrichment_results), function(i) {
  name <- names(params$enrichment_results)[i] %||% paste("Set", i)
  df <- params$enrichment_results[[i]]
  render_enrichment_table(df, name)
}))
```

## Dotplot of the pathways by cell types
```{r pathways visualization , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=7}
create_dotplot <- function(df, plot_name) {
  if (nrow(df) > 0) {
    enr <- convertdf2enr(df)
    return(dotplot(enr, showCategory = 10, title = paste("Top enriched Pathways -", plot_name)))
  } else {
    return(NULL)
  }
}

all_plots <- list()

for (i in seq_along(params$enrichment_results)) {
  df <- params$enrichment_results[[i]]
  plot_name <- names(params$enrichment_results)[i] %||% paste("Enrichment", i)
  
  p <- create_dotplot(df, plot_name)
  if (!is.null(p)) {
    all_plots[[plot_name]] <- p
  }
}

# Display plots
if (length(all_plots) >= 1) {
  for (plot_name in names(all_plots)) {
    cat(sprintf("\n\n### DotPlot - %s\n\n", plot_name))
    grid.arrange(grobs = list(all_plots[[plot_name]]), ncol = 1)
  }
}
```


# Protein-protein interaction network (PPI)

```{r grn-summary , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=10}
all_plots <- list()

for (i in seq_along(params$grn_object)) {
  g <- params$grn_object[[i]]
  network_name <- names(params$grn_object)[i] %||% paste("Network", i)

  if (vcount(g) > 2) {
    # Compute degree
    degree_vals <- degree(g, mode = "all")
    V(g)$degree <- degree_vals

    # Identify top hub nodes
    top_n <- min(20, vcount(g))
    top_nodes <- names(sort(degree_vals, decreasing = TRUE))[1:top_n]

    # Create layout
    layout <- create_layout(g, layout = "fr")

    # Add labels
    layout$label <- if (vcount(g) < 20) {
      layout$name
    } else {
      ifelse(layout$name %in% top_nodes, layout$name, "")
    }

    all_plots[[network_name]] <- ggraph(layout) +
      geom_edge_link(aes(edge_alpha = 0.5), color = "grey", show.legend = FALSE) +
      geom_node_point(aes(size = degree, fill = degree), shape = 21, color = "black", stroke = 0.5) +
      scale_size(range = c(1, 10)) +
      scale_fill_gradient(low = "azure", high = "darkcyan") +
      geom_label_repel(
        aes(x = x, y = y, label = label),
        color = "black", size = 3, fill = "white", box.padding = 0.5,
        point.padding = 0.5, max.overlaps = 1000
      ) +
      theme_void() +
      labs(
        title = paste("Protein-protein interaction network -", network_name),
        size = "Node Degree",
        fill = "Node Degree"
      ) +
      theme(legend.position = "right", plot.title = element_text(hjust = 0.5, size = 14))
  }
}

# Plot all
if (length(all_plots) >= 1) {
  for (plot_name in names(all_plots)) {
    cat(sprintf("\n\n### PPI - %s\n\n", plot_name))
    grid.arrange(grobs = list(all_plots[[plot_name]]), ncol = 1)
  }
}

```

------------------------------------------------------------------------

# Session Info

```{r session-info}
sessionInfo()
```

------------------------------------------------------------------------

### Auto-generated on `r Sys.time()` using `XYomics` reporting

---
title: "Global Report"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Template_report}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(ggplot2)
library(DT)
library(dplyr)
library(igraph)
library(ggraph)
library(knitr)
library(kableExtra)
library(ggrepel)
library(htmltools)
library(gridExtra)
#library(XYomics)

# fallback operator
`%||%` <- function(x, y) if (!is.null(x)) x else y
```

#  Overview

This is a global report built for **XYomics output on Single-cell data**.

**Date:** `r params$report_date`\
**Includes:** - Differential expression analysis (DEGs) - Pathway enrichment analysis - Protein-protein interaction networks analysis

------------------------------------------------------------------------

#  Differential Expression Results of categorized DEGs by cell types

```{r de-results, echo=F}


get_name <- function(df){
  for (i in seq_along(params$de_results)){

    name=names(params$de_results)[i]

    if (length(all.equal(df, params$de_results[[name]]))==1){
      
      name_df=name

    } 
  } 
  return(name_df)
}

render_de_table <-  function(df) {
    if (nrow(df) == 0) {
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p("No significant DEGs for this category.")
    ))
  }
  else
  {   name <- get_name(df) 
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p( datatable(df%>% mutate(across(ends_with("logFC"), ~round(.,digits=3))) %>% mutate(across(ends_with("FDR"), ~format(.,scientific=T,digits=3))),
            options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )

  }

 
  
  
}

tagList(
  lapply(params$de_results, render_de_table)
)
```
## Volcano Plots by cell types and categories of DEGs

```{r volcano-plots , echo=FALSE,results='asis', fig.keep='all', fig.width=8, fig.height=6}


all_plots <- plot_volcano_deg(params$de_results)



for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}
```

------------------------------------------------------------------------

# Enrichment / Pathway Analysis

```{r enrichment-tables, echo=FALSE}

render_enrichment_table <- function(df, name) {
  if (is.null(df)) {
    return(tags$div(
      tags$h4(paste("Pathway Enrichment -", name)),
      tags$p("No significant pathways for this category.")
    ))
  }
  else
  {  
    df=df %>% relocate(DEG_Type)
    return(tags$div(
    tags$h4(paste("Pathway Enrichment -", name)),
    tags$p(datatable(
      df %>% mutate_if(is.numeric, ~format(., scientific=T, digits=3)),
      options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )
    
  }
  
  
}



tagList(lapply(seq_along(params$enrichment_results ), function(i) {
  name <- names(params$enrichment_results )[i] %||% paste("Set", i)
  sublist <-params$enrichment_results [[i]]
  combined_df <- do.call(rbind, lapply(names(sublist ), function(name) {
    df <- as.data.frame(sublist [[name]])
    if (nrow(df) == 0) return(NULL) 
    df$DEG_Type <- name
    return(df)
  }))


  render_enrichment_table(combined_df, name)
}))

```

## Dotplot of the pathways by cell types
```{r pathways visualization , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE}

all_plots <-plot_enrichment_dotplots(params$enrichment_results)



for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}


```

# Protein-protein interaction network (PPI)

```{r grn-summary , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=10, message=FALSE}


all_plots <-plot_network_pipeline(params$grn_object, params$de_results)

for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}

```

------------------------------------------------------------------------

# Session Info

```{r session-info}
# sessionInfo()
```

------------------------------------------------------------------------

### Auto-generated on `r Sys.time()` using `XYomics` reporting

---
title: "Global Report with Hormonal Network Analysis"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Template_report_hormonal}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include=FALSE}
library(ggplot2)
library(DT)
library(dplyr)
library(igraph)
library(ggraph)
library(knitr)
library(kableExtra)
library(ggrepel)
library(htmltools)
library(gridExtra)

# fallback operator
`%||%` <- function(x, y) if (!is.null(x)) x else y
```

#  Overview

This is a global report built for **XYomics output on Single-cell data**.

**Date:** `r params$report_date`\
**Includes:** - Differential expression analysis (DEGs) - Pathway enrichment analysis - Hormonal protein-protein interaction networks analysis

------------------------------------------------------------------------

#  Differential Expression Results of categorized DEGs by cell types

```{r de-results, echo=F}
name_df=""

get_name <- function(df){
  for (i in seq_along(params$de_results)){

    name=names(params$de_results)[i]

    if (length(all.equal(df, params$de_results[[name]]))==1){
      
      name_df=name

    } 
  } 
  return(name_df)
}

render_de_table <-  function(df) {
    if (nrow(df) == 0) {
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p("No significant DEGs for this category.")
    ))
  }
  else
  {   name <- get_name(df) 
    return(tags$div(
      tags$h4(paste("DEGs Results -", name)),
      tags$p( datatable(df%>% mutate(across(ends_with("logFC"), ~round(.,digits=3))) %>% mutate(across(ends_with("FDR"), ~format(.,scientific=T,digits=3))),
            options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )

  }

 
  
  
}

tagList(
  lapply(params$de_results, render_de_table)
)
```
## Volcano Plots by cell types and categories of DEGs

```{r volcano-plots , echo=FALSE,results='asis', fig.keep='all', fig.width=8, fig.height=6}

create_volcano_plot <- function(df, deg_type, x_col, y_col, title_suffix) {
  df_filtered <- df %>% filter(DEG_Type == deg_type)
  
  
   df_filtered  <-   df_filtered  %>%
  mutate(GeneName = rownames(.)) %>%
  mutate(Significant = GeneName %in% c(
      df_filtered  %>% filter(Male_FDR < 0.05, abs(Male_avg_logFC) > 1.5) %>%
      slice_min(Male_FDR, n = 5, with_ties = FALSE) %>%
      rownames(),
      df_filtered  %>% filter(Female_FDR < 0.05, abs(Female_avg_logFC) > 1.5) %>%
      slice_min(Female_FDR, n = 5, with_ties = FALSE) %>%
      rownames()
  ))
  
  ggplot(df_filtered, aes_string(x = x_col, y = y_col, color = "Significant")) +
    geom_point(alpha = 0.6) +
    scale_color_manual(values = c("grey70", "firebrick")) +
    theme_minimal() +
    labs(
      x = "log2 Fold Change",
      y = "-log10(p-value)",
      title = paste("Volcano Plot -", title_suffix, "-", deg_type)
    ) +
    theme(legend.position = "bottom", plot.title = element_text(size = 10, hjust = 0.5, margin = margin(b = 10))) +
    geom_text_repel(
      data = df_filtered %>% filter(Significant),
      aes(label = Gene_Symbols),
      size = 3,
      max.overlaps = 30
    )
}


# Store plots in a list
all_plots <- list()

# Generate plots
for (i in seq_along(params$de_results)) {
  df <-params$de_results[[i]]
  name <- names(params$de_results)[i] %||% paste("Set", i)
  

  
  plots <- list(
    create_volcano_plot(df, "female-specific", "Female_avg_logFC", "-log10(Female_FDR)", name),
    create_volcano_plot(df, "male-specific", "Male_avg_logFC", "-log10(Male_FDR)", name),
    create_volcano_plot(df, "sex-dimorphic", "Male_avg_logFC - Female_avg_logFC", "-log10(Male_FDR + Female_FDR)", name),
    create_volcano_plot(df, "sex-neutral", "Male_avg_logFC + Female_avg_logFC", "-log10(Male_FDR + Female_FDR)", name)
  )
  
  all_plots[[name]] <- plots
  
}


for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  grid.arrange(grobs = all_plots[[set_name]], ncol = 2)
}
```

------------------------------------------------------------------------

# Enrichment / Pathway Analysis

```{r enrichment-tables, echo=FALSE}

render_enrichment_table <- function(df, name) {
  if (is.null(df)) {
    return(tags$div(
      tags$h4(paste("Pathway Enrichment -", name)),
      tags$p("No significant pathways for this category.")
    ))
  }
  else
  {  
    df=df %>% relocate(DEG_Type)
    return(tags$div(
    tags$h4(paste("Pathway Enrichment -", name)),
    tags$p(datatable(
      df %>% mutate_if(is.numeric, ~format(., scientific=T, digits=3)),
      options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )
    
  }
  
  
}



tagList(lapply(seq_along(params$enrichment_results), function(i) {
  name <- names(params$enrichment_results)[i] %||% paste("Set", i)
  sublist <- params$enrichment_results[[i]]
  combined_df <- do.call(rbind, lapply(names(sublist ), function(name) {
    df <- sublist [[name]]
    if (nrow(df) == 0) return(NULL) 
    df$DEG_Type <- name
    return(df)
  }))


  render_enrichment_table(combined_df, name)
}))

```

## Dotplot of the pathways by cell types
```{r pathways visualization , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=7}

create_dotplot <- function(df, inner_name) {
  if (nrow(df) > 0) {
    enr <- convertdf2enr(df)
    p <-  suppressMessages(dotplot(enr, showCategory = 10, title = paste("Top enriched Pathways", inner_name)) +
      scale_y_discrete(expand = expansion(mult = c(0.1, 0.1))) +
      theme(plot.margin = margin(10, 10, 15, 10)) +
      scale_size_continuous(labels = function(x) as.integer(round(x)))  )
    return(p)
  } else {
    return(NULL)
  }
}

all_plots <- list()

for (i in seq_along(params$enrichment_results)) {
  df <- params$enrichment_results[[i]]
  outer_name <- names(params$enrichment_results)[i] %||% paste("Set", i)
  sublist <- params$enrichment_results[[i]]
  
  dplots <- list()  # <-- Move this INSIDE the outer loop
  
  for (j in seq_along(sublist)) {
    df <- sublist[[j]]
    inner_name <- names(sublist)[j] %||% paste("Subset", j)
    dplots[[inner_name]] <- create_dotplot(df, inner_name)
  }
  
  all_plots[[outer_name]] <- dplots
}


for (set_name in names(all_plots)) {
  if (length(all_plots[[set_name]])>=1){
    cat(sprintf("\n\n###  DotPlot - %s\n\n", set_name))
    grid.arrange(grobs = all_plots[[set_name]], ncol =2)
  }
 
}

```

# Hormonal protein-protein interaction network (PPI)

```{r grn-summary , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=10}


all_plots <- list()

for (i in seq_along(params$grn_object)) {
  outer_name <- names(params$grn_object)[i] %||% paste("GRN_Group", i)
  sublist <- params$grn_object[[i]]
  gplots <- list()

  for (j in seq_along(sublist)) {
    g <- sublist[[j]]
    inner_name <- names(sublist)[j] %||% paste("Network", j)

    if (vcount(g) > 2) {
      # Compute degree
      degree_vals <- degree(g, mode = "all")
      V(g)$degree <- degree_vals

      # Identify top hub nodes
      top_n <- min(20, vcount(g))
      top_nodes <- names(sort(degree_vals, decreasing = TRUE))[1:top_n]

      # Create layout
      layout <- create_layout(g, layout = "fr")

      # Add labels
      layout$label <- if (vcount(g) < 20) {
        layout$name
      } else {
        ifelse(layout$name %in% top_nodes, layout$name, "")
      }

      gplots[[inner_name]] <- ggraph(layout) +
        geom_edge_link(aes(edge_alpha = 0.5), color = "grey", show.legend = FALSE) +
        geom_node_point(aes(size = degree, fill = degree), shape = 21, color = "black", stroke = 0.5,  show.legend = c(size = FALSE, fill = TRUE)) +
        scale_size(range = c(1, 10)) +
        scale_fill_gradient(low = "azure", high = "darkcyan",  labels = scales::number_format(accuracy = 1)) +
        geom_label_repel(
          aes(x = x, y = y, label = label),
          color = "black", size = 3, fill = "white", box.padding = 0.5,
          point.padding = 0.5, max.overlaps = 1000
        ) +
        theme_void() +
        labs(
          title = paste("Hormonal protein-protein interaction network -", outer_name, "-", inner_name),
          fill = "Node Degree"
        ) +
        theme(legend.position = "right", plot.title = element_text(hjust = 0.5, size = 14)) 
    }
  }
  all_plots[[outer_name]] <- gplots
}

for (set_name in names(all_plots)) {
  if (length(all_plots[[set_name]])>=1){
    cat(sprintf("\n\n### Hormonal PPI - %s\n\n", set_name))
    grid.arrange(grobs = all_plots[[set_name]], ncol =2)
  } 
 
}

```

------------------------------------------------------------------------

# Session Info

```{r session-info}
sessionInfo()
```

------------------------------------------------------------------------

### Auto-generated on `r Sys.time()` using `XYomics` reporting

---
title: "Single Cell RNA-Seq Example"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Single Cell Example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Single-Cell RNA-Seq Analysis with XYomics
Authors: Sophie Le Bars, Mohamed Soudy, and Enrico Glaab

# Introduction

This vignette provides a guide for using the **XYomics** package to analyze sex-related patterns in single-cell RNA-sequencing (scRNA-seq) data. We will walk through a complete workflow, from data simulation to network analysis, demonstrating how to identify and interpret sex-specific gene expression changes at the cell-type level.

The tutorial covers:
1.  **Simulating a scRNA-seq dataset**.
2.  **Standard preprocessing** using the Seurat package.
3.  **Performing differential expression analysis** using different strategies: sex-stratified analysis and interaction analysis.
4.  **Detecting and categorizing sex-specific DEGs** for each cell type.
5.  **Visualizing gene expression** using dot plots.
6.  **Performing pathway enrichment analysis** to uncover biological functions.
7.  **Constructing and visualizing protein-protein networks**.

# Data Simulation and Preprocessing

## Simulating Single-Cell Data
We start by simulating a scRNA-seq dataset containing 10 mock samples. This creates a Seurat object with a count matrix ready for analysis.

```{r simulate-data, message=FALSE, warning=FALSE}
library(Seurat)
library(org.Hs.eg.db)
library(stringr)
library(clusterProfiler)
library(igraph)
library(XYomics)
library(ggrepel)
library(ggraph)
library(dplyr)
library(tidyr)

set.seed(123)
gene_sample <- keys(org.Hs.eg.db, keytype = "SYMBOL") %>% 
  as.data.frame() %>% 
  setNames("genes") %>%
  filter(!str_starts(genes, "LOC")) 

gene_sample <- sample(gene_sample$genes, 300, replace = FALSE)

matrices <- lapply(1:10, function(i) {
  batch_effect <- rgamma(300, shape = 2, scale = 1.5)
  m <- matrix(
    ifelse(runif(300*100) < 0.1, 0, rnbinom(300*100, size = 1/0.5, mu = 10 * batch_effect)),
    nrow = 300,
    dimnames = list(gene_sample, paste("Sample", i, ": Cell", 1:100))
  )
  CreateSeuratObject(counts = m, meta.data = data.frame(sample = paste("Sample", i)))
})

sim.seurat <- Reduce(merge, matrices)
sim.seurat@meta.data$sample <- sapply(strsplit(rownames(sim.seurat@meta.data), ":"), "[", 1)
```

## Preprocessing and Clustering
We perform standard scRNA-seq preprocessing, including normalization, feature selection, and scaling. We then apply dimensionality reduction (PCA and UMAP) and clustering to identify cell populations.

```{r preprocessing, message=FALSE, warning=FALSE, results='hide'}
sim.seurat <- NormalizeData(sim.seurat)
sim.seurat <- FindVariableFeatures(sim.seurat, selection.method = "vst", nfeatures = 2000)
sim.seurat <- ScaleData(sim.seurat)
sim.seurat <- RunPCA(sim.seurat)
sim.seurat <- FindNeighbors(sim.seurat, dims = 1:10)
sim.seurat <- FindClusters(sim.seurat)
sim.seurat <- RunUMAP(sim.seurat, dims = 1:10)
DimPlot(sim.seurat)
```

## Annotating Cell Types and Conditions
For the analysis, the Seurat object must contain metadata columns for cell type (`cell_type`), sex (`sex`), and condition (`status`). Here, we add mock annotations to our simulated data.

```{r annotate-data, warning=FALSE}
# Assign mock cell types
cellTypes <- c("cell type 1", "cell type 2", "cell type 3", "cell type 4", "cell type 5")
sim.seurat@meta.data$cell_type <- sample(cellTypes, nrow(sim.seurat@meta.data), replace = TRUE)
Idents(sim.seurat) <- "cell_type"

# Assign mock status (WT/TG) and sex (M/F)
samples <- sim.seurat@meta.data$sample
sim.seurat@meta.data$status <- ifelse(grepl("1|3|5|8|9", samples), "TG", "WT")
sim.seurat@meta.data$sex <- ifelse(grepl("1|2|4|8|10", samples), "M", "F")

```

# Differential Expression Analysis Strategies

When analyzing sex differences, researchers must be aware of the pitfalls of associated statistical analyses, including the limitations of sex-stratified analyses and the challenges of analyzing interactions between sex and disease state.

**Sex-stratified analyses** use standard statistical tests for differential molecular abundance analysis to test for disease-associated changes in each sex separately. However, a pure sex-stratified analysis may misclassify a change as sex-specific if it uses a standard significance threshold to assess both the presence and absence of an effect. Stochastic variation in significance scores around a chosen threshold may lead to the erroneous detection of significance specific to only one sex, especially if the p-value in the other sex marginally exceeds the chosen threshold. In addition, such an analysis may miss sex-modulated changes, where significant changes in both sexes share the same direction but differ significantly in magnitude; these changes require cross-sex comparisons for accurate detection.

**Interaction analysis** (using a multiplicative term like `sex*disease`) formally tests whether the relationship between disease and molecular changes differs significantly between males and females. Not only can such interaction terms reveal complexities in disease mechanisms that might otherwise be obscured in analyses that do not consider SABV, but they also have the potential to detect changes that are limited to the magnitude of an effect, an aspect that sex-stratified analyses do not capture. Nevertheless, robust estimation of interaction effects requires large sample sizes, which are often not available due to the costs associated with advanced molecular profiling techniques such as single-cell RNA sequencing.

The **XYomics** package provides functions for both types of analyses.

## Method 1: Sex-Stratified Analysis

We use `sex_stratified_analysis_sc()` to identify DEGs between conditions separately for males and females within each cell type.

```{r sex-stratified-analysis, message=FALSE, warning=FALSE}
# Run for all cell types
sim.seurat <- JoinLayers(sim.seurat) #this is for the simulated object but not always required for your own object
results <- sex_stratified_analysis_sc(sim.seurat, min_logfc = -Inf)

sex_degs <- lapply(levels(as.factor(sim.seurat@meta.data$cell_type)), function(cell) {
  list(
    male = results$male_DEGs[[cell]],
    female = results$female_DEGs[[cell]]
  )
})

names(sex_degs) <- levels(as.factor(sim.seurat@meta.data$cell_type))
result_categories <- lapply(sex_degs, function(degs) categorize_sex_sc(degs$male, degs$female))

# Example for one cell type
result_one <- result_categories$`cell type 1`
cat("\nTop categorized DEGs for 'cell type 1' (from stratified analysis):\n")
head(result_one)
```

## Method 2: Sex-Phenotype Interaction Analysis

We use `sex_interaction_analysis_sc()` to perform a formal interaction analysis, directly comparing the condition effect between sexes.

```{r sex-interaction-analysis, message=FALSE, warning=FALSE}
# Example for one cell type (e.g., "cell type 1")
target_cell <- "cell type 1"
interaction_results_one_cell <- sex_interaction_analysis_sc(sim.seurat, target_cell_type = target_cell)
cat(paste0("\nSummary of Sex-Phenotype Interaction analysis results for '", target_cell, "':\n"))
print(interaction_results_one_cell$summary_stats)
cat(paste0("\nTop genes from Sex-Phenotype Interaction analysis for '", target_cell, "':\n"))
print(head(interaction_results_one_cell$all_results[[1]]))
```

# Visualization of Gene Expression
Dot plots are an effective way to visualize gene expression in scRNA-seq data. Here, we show the expression of the top male-specific, female-specific, and sex-dimorphic genes across all cell types, split by sex and condition.

```{r dot-plots, message=FALSE, warning=FALSE, fig.width=10, fig.height=8}
# Get top gene from each category for 'cell type 1'
top_male_gene <- result_one %>% filter(DEG_Type == "male-specific") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)
top_female_gene <- result_one %>% filter(DEG_Type == "female-specific") %>% top_n(1, -Female_FDR) %>% pull(Gene_Symbols)
top_dimorphic_gene <- result_one %>% filter(DEG_Type == "sex-dimorphic") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)

top_genes <- c(top_male_gene, top_female_gene, top_dimorphic_gene)

# Create dot plot
if (length(top_genes) > 0) {
  sim.seurat$group_plot <- paste(sim.seurat$cell_type, sim.seurat$sex, sim.seurat$status, sep = "_")
  Idents(sim.seurat) <- "group_plot"
  # Seurat v5 requires specifying assay for DotPlot
  DotPlot(sim.seurat, features = top_genes, cols = c("blue", "red"), assay = "RNA") +
    coord_flip() +
    labs(title = "Expression of Top Sex-Specific and Dimorphic Genes") +
    theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5))
}
```

# Pathway Enrichment Analysis
We perform pathway enrichment analysis to identify biological pathways over-represented in our DEG categories. The `categorized_enrich_sc` function automates this for all categories.

```{r pathway-analysis, message=FALSE, warning=FALSE}
# Run for all cell types
pathway_category <- lapply(result_categories, function(cell) categorized_enrich_sc(cell, enrichment_db = "REACTOME"))
cat("\nTop pathway results for 'cell type 1':\n")
print(head(pathway_category$`cell type 1`))

# Run for one specific cell type
pathway_category_one <- categorized_enrich_sc(result_one)
cat("\nTop pathway results for 'cell type 1' (re-run separately):\n")
print(head(pathway_category_one))
```

# Protein-protein interaction Network Analysis
Finally, we construct a protein-protein interaction network to explore the interactions between our DEGs.

## Building the Network
We first fetch a protein-protein interaction network from the STRING database. Then, we use the PCSF algorithm via `construct_ppi_pcsf` to extract a relevant subnetwork based on "prizes" derived from our DEG p-values.

```{r build-network, warning=FALSE, message=FALSE}
# Fetch STRING network (can be replaced with a custom network)
# g <- get_string_network(organism = "9606", score_threshold = 900)

# Load a pre-existing network from a file
g <- readRDS(system.file("extdata", "string_example_network.rds", package = "XYomics")) # This should load the 'g' variable

```


```{r network-analysis, warning=FALSE, message=FALSE}

### Run for all cell type


network_results <- list()

for (cell_type in names(result_categories)) {
  
  # Extract DEG results for the current cell type
  cell_results <- result_categories[[cell_type]]
  
  # Filter DEGs by type
  male_specific       <- cell_results[cell_results$DEG_Type == "male-specific", ]
  female_specific     <- cell_results[cell_results$DEG_Type == "female-specific", ]
  sex_dimorphic       <- cell_results[cell_results$DEG_Type == "sex-dimorphic", ]
  sex_neutral         <- cell_results[cell_results$DEG_Type == "sex-neutral", ]
  
  # Convert to log-transformed prizes
  male_prizes <- -log10(male_specific$Male_FDR)
  names(male_prizes) <- male_specific$Gene_Symbols
  
  female_prizes <- -log10(female_specific$Female_FDR)
  names(female_prizes) <- female_specific$Gene_Symbols
  
  dimorphic_prizes <- -log10((sex_dimorphic$Male_FDR + sex_dimorphic$Female_FDR) / 2)
  names(dimorphic_prizes) <- sex_dimorphic$Gene_Symbols
  
  neutral_prizes <- -log10((sex_neutral$Male_FDR + sex_neutral$Female_FDR) / 2)
  names(neutral_prizes) <- sex_neutral$Gene_Symbols
  
  # Construct ppis
  male_network     <- construct_ppi_pcsf(g = g, prizes = male_prizes)
  female_network   <- construct_ppi_pcsf(g = g, prizes = female_prizes)
  dimorphic_network <- construct_ppi_pcsf(g = g, prizes = dimorphic_prizes)
  neutral_network   <- construct_ppi_pcsf(g = g, prizes = neutral_prizes)
  
  # Store all results per cell type
  network_results[[cell_type]] <- list(
    male_network      = male_network,
    female_network    = female_network,
    dimorphic_network = dimorphic_network,
    neutral_network   = neutral_network
  )
}

network_results


# Example for one specific cell type

# Create prizes from dimorphic DEGs in 'cell type 1'
dimorphic_specific <- result_one[result_one$DEG_Type == "sex-dimorphic", ]
dimorphic_prizes <- -log10((dimorphic_specific$Male_FDR + dimorphic_specific$Female_FDR) / 2)
names(dimorphic_prizes) <-dimorphic_specific$Gene_Symbols

# Construct the PCSF subnetwork
dimorphic_network <- construct_ppi_pcsf(g = g, prizes = dimorphic_prizes)
```

## Visualizing the Network
We use plot_network() to visualize the network, highlighting nodes based on their degree (i.e., significance). Each node also includes a barplot showing logFC values, blue for males and pink for females.

```{r visualize-network, warning=FALSE, message=FALSE}
if (!is.null(dimorphic_network) && igraph::vcount(dimorphic_network) > 0) {
  #Generate network visualization
  plot_network(dimorphic_network, "Cell type 1", "sex-dimorphic", result_one)
} else {
  cat("No network could be constructed for this category.")
}
```

# Generating a Report
The `generate_cat_report` function can be used to compile all results into a single HTML report.

```{r report, eval=FALSE}
# This command generates a comprehensive HTML report
network_results  <- lapply(network_results , function(cell) {
  Filter(Negate(is.null), cell)
})


generate_cat_report(result_categories, pathway_category, network_results)


```

---
title: "Bulk RNA-Seq Example"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Bulk RNA-Seq Example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Bulk RNA-Seq Analysis with XYomics
Authors: Enrico Glaab and Sophie Le Bars

# Introduction

This vignette demonstrates a standard workflow for analyzing bulk RNA-seq data to identify sex-specific effects using the **XYomics** package. The package provides a streamlined set of tools for differential expression, pathway analysis, and network-based interpretation.

This tutorial covers the following key steps:

1.  **Simulating a bulk RNA-seq dataset** with built-in sex-specific and shared gene expression changes.
2.  **Discussing and applying two different strategies for differential expression analysis**: sex-stratified analysis, and interaction analysis.
3.  **Identifying and categorizing sex-specific genes** into distinct groups (male-specific, female-specific, etc.) based on stratified analysis.
4.  **Visualizing expression patterns** of top genes using the package's built-in plotting functions.
5.  **Conducting pathway enrichment analysis** to determine the biological functions of the identified gene sets.
6.  **Constructing and visualizing a protein-protein interaction network** to explore the interactions between key genes.

# Data Simulation

We begin by simulating a bulk RNA-seq dataset to illustrate the package's capabilities. The `simulate_omics_data` function creates a dataset with 500 genes and 100 samples, balanced across phenotype (Control/Disease) and sex (male/female).

The four categorized groups are:
-   **Male-specific:** Genes up-regulated in the disease state only in males.
-   **Female-specific:** Genes up-regulated in the disease state only in females.
-   **Sex-dimorphic:** Genes regulated in opposite directions between males and females in the disease state.
-   **Sex-neutral:** Genes regulated with same directions between males and females in the disease state.

```{r simulate-data, message=FALSE, warning=FALSE}
library(XYomics)
library(dplyr)

# Load precomputed bulk dataset
expression_data <- readRDS(
  system.file("extdata", "bulk_expression.rds", package = "XYomics")
)

sex <- readRDS(
  system.file("extdata", "bulk_sex.rds", package = "XYomics")
)

phenotype <- readRDS(
  system.file("extdata", "bulk_phenotype.rds", package = "XYomics")
)
```

# Differential Expression Analysis Strategies

When analyzing sex differences, researchers must be aware of the pitfalls of associated statistical analyses, including the limitations of sex-stratified analyses and the challenges of analyzing interactions between sex and disease state.

**Sex-stratified analyses** use standard statistical tests for differential molecular abundance analysis to test for disease-associated changes in each sex separately. These may include classical parametric hypothesis tests, such as Welch’s test for normally distributed data, or non-parametric tests, such as the Mann–Whitney U test, as well as special moderated statistics for high-dimensional omics data analysis, such as the empirical Bayes moderated t-statistic. However, a pure sex-stratified analysis may misclassify a change as sex-specific if it uses a standard significance threshold to assess both the presence and absence of an effect. Stochastic variation in significance scores around a chosen threshold may lead to the erroneous detection of significance specific to only one sex, especially if the p-value in the other sex marginally exceeds the chosen threshold. In addition, such an analysis may miss sex-modulated changes, where significant changes in both sexes share the same direction but differ significantly in magnitude; these changes require cross-sex comparisons for accurate detection.

**Interaction analysis**  formally tests whether the relationship between disease and molecular changes differs significantly between males and females. Not only can such interaction terms reveal complexities in disease mechanisms that might otherwise be obscured in analyses that do not consider SABV, but they also have the potential to detect changes that are limited to the magnitude of an effect, an aspect that sex-stratified analyses do not capture. Nevertheless, robust estimation of interaction effects requires large sample sizes, which are often not available due to the costs associated with advanced molecular profiling techniques such as single-cell RNA sequencing.

The **XYomics** package provides functions for both types of analyses.

## Method 1: Sex-Stratified Analysis

Here, we use the `sex_stratified_analysis_bulk()` function to perform a stratified analysis.

```{r stratified-analysis, eval=FALSE}
res <- sex_stratified_analysis_bulk(expression_data, sex, phenotype)
# The `min_samples` parameter defines the minimum number of cells required
# to perform the interaction analysis. The default value is 3.


```

```{r load-stratified-results}
res <- readRDS(
  system.file("extdata", "bulk_results_degs.rds", package = "XYomics")
)
# Internal QC results from XYomics can be accessed through:
# res$validation
#
# This includes validation of sex and phenotype balance, group sample sizes,
# detection of design imbalances, and identification of groups below the
# minimum sample threshold required for reliable downstream analysis.
res$validation
```


# Identification of Sex-Specific Genes (from Stratified Analysis)

The `categorize_sex()` function categorizes genes based on the results of the **stratified analysis**. This provides a useful, albeit potentially less robust, classification.

```{r sex-specific-analysis, eval=FALSE}
res_cat <- categorize_sex(res$male_DEGs, res$female_DEGs)
```

```{r load-categorized-results}
res_cat <- readRDS(
  system.file("extdata", "bulk_results_cat.rds", package = "XYomics")
)

cat("Top sex-stratified differentially expressed genes:\n")
head(res_cat)

table(res_cat$DEG_Type)
```


## Method 2: Interaction Term Analysis

We use the `sex_interaction_analysis_bulk()` function to perform a formal interaction analysis.

```{r interaction-term-analysis-bulk, eval=FALSE}
interaction_term_results_bulk <- sex_interaction_analysis_bulk(expression_data, phenotype, sex)
# The `min_samples` parameter defines the minimum number of cells required
# to perform the interaction analysis. The default value ofr interaction analysis is 20.

```

```{r load-interaction-results}
interaction_term_results_bulk <- readRDS(
  system.file("extdata", "bulk_interaction_results.rds", package = "XYomics")
)

head(interaction_term_results_bulk$summary_stats)# signficant sex-modulated DEGs: interaction_term_results_bulk$sig_results
```





# Visualization of Expression Patterns

We use `generate_violinplot_bulk()` to visualize the expression of top genes identified by the stratified analysis.

```{r visualization}
top_male_gene <- res_cat %>%
  filter(DEG_Type == "male-specific") %>%
  arrange(Male_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)

top_female_gene <- res_cat %>%
  filter(DEG_Type == "female-specific") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)
top_dimorphic_gene <- res_cat %>%
  filter(DEG_Type == "sex-dimorphic") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)
top_neutral_gene <- res_cat %>%
  filter(DEG_Type == "sex-neutral") %>%
  arrange(Female_FDR) %>%
  pull(Gene_Symbols) %>%
  head(1)

top_genes = c(top_male_gene, top_female_gene, top_dimorphic_gene, top_neutral_gene )

generate_violinplot_bulk(expression_data,sex,phenotype , top_genes)
```

# Pathway Enrichment Analysis

We perform pathway enrichment on the gene lists derived from the stratified analysis using `categorized_enrich()`.

```{r pathway-analysis, eval=FALSE}
pathway_results <- categorized_enrich(res_cat, enrichment_db = "GO")

# The `enrichment_db` parameter can be set to "GO", "KEGG", "REACTOME".
# If the user wants to use a custom database, a TERM2GENE data frame must be
# provided via the `custom_db` parameter.
#
# Gene identifiers can be either any Gene identifier type supported by OrgDb
#' (e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT")
# using the `gene_type` parameter.
#
# The `return_df` parameter controls the output format:
# if set to TRUE, results are returned as data frames;
# by default (FALSE), enrichResult objects are returned
```

```{r load-pathway-results}
pathway_results <- readRDS(
  system.file("extdata", "bulk_pathway_results.rds", package = "XYomics")
)

head(pathway_results)
```

```{r celltype1_plot , results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE}
# visualize via dotplot
plot <- plot_enrichment_dotplots(pathway_results)
print(plot)
```

# Protein-protein Interaction Network Analysis

Finally, we construct a protein-protein interaction network using the results from the stratified analysis.

### 1. Fetching the STRING Network

First, we download a protein-protein interaction network from the STRING database using `get_string_network()` or directly from the package.

```{r get-string-network, message=FALSE}
# Fetch STRING network (can be replaced with a custom network)
# g <- get_string_network(organism = "9606", score_threshold = 900)

# Load a pre-existing network from a file
g <- readRDS(system.file("extdata", "string_example_network.rds", package = "XYomics")) # You can also download the hormonal network "hormonal_network_metacore.rds" instead of "string_example_network.rds".
```

### 2. Constructing the PCSF Network

Next, we define "prizes" for our genes based on their statistical significance (e.g., -log10 of the p-value) and use the `construct_ppi_pcsf` function to build a context-specific network.

```{r construct-pcsf-network, eval=FALSE}
# Use dimorphic DE results to define prizes
dimorphic_specific <- res_cat[res_cat$DEG_Type == "sex-dimorphic", ]
dimorphic_prizes <- -log10((dimorphic_specific$Male_FDR + dimorphic_specific$Female_FDR) / 2)
names(dimorphic_prizes) <-dimorphic_specific$Gene_Symbols

# Construct the PCSF subnetwork

dimorphic_network <- construct_ppi_pcsf(g = g, prizes = dimorphic_prizes)
network <- ppi_pipeline(res_cat, g)
```

```{r load-network-results}
network <- readRDS(
  system.file("extdata", "bulk_network_results.rds", package = "XYomics")
)

neutral_network <- readRDS(
  system.file("extdata", "bulk_neutral_network.rds", package = "XYomics")
)

neutral_specific <- res_cat[res_cat$DEG_Type == "sex-neutral", ]
```

### 3. Visualizing the Network

Finally, we use the `plot_network()` function to visualize the network, highlighting nodes based on their degree (i.e., significance). Each node also includes a barplot showing logFC values, blue for males and pink for females.

```{r visualize-network}


plot_network(neutral_network, "sex-neutral", neutral_specific, show_barplot = T)


#Generate plots for all categories

all_plots <- plot_network_pipeline(network, res_cat)

```

### Generating a Report
The `generate_cat_report` function can be used to compile all results into a single HTML report.

```{r report, eval=FALSE}
# This command generates a comprehensive HTML report 

template_path = system.file("extdata", "Template_report_bulk.Rmd", package = "XYomics") # fetch the template for bulk from the package

generate_cat_report(res_cat, pathway_results, network, template_path = template_path)


```

# Conclusion

This vignette has demonstrated two key approaches for analyzing sex-specific effects in bulk RNA-seq data. While sex-stratified analysis is a common first step, **interaction analysis provides a more statistically robust method** for identifying genes whose regulation by disease is truly dependent on sex. We recommend using an interaction-based approach when possible, while being mindful of the need for adequate sample sizes.

---
title: "Global Report"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Template_report}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(ggplot2)
library(DT)
library(dplyr)
library(igraph)
library(ggraph)
library(knitr)
library(kableExtra)
library(ggrepel)
library(htmltools)
library(gridExtra)
#library(XYomics)

# fallback operator
`%||%` <- function(x, y) if (!is.null(x)) x else y
```

#  Overview

This is a global report built for **XYomics output on Single-cell data**.

**Date:** `r params$report_date`\
**Includes:** - Differential expression analysis (DEGs) - Pathway enrichment analysis - Protein-protein interaction networks analysis

------------------------------------------------------------------------

#  Differential Expression Results of categorized DEGs by cell types

```{r de-results, echo=F}
render_de_table <-  function(df) {
    if (nrow(df) == 0) {
    return(tags$div(
      tags$h4(paste("DEGs Results")),
      tags$p("No significant DEGs for this dataset.")
    ))
  }
  else
  {   
    return(tags$div(
      tags$h4(paste("DEGs Results ")),
      tags$p( datatable(df%>% mutate(across(ends_with("logFC"), ~round(.,digits=3))) %>% mutate(across(ends_with("FDR"), ~format(.,scientific=T,digits=3))),
            options = list(pageLength = 10, scrollX = TRUE), rownames = FALSE)))
  )

  }

 
  
  
}

tagList(
  render_de_table(params$de_results)
)
```
## Volcano Plots by cell types and categories of DEGs

```{r volcano-plots , echo=FALSE,results='asis', fig.keep='all', fig.width=8, fig.height=6}


all_plots <- plot_volcano_deg(params$de_results)



for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}
```

------------------------------------------------------------------------

# Enrichment / Pathway Analysis

```{r enrichment-tables, echo=FALSE}

render_enrichment_table <- function(df, name) {

  if (is.null(df) || !is.data.frame(df) || nrow(df) == 0) {
    return(tags$div(
      tags$h4(paste("Pathway Enrichment -", name)),
      tags$p("No significant pathways for this category.")
    ))
  }

  df <- df %>% dplyr::relocate(DEG_Type)

  tags$div(
    tags$h4(paste("Pathway Enrichment -", name)),
    DT::datatable(
      df %>% dplyr::mutate_if(is.numeric, ~ format(., scientific = TRUE, digits = 3)),
      options = list(pageLength = 10, scrollX = TRUE),
      rownames = FALSE
    )
  )
}

```

## Dotplot of the pathways by cell types
```{r pathways visualization , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE}

all_plots <-plot_enrichment_dotplots(params$enrichment_results)



for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}


```

# Protein-protein interaction network (PPI)

```{r grn-summary , echo = FALSE,  results='asis', fig.keep='all', fig.width=15, fig.height=10, message=FALSE}


all_plots <-plot_network_pipeline(params$grn_object, params$de_results)

for (set_name in names(all_plots)) {
  cat(sprintf("\n\n### Volcano Plot - %s\n\n", set_name))
  print(all_plots[[set_name]])
}

```

------------------------------------------------------------------------

# Session Info

```{r session-info}
sessionInfo()
```

------------------------------------------------------------------------

### Auto-generated on `r Sys.time()` using `XYomics` reporting

---
title: "Single Cell RNA-Seq Example"
output:
  html_document:
    toc: true
    toc_depth: 2
    number_sections: true
    theme: flatly
params:
  de_results: NULL
  enrichment_results: NULL
  grn_object: NULL
  report_date: !r Sys.Date()
vignette: >
  %\VignetteIndexEntry{Single Cell Example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Single-Cell RNA-Seq Analysis with XYomics
Authors: Sophie Le Bars, Mohamed Soudy, and Enrico Glaab

# Introduction

This vignette provides a guide for using the **XYomics** package to analyze sex-related patterns in single-cell RNA-sequencing (scRNA-seq) data. We will walk through a complete workflow using simulated data demonstrating how to identify and interpret sex-specific gene expression changes at the cell-type level. To ensure fast execution, all heavy computations are **precomputed** and loaded from the package.

The tutorial covers:
1.  **load a simulated scRNA-seq dataset**.
2.  **Standard preprocessing** using the Seurat package.
3.  **Performing differential expression analysis** using different strategies: sex-stratified analysis and interaction analysis.
4.  **Detecting and categorizing sex-specific DEGs** for each cell type.
5.  **Visualizing gene expression** using dot plots.
6.  **Performing pathway enrichment analysis** to uncover biological functions.
7.  **Constructing and visualizing protein-protein networks**.

# Data Simulation and Preprocessing

## Simulating Single-Cell Data
We start by load a simulated scRNA-seq dataset containing 10 simulated samples. The code for generating this simulated dataset can be found in the Gitlab in the **data_vignette** folder.

```{r simulate-data, message=FALSE, warning=FALSE}
library(Seurat)
library(XYomics)
library(dplyr)

sim.seurat <- readRDS(
  system.file("extdata", "sc_toy_seurat.rds", package = "XYomics")
)
```

## Preprocessing and Clustering
We performed standard scRNA-seq preprocessing, including normalization, feature selection, and scaling. We then apply dimensionality reduction (PCA and UMAP) and clustering to identify cell populations.

```{r preprocessing, message=FALSE, warning=FALSE, results='hide'}
# Precomputed during data generation

# sim.seurat <- NormalizeData(sim.seurat)
# sim.seurat <- FindVariableFeatures(sim.seurat)
# sim.seurat <- ScaleData(sim.seurat)
# sim.seurat <- RunPCA(sim.seurat)
# sim.seurat <- FindNeighbors(sim.seurat)
# sim.seurat <- FindClusters(sim.seurat)
# sim.seurat <- RunUMAP(sim.seurat)
DimPlot(sim.seurat)
```

## Annotating Cell Types and Conditions
For the analysis, the Seurat object must contain metadata columns for cell type (`cell_type`), sex (`sex`), and condition (`status`). Here, we added mock annotations to our simulated data.

```{r annotate-data, warning=FALSE}
table(sim.seurat$cell_type)
table(sim.seurat$sex)
table(sim.seurat$status)

```

# Differential Expression Analysis Strategies

When analyzing sex differences, researchers must be aware of the pitfalls of associated statistical analyses, including the limitations of sex-stratified analyses and the challenges of analyzing interactions between sex and disease state.

**Sex-stratified analyses** use standard statistical tests for differential molecular abundance analysis to test for disease-associated changes in each sex separately. However, a pure sex-stratified analysis may misclassify a change as sex-specific if it uses a standard significance threshold to assess both the presence and absence of an effect. Stochastic variation in significance scores around a chosen threshold may lead to the erroneous detection of significance specific to only one sex, especially if the p-value in the other sex marginally exceeds the chosen threshold. In addition, such an analysis may miss sex-modulated changes, where significant changes in both sexes share the same direction but differ significantly in magnitude; these changes require cross-sex comparisons for accurate detection.

**Interaction analysis** formally tests whether the relationship between disease and molecular changes differs significantly between males and females. Not only can such interaction terms reveal complexities in disease mechanisms that might otherwise be obscured in analyses that do not consider SABV, but they also have the potential to detect changes that are limited to the magnitude of an effect, an aspect that sex-stratified analyses do not capture. Nevertheless, robust estimation of interaction effects requires large sample sizes, which are often not available due to the costs associated with advanced molecular profiling techniques such as single-cell RNA sequencing.

The **XYomics** package provides functions for both types of analyses.

## Method 1: Sex-Stratified Analysis

We use `sex_stratified_analysis_sc()` to identify DEGs between conditions separately for males and females within each cell type. We use `sex_stratified_pipeline_sc()` to obtain the categorized DEGs for each cell type in our seurat object.

```{r sex-stratified-analysis, eval=FALSE}
# Run for all cell types
# if you want to have the female and male degs 

results_degs <- sex_stratified_analysis_sc(sim.seurat) 
# The differential expression method can be specified using the `test.use` parameter.
# Available options include "MAST", "bimod", "poisson", "LR", and any other
# method supported by Seurat::FindMarkers(). 
#
# The `min_samples` parameter defines the minimum number of cells required
# to perform the analysis. The default value is 3.

#
# Internal QC results generated by XYomics can be accessed via:
# results_degs$validation
#
# This QC output includes assessment of sex and phenotype balance, per-group cell
# counts, detection of under-represented groups below the minimum cell threshold,
# and overall design imbalance warnings, as well as dataset sparsity metrics.

# if you want the categorized DEGs by cell type more lenient than the default thesholds as it is simulated data 

results <- sex_stratified_pipeline_sc(sim.seurat, min_abs_logfc = 0.1,  alpha = 0.1) 
# By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# You can also run on one or multiple cell types, for example:
# target_cell_type_flag = "cell type 1", "cell type 2"

```




```{r load-stratified-results}
results_degs <- readRDS(
  system.file("extdata", "sc_results_degs_small.rds", package = "XYomics")
)

results <- readRDS(
  system.file("extdata", "sc_results_small.rds", package = "XYomics")
)

head(results$`cell type 1`)
```

## Method 2: Sex-Phenotype Interaction Analysis

We use `sex_interaction_pipeline_sc()` to perform a formal interaction analysis, directly comparing the condition effect between sexes for all cell types.

```{r sex-interaction-analysis, eval=FALSE}
# Example for one cell type (e.g., "cell type 1")

interaction_results <- sex_interaction_pipeline_sc(sim.seurat)
# By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# You can also specify one or multiple cell types, for example:
# target_cell_type_flag = c("cell type 1", "cell type 2")
#
# The `min_samples` parameter defines the minimum number of cells required
# to perform the interaction analysis. The default value is 20.
```

```{r load-interaction-results}
interaction_results <- readRDS(
  system.file("extdata", "sc_interaction_results_small.rds", package = "XYomics")
)

head(interaction_results$`cell type 1`$summary_stats)# signficant sex-modulated DEGs: interaction_results$`cell type 1`$sig_results 
```


# Visualization of Gene Expression
Dot plots are an effective way to visualize gene expression in scRNA-seq data. Here, we show the expression of the top male-specific, female-specific, sex-neutral and sex-dimorphic genes across all cell types, split by sex and condition.

```{r dot-plots, message=FALSE, warning=FALSE, fig.width=10, fig.height=8}
# Get top gene from each category for 'cell type 1'
top_male_gene <- results$`cell type 1` %>% filter(DEG_Type == "male-specific") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)
top_female_gene <- results$`cell type 1`  %>% filter(DEG_Type == "female-specific") %>% top_n(1, -Female_FDR) %>% pull(Gene_Symbols)
top_dimorphic_gene <- results$`cell type 1`  %>% filter(DEG_Type == "sex-dimorphic") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)
top_neutral_gene <- results$`cell type 1`  %>% filter(DEG_Type == "sex-neutral") %>% top_n(1, -Male_FDR) %>% pull(Gene_Symbols)

top_genes <- c(top_male_gene, top_female_gene, top_dimorphic_gene, top_neutral_gene)

generate_dotplot_sc(sim.seurat, top_genes, target_cell_type_flag = "cell type 1")
# By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# You can also specify one or multiple cell types, for example:
# target_cell_type_flag = c("cell type 1", "cell type 2")
```

# Pathway Enrichment Analysis
We perform pathway enrichment analysis to identify biological pathways over-represented in our DEG categories. The `categorized_enrich()` function automates this for all categories.

```{r pathway-analysis, eval=FALSE}
# Run for all cell types
pathway_category <- lapply(
  results,
  function(cell) categorized_enrich(cell, enrichment_db = "GO")
)
# The `enrichment_db` parameter can be set to "GO", "KEGG", "REACTOME".
# If the user wants to use a custom database, a TERM2GENE data frame must be
# provided via the `custom_db` parameter.
#
# Gene identifiers can be either any Gene identifier type supported by OrgDb
#' (e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT")
# using the `gene_type` parameter.
#
# The `return_df` parameter controls the output format:
# if set to TRUE, results are returned as data frames;
# by default (FALSE), enrichResult objects are returned.
# Run for one specific cell type
# pathway_category_one <- categorized_enrich(results$`cell type 1`, return_df =T, enrichment_db = "GO")
```


```{r load-pathway-results}
pathway_category <- readRDS(
  system.file("extdata", "sc_pathway_single_small.rds", package = "XYomics")
)

pathway_category_one <- readRDS(
  system.file("extdata", "sc_pathway_single_small_df.rds", package = "XYomics")
)

head(pathway_category_one$`female-specific`)
```


You can then visualize them using `plot_enrichment_dotplots()` function. 

```{r celltype1_plot ,  results='asis', fig.keep='all', fig.width=15, fig.height=7, message=FALSE}
# visualize via dotplot
plot <- plot_enrichment_dotplots(pathway_category)
print(plot)
```

# Protein-protein interaction Network Analysis
Finally, we construct a protein-protein interaction network to explore the interactions between our DEGs.

## Building the Network
We first fetch a protein-protein interaction network from the STRING database or directly without the package (hormonal_network_metacore.rds). Then, we use the PCSF algorithm via `ppi_pipeline()` to extract a relevant subnetwork based on "prizes" derived from our DEG p-values for all cell types.


```{r build-network, message=FALSE, warning=FALSE}
# Fetch STRING network (can be replaced with a custom network)
# g <- get_string_network(organism = "9606", score_threshold = 900)

# Load a pre-existing network from a file
g <- readRDS(system.file("extdata", "string_example_network.rds", package = "XYomics")) # You can also download the hormonal network "hormonal_network_metacore.rds" instead of "string_example_network.rds".

```

```{r network-analysis, eval=FALSE}

### Run for all cell type
network_results <- ppi_pipeline(results, g)

# By default, `target_cell_type_flag = "all"`, meaning that all cell types are plotted.
# You can also specify one or multiple cell types, for example:
# target_cell_type_flag = c("cell type 1", "cell type 2")

# Example for one specific cell type


network_results_one <- ppi_pipeline(results,g , target_cell_type_flag = "cell type 2")
```

```{r load-network-results}
network_results <- readRDS(
  system.file("extdata", "sc_network_results_small.rds", package = "XYomics")
)
```




## Visualizing the Network
We use plot_network() to visualize the network, highlighting nodes based on their degree (i.e., significance). Each node also includes a barplot showing logFC values, blue for males and pink for females.

```{r visualize-network, warning=FALSE, message=FALSE}
female_network <- network_results$`cell type 2`$female_network

#Generate network visualization
plot_network(female_network, "female-specific", results$`cell type 2`, "Cell type 2", show_barplot = T) 

#Generate all_plots for all categories and all cell types

all_plots <- plot_network_pipeline(network_results, results)

```

# Generating a Report
The `generate_cat_report` function can be used to compile all results into a single HTML report.

```{r report, eval=FALSE}
# This command generates a comprehensive HTML report for single-cell dataset
generate_cat_report(results, pathway_category, network_results)


```


# Notes

- All heavy computations are precomputed to ensure fast vignette build time.
- Users can reproduce the full workflow using the functions provided in **XYomics**.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_report.R
\name{generate_cat_report}
\alias{generate_cat_report}
\title{Generate a Comprehensive Analysis Report}
\usage{
generate_cat_report(
  results_cat,
  enrichment_cat,
  grn_cat,
  output_file = "cat_analysis_report.html",
  output_dir = tempdir(),
  template_path = NULL,
  quiet = TRUE
)
}
\arguments{
\item{results_cat}{A data frame or list containing differential expression results.}

\item{enrichment_cat}{A list with enrichment objects (e.g., BP, MF, KEGG, and optionally GSEA results).}

\item{grn_cat}{An igraph object representing the gene regulatory network (e.g., from PCSF analysis).}

\item{output_file}{Character. The desired name (and optionally path) for the rendered report (default: "analysis_report.html").}

\item{output_dir}{Character. Output directory to save the report to.}

\item{template_path}{Character. Path to the R Markdown template file. If \code{NULL}, the function uses the built-in template located in \code{inst/rmd/template_report.Rmd}.}

\item{quiet}{Logical. If \code{TRUE} (default), rendering will be quiet.}
}
\value{
A character string with the path to the rendered report.
}
\description{
This function creates an integrated report that combines key analysis outputs,
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{plot_network}
\alias{plot_network}
\title{Plot a condition-specific protein–protein interaction network with DEG annotations}
\usage{
plot_network(
  g,
  DEG_type,
  result_categories,
  cell_type = "",
  top_hubs = 20,
  show_barplot = FALSE
)
}
\arguments{
\item{g}{An igraph object representing the interaction network.}

\item{DEG_type}{Character string specifying the DEG category to visualize.}

\item{result_categories}{A data.frame containing DEG results with at least
the columns DEG_Type, Gene_Symbols, Male_avg_logFC, and Female_avg_logFC.}

\item{cell_type}{Optional character string used in the plot title.}

\item{top_hubs}{Integer specifying the number of hub genes to highlight.}

\item{show_barplot}{Logical indicating whether hub barplots are displayed.}
}
\value{
A ggplot object representing the network visualization.
}
\description{
Visualizes a gene or protein interaction network for a selected differential
expression category. Nodes are sized by degree centrality, and hub genes can be
optionally annotated with mini barplots showing sex-specific log fold changes.
}
\details{
Nodes represent genes or proteins and edges represent interactions. Node size
reflects degree centrality (number of connections). The top \code{top_hubs}
nodes by degree are labeled. When \code{show_barplot = TRUE}, mini barplots are
overlaid on hub nodes to display sex-specific log fold changes. The visualization
is restricted to the largest connected component of the network.
}
\examples{
# Minimal reproducible example (CRAN-safe)
library(igraph)

# Create a small toy network
g <- make_ring(5)
V(g)$name <- paste0("Gene", 1:5)

# Create minimal DEG table
result_categories <- data.frame(
  DEG_Type = rep("example", 5),
  Gene_Symbols = paste0("Gene", 1:5),
  Male_avg_logFC = runif(5, -1, 1),
  Female_avg_logFC = runif(5, -1, 1)
)

# Run function
plot_network(g, DEG_type = "example", result_categories = result_categories)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_interaction_analysis_sc.R
\name{sex_interaction_analysis_sc}
\alias{sex_interaction_analysis_sc}
\title{Perform Sex-Phenotype Interaction Analysis for Single-Cell Data}
\usage{
sex_interaction_analysis_sc(
  seurat_obj,
  target_cell_type,
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  min_logfc = 0.25,
  alpha = 0.05,
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  min_samples = 20
)
}
\arguments{
\item{seurat_obj}{A Seurat object.}

\item{target_cell_type}{Character. Cell type to analyze.}

\item{sex}{Character. Column name for sex (default "sex").}

\item{phenotype}{Character. Column name for phenotype (default "status").}

\item{celltype_col}{Character. Column name for cell type (default "cell_type").}

\item{min_logfc}{Numeric. Minimum absolute log fold change for reporting significant genes (default 0.25).}

\item{alpha}{Numeric. FDR threshold for significance (default 0.05).}

\item{sex_labels_vector}{Character vector of sex labels (default c("F","M")).}

\item{phenotype_labels_vector}{Character vector of phenotype groups (default c("WT","TG")).}

\item{min_samples}{Integer. Minimum number of cells per group (default 20).}
}
\value{
A list with all DE results, filtered significant DEGs, and summary statistics.
}
\description{
Performs differential difference analysis for a given cell type
to identify genes modulated by sex-phenotype interactions using limma.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_interaction_analysis_sc.R
\name{sex_interaction_pipeline_sc}
\alias{sex_interaction_pipeline_sc}
\title{Run sex phenotype interaction analysis pipeline for single cell data}
\usage{
sex_interaction_pipeline_sc(
  seurat_obj,
  target_cell_type_flag = "all",
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  min_logfc = 0.25,
  alpha = 0.05,
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  min_samples = 20
)
}
\arguments{
\item{seurat_obj}{Seurat object containing single cell data.}

\item{target_cell_type_flag}{Character. Specific cell type to analyze or "all".}

\item{sex}{Character. Metadata column for sex.}

\item{phenotype}{Character. Metadata column for phenotype.}

\item{celltype_col}{Character. Metadata column for cell type.}

\item{min_logfc}{Numeric. Minimum absolute log fold change threshold.}

\item{alpha}{Numeric. FDR threshold for significance.}

\item{sex_labels_vector}{Character vector specifying sex levels.}

\item{phenotype_labels_vector}{Character vector specifying phenotype levels.}

\item{min_samples}{Integer. Minimum number of cells per group (default 20).}
}
\value{
A named list containing interaction analysis results per cell type.
}
\description{
Performs sex phenotype interaction analysis per cell type
using a limma based difference in differences model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_and_reporting_functions.R
\name{plot_volcano_deg}
\alias{plot_volcano_deg}
\title{Generate volcano plots for categorized DEGs}
\usage{
plot_volcano_deg(de_results, top_n = 5, logfc_thresh = 1.5, ncol = 2)
}
\arguments{
\item{de_results}{A data.frame or named list of data.frames containing categorized DEGs.
Required columns: DEG_Type, Gene_Symbols, Male_avg_logFC, Male_FDR,
Female_avg_logFC, Female_FDR}

\item{top_n}{Number of top genes to highlight per sex (default = 5).}

\item{logfc_thresh}{Minimum absolute logFC to consider a gene for highlighting (default = 1.5).}

\item{ncol}{Number of columns for arranging plots (default = 2).}
}
\value{
Named list of combined volcano plots per cell type or "all" for bulk.
}
\description{
Creates volcano plots for all DEG categories (male-specific, female-specific,
sex-dimorphic, sex-neutral). Accepts either a single data.frame (bulk) or a
list of data.frames per cell type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{get_top_hubs}
\alias{get_top_hubs}
\title{Get Top Hub Genes by Betweenness Centrality}
\usage{
get_top_hubs(g, top_n = 10, directed = FALSE)
}
\arguments{
\item{g}{An `igraph` object representing the network.}

\item{top_n}{Integer. Number of top hub nodes to return. Default is 10.}

\item{directed}{Logical. Whether the network is directed. Default is FALSE.}
}
\value{
A named numeric vector of betweenness scores for the top hub nodes.
}
\description{
Computes normalized betweenness centrality for an igraph network and
returns the top N hub nodes ranked by centrality.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{plot_network_pipeline}
\alias{plot_network_pipeline}
\title{Generate protein-protein interaction network plots for single-cell DEGs}
\usage{
plot_network_pipeline(
  network_list,
  result_categories_list,
  top_hubs = 20,
  ncol = 2,
  show_barplot = FALSE
)
}
\arguments{
\item{network_list}{A single igraph object or a nested list: cell_type → DEG_type → igraph}

\item{result_categories_list}{A data.frame or named list matching the networks, containing DEG results.}

\item{top_hubs}{Number of hub genes to annotate (default = 20)}

\item{ncol}{Number of columns when combining DEG type plots (default = 2)}

\item{show_barplot}{Logical indicating whether hub barplots are displayed.}
}
\value{
Named list of network plots per cell type (or `"all"` for single network)
}
\description{
Accepts  a nested list of igraph networks (cell_type × DEG_type) object and produces annotated network visualizations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{ppi_pipeline}
\alias{ppi_pipeline}
\title{Run PPI PCSf pipeline for single cell DEG categories}
\usage{
ppi_pipeline(result_categories, g, target_cell_type_flag = "all")
}
\arguments{
\item{result_categories}{A named list of data.frames containing DEG results per cell type.
Each data.frame must include DEG_Type, Gene_Symbols, Male_FDR, and Female_FDR.}

\item{g}{An igraph object representing the global protein interaction network.}

\item{target_cell_type_flag}{Character. Specific cell type to analyze or "all".}
}
\value{
A named list containing PCSf networks per cell type and DEG category.
}
\description{
Builds protein protein interaction subnetworks for each cell type
based on DEG categories and FDR derived node prizes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{get_string_network}
\alias{get_string_network}
\title{Download and Process STRING Protein-Protein Interaction Network}
\usage{
get_string_network(
  organism = "9606",
  score_threshold = 700,
  use_default = TRUE
)
}
\arguments{
\item{organism}{Character string specifying the NCBI taxonomy identifier.
Default is "9606" (Homo sapiens).}

\item{score_threshold}{Numeric value between 0 and 1000 specifying the minimum
combined score threshold for including interactions. Default is 700.}

\item{use_default}{it will return the default network (9606 and score of 700)}
}
\value{
An igraph object representing the largest connected component of
        the filtered STRING network, with the following properties:
        \itemize{
          \item Undirected edges
          \item No self-loops
          \item No multiple edges
          \item Edge weights (1 - combined_score/1000)
          \item Vertex names as ENTREZ IDs
        }
}
\description{
Downloads and processes the STRING protein-protein interaction network,
converting it to a simplified igraph object. The function downloads the
network from STRING database, filters interactions by confidence score,
converts STRING IDs to ENTREZ IDs, and returns the largest connected
component as an undirected graph.
}
\details{
STRING combined scores range from 0 (low confidence) to 1000 (high confidence). 
Because PCSF treats edge weights as traversal costs to be minimized, 
scores are inverted via 1 - score/1000 so that higher-confidence interactions correspond to lower costs.
The function performs the following steps:
\enumerate{
  \item Downloads protein interactions from STRING database
  \item Filters interactions based on combined score
  \item Downloads and processes STRING ID to ENTREZ ID mappings
  \item Creates an igraph object with filtered interactions
  \item Removes self-loops and multiple edges
  \item Extracts the largest connected component
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_and_reporting_functions.R
\name{generate_violinplot_bulk}
\alias{generate_violinplot_bulk}
\title{Generate Violin Plots for Bulk Expression Data}
\usage{
generate_violinplot_bulk(
  expression_data,
  sex,
  phenotype,
  genes,
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG")
)
}
\arguments{
\item{expression_data}{A matrix or data.frame of bulk expression values
with genes in rows and samples in columns.}

\item{sex}{A vector indicating the sex of each sample.}

\item{phenotype}{A vector indicating the phenotype or condition of each sample.}

\item{genes}{Character vector of genes to visualize (maximum 10 recommended).}

\item{sex_labels_vector}{Character vector specifying sex labels.}

\item{phenotype_labels_vector}{Character vector specifying phenotype labels.}
}
\value{
A `ggplot` object containing the violin plots.
}
\description{
Creates violin plots displaying the distribution of gene expression values
across Sex and Phenotype groups for a set of selected genes. Multiple genes
(up to 10) are visualized using faceting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/XYomics-package.R
\docType{package}
\name{XYomics-package}
\alias{XYomics}
\alias{XYomics-package}
\title{XYomics: Analysis of Sex Differences in Omics Data}
\description{
The **XYomics** package provides functions for performing differential expression analysis,
pathway enrichment, gene regulatory network analysis, and comprehensive report generation for omics data.
}
\details{
This package is designed to integrate various omics analyses (e.g., functional omics and single-cell data)
with advanced visualization and reporting tools.
}
\author{
\strong{Maintainer}: Enrico Glaab \email{enrico.glaab@uni.lu}

Authors:
\itemize{
  \item Sophie Le Bars \email{sophie.lebars@uni.lu}
  \item Mohamed Soudy \email{mohamed.soudy@uni.lu}
}

Other contributors:
\itemize{
  \item Murodzhon Akhmedov [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi_analysis.R
\name{construct_ppi_pcsf}
\alias{construct_ppi_pcsf}
\title{Construct Protein-protein interaction Network using Prize-Collecting Steiner Forest}
\usage{
construct_ppi_pcsf(
  g,
  prizes,
  w = 2,
  b = 1,
  mu = 5e-04,
  seed = 1,
  min_nodes = 1
)
}
\arguments{
\item{g}{An igraph object representing the base network.}

\item{prizes}{A named numeric vector of gene scores (prizes). Names must match vertex names in g.}

\item{w}{Numeric. Edge cost scaling weight. Default is 2.}

\item{b}{Numeric. Balance between prizes and edge costs. Default is 1.}

\item{mu}{Numeric. Trade-off parameter for sparsity. Default is 5e-04.}

\item{seed}{Integer. Random seed. Default is 1.}

\item{min_nodes}{Integer. Minimum number of nodes in subnetwork. Default is 1.}
}
\value{
An igraph object representing the extracted subnetwork. Returns NULL invisibly 
  if no prize genes are present, the subnetwork is too small, or the PCSF algorithm fails.

An igraph object representing the extracted subnetwork. Returns NULL invisibly if no prize genes are present, the subnetwork is too small, or the PCSF algorithm fails
}
\description{
Constructs a condition-specific gene regulatory network based on differential
expression results using the PCSF algorithm.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_and_reporting_functions.R
\name{generate_dotplot_sc}
\alias{generate_dotplot_sc}
\title{Generate dotplot of selected genes across groups}
\usage{
generate_dotplot_sc(
  seurat_obj,
  genes,
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  target_cell_type_flag = "all"
)
}
\arguments{
\item{seurat_obj}{A Seurat object}

\item{genes}{Character vector of genes to plot}

\item{sex}{Metadata column for sex}

\item{phenotype}{Metadata column for phenotype/status}

\item{celltype_col}{Metadata column for cell types}

\item{target_cell_type_flag}{Either "all" or a specific cell type}
}
\value{
A ggplot object (DotPlot)
}
\description{
Visualizes gene expression across Sex / Phenotype / Cell type combinations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCSF.R
\name{PCSF}
\alias{PCSF}
\title{Prize-collecting Steiner Forest (PCSF)}
\usage{
PCSF(ppi, terminals, w = 2, b = 1, mu = 5e-04, dummies)
}
\arguments{
\item{ppi}{An interaction network, an \pkg{igraph} object.}

\item{terminals}{A list of terminal genes with prizes to be analyzed in the PCSF context.
A named \code{numeric} vector, where terminal genes are named same as in the interaction network
and numeric values correspond to the importance of the gene within the study.}

\item{w}{A \code{numeric} value for tuning the number of trees in the output. A default value is 2.}

\item{b}{A \code{numeric} value for tuning the node prizes. A default value is 1.}

\item{mu}{A \code{numeric} value for a hub penalization. A default value is 0.0005.}

\item{dummies}{A list of nodes that are to connected to the root of the tree. If missing the root will be connected to all terminals.}
}
\value{
The final subnetwork obtained by the PCSF.
It return an \pkg{igraph} object with the node prize and edge cost attributes.
}
\description{
\code{PCSF} returns a subnetwork obtained by solving the PCSF on the given interaction network.
}
\details{
The PCSF is a well-know problem in graph theory.
Given an undirected graph \emph{G = (V, E)}, where the vertices are labeled with prizes
\eqn{p_{v}} and the edges are labeled with costs \eqn{c_{e} > 0}, the goal is to identify
a subnetwork \emph{G' = (V', E')} with a forest structure. The target is to minimize
the total edge costs in \emph{E'}, the total node prizes left out of \emph{V'}, and the
number of trees in \emph{G'}. This is equivalent to  minimization of the following
objective function:

\deqn{F(G')= Minimize \sum_{ e \in E'} c_{e} + \beta*\sum_{v \not\in V'} p_v + \omega*k}
where, \emph{k} is the number of trees in the forest, and it is regulated by parameter \eqn{\omega}.
The parameter \eqn{\beta} is used to tune the prizes of nodes.

This optimization problem nicely maps onto the problem of finding differentially
enriched subnetworks in the cell protein-protein interaction (PPI) network.
The vertices of interaction network correspond to genes or proteins, and edges
represent the interactions among them. We can assign prizes
to vertices based on measurements of differential expression, copy number, or
mutation, and costs to edges based on confidence scores for those intra-cellular
interactions from experimental observation, yielding a proper input to the PCSF
problem. Vertices that are assigned a prize are referred to \emph{terminal} nodes,
whereas the vertices which are not observed in patient data are not assigned a
prize and are called \emph{Steiner} nodes. After scoring the interactome, the
PCSF is used to detect a relevant subnetwork (forest), which corresponds to a
portion of the interactome, where many genes are highly correlated in terms of
their functions and may regulate the differentially active biological process
of interest. The PCSF aims to identify  neighborhoods in interaction networks
potentially belonging to the key dysregulated pathways of a disease.
In order to avoid a bias towards the hub nodes of PPI networks to appear in solution
of PCSF, we penalize the prizes of \emph{Steiner} nodes according to their degree
distribution in PPI, and it is regulated by parameter \eqn{\mu}:

 \deqn{p'_{v} = p_{v} - \mu*degree(v)}

 The parameter \eqn{\mu} also affects the total number of \emph{Steiner} nodes in the solution.
 Higher the value of \eqn{\mu} smaller the number of \emph{Steiners} in the subnetwork,
 and vice-versa. Based on our previous analysis the recommended range of \eqn{\mu}
 for biological networks is between 1e-4 and 5e-2, and users can choose the values
 resulting subnetworks with vertex sets that have desirable \emph{Steiner/terminal}
 node ratio and average \emph{Steiner/terminal} in-degree ratio
 in the template interaction network.
}
\references{
Akhmedov M., LeNail A., Bertoni F., Kwee I., Fraenkel E., and Montemanni R. (2017)
A Fast Prize-Collecting Steiner Forest Algorithm for Functional Analyses in Biological Networks.
\emph{Lecture Notes in Computer Science}, to appear.
}
\author{
Murodzhon Akhmedov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_enrichment.R
\name{categorized_enrich}
\alias{categorized_enrich}
\title{Perform Pathway Enrichment Analysis for Pre-Categorized DEGs}
\usage{
categorized_enrich(
  DEGs_category,
  enrichment_db = "KEGG",
  organism = "hsa",
  gene_type = "SYMBOL",
  org_db = NULL,
  custom_db = NULL,
  pvalueCutoff = 0.05,
  qvalueCutoff = 0.2,
  return_df = FALSE
)
}
\arguments{
\item{DEGs_category}{Data frame with columns 'DEG_Type' and 'Gene_Symbols'.}

\item{enrichment_db}{"KEGG", "GO", "REACTOME", or "CUSTOM" (default: "KEGG").}

\item{organism}{Organism code for KEGG (default: "hsa").}

\item{gene_type}{Gene identifier type supported by OrgDb
(e.g., "SYMBOL", "ENTREZID", "ENSEMBL", "UNIPROT").}

\item{org_db}{OrgDb object for gene annotation (default: org.Hs.eg.db).}

\item{custom_db}{TERM2GENE data frame, required if enrichment_db = "CUSTOM".}

\item{pvalueCutoff}{Numeric p-value cutoff (default: 0.05).}

\item{qvalueCutoff}{Numeric q-value cutoff (default: 0.2).}

\item{return_df}{Logical, if TRUE returns data.frames, else enrichResult objects (default: FALSE).}
}
\value{
Named list of enrichment results per DEG_Type.
}
\description{
Performs pathway enrichment analysis for categorized DEGs using GO, KEGG, Reactome, or a custom TERM2GENE database.
}
\details{
Gene IDs converted only when needed; KEGG/Reactome require ENTREZID; GO/CUSTOM use provided gene_type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_interaction_analysis_bulk.R
\name{sex_interaction_analysis_bulk}
\alias{sex_interaction_analysis_bulk}
\title{Perform Sex-Phenotype Interaction Analysis for Bulk Data}
\usage{
sex_interaction_analysis_bulk(
  expression_data,
  phenotype,
  sex,
  phenotype_labels_vector = c("WT", "TG"),
  sex_labels_vector = c("F", "M"),
  min_logfc = 0.25,
  alpha = 0.05,
  min_samples = 20
)
}
\arguments{
\item{expression_data}{Numeric matrix of expression data (features x samples).}

\item{phenotype}{Character or factor vector of sample phenotypes.}

\item{sex}{Character or factor vector of sample sexes.}

\item{phenotype_labels_vector}{Character vector of phenotype levels (default c("WT","TG")).}

\item{sex_labels_vector}{Character vector of sex levels (default c("F","M")).}

\item{min_logfc}{Numeric. Minimum absolute log fold change for reporting significant genes (default 0.25).}

\item{alpha}{Numeric. FDR threshold for significance (default 0.05).}

\item{min_samples}{Integer. Minimum number of samples per group (default 20).}
}
\value{
A list with all DE results, filtered significant DEGs, and summary statistics.
}
\description{
Identifies genes whose expression is modulated by the interaction
between sex and phenotype using a differential difference contrast
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_stratified_analysis_bulk.R
\name{validate_input_bulk}
\alias{validate_input_bulk}
\title{Validate bulk input data for sex-stratified analysis}
\usage{
validate_input_bulk(
  expression_data,
  sex,
  phenotype,
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  min_samples = 3
)
}
\arguments{
\item{expression_data}{Numeric matrix (features x samples)}

\item{sex}{Vector of sex labels}

\item{phenotype}{Vector of phenotype labels}

\item{sex_labels_vector}{Expected sex labels (length 2)}

\item{phenotype_labels_vector}{Expected phenotype labels (length 2)}

\item{min_samples}{Minimum samples per group}
}
\value{
A list with validation status, ratios, imbalance flag, and group counts
}
\description{
Validate bulk input data for sex-stratified analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_stratified_analysis_sc.R
\name{sex_stratified_pipeline_sc}
\alias{sex_stratified_pipeline_sc}
\title{Run sex stratified differential expression pipeline for single cell data}
\usage{
sex_stratified_pipeline_sc(
  seurat_obj,
  target_cell_type_flag = "all",
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  test.use = "wilcox",
  alpha = 0.05,
  beta = 0.5,
  min_abs_logfc = 0.25,
  min_samples = 3
)
}
\arguments{
\item{seurat_obj}{Seurat object containing single cell data.}

\item{target_cell_type_flag}{Character. Cell type to analyze.}

\item{sex}{Character. Metadata column for sex.}

\item{phenotype}{Character. Metadata column for phenotype.}

\item{celltype_col}{Character. Metadata column for cell type.}

\item{sex_labels_vector}{Character vector specifying sex levels.}

\item{phenotype_labels_vector}{Character vector specifying phenotype levels.}

\item{test.use}{Character. Statistical test used in FindMarkers.}

\item{alpha}{Numeric. FDR threshold for significance.}

\item{beta}{Numeric. P value threshold for exclusion in opposite sex.}

\item{min_abs_logfc}{Numeric. Minimum absolute log2 fold change for categorization.}

\item{min_samples}{Integer. Minimum number of cells per group (default 3).}
}
\value{
A named list of categorized differential expression results per cell type.
}
\description{
Performs sex specific differential expression analysis per cell type
and categorizes genes into male specific, female specific, sex dimorphic,
and sex neutral groups.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/categorise_sex.R
\name{categorize_sex}
\alias{categorize_sex}
\title{Compute sex-specific differentially expressed genes (DEGs) per category}
\usage{
categorize_sex(
  male_degs,
  female_degs,
  alpha = 0.05,
  beta = 0.5,
  min_abs_logfc = 0.25
)
}
\arguments{
\item{male_degs}{Data frame containing male differential expression results from one specific cell-type or bulk dataset.}

\item{female_degs}{Data frame containing female differential expression results from one specific cell-type or bulk dataset.}

\item{alpha}{Numeric. FDR threshold for significance.}

\item{beta}{Numeric. P-value threshold for excluding genes in opposite sex.}

\item{min_abs_logfc}{Numeric. Minimum absolute log2 fold change threshold.}
}
\value{
Data frame containing categorized DEGs with associated statistics.
}
\description{
Identifies male-specific, female-specific, sex-dimorphic, and sex-neutral DEGs from differential expression results. 
"sex-modulated" requires the interaction mode and is not identified by categorize_sex.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_and_reporting_functions.R
\name{plot_enrichment_dotplots}
\alias{plot_enrichment_dotplots}
\title{Create enrichment dotplots from enrichResult objects}
\usage{
plot_enrichment_dotplots(enrichment_results, showCategory = 10, ncol = 2)
}
\arguments{
\item{enrichment_results}{A list or nested list of enrichResult objects.}

\item{showCategory}{Number of categories to display (default = 10).}

\item{ncol}{Number of columns for arranged plots (default = 2).}
}
\value{
Named list of combined ggplots. For nested input, names correspond to cell types;
for flat input, a single `"all"` entry is returned.
}
\description{
Generates dotplots from enrichment results. Input must contain enrichResult
objects (clusterProfiler / ReactomePA). Invalid entries are skipped with messages.
Works for single enrichResult, flat list (DEG types), or nested list (cell type → DEG type).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_enrichment.R
\name{convert_gene_ids}
\alias{convert_gene_ids}
\title{Convert gene identifiers in a DEG table}
\usage{
convert_gene_ids(
  df,
  fromType,
  toType = "SYMBOL",
  org_db = NULL,
  gene_col = "Gene_Symbols",
  remove_na = TRUE,
  remove_duplicates = TRUE
)
}
\arguments{
\item{df}{Data frame containing a gene column}

\item{fromType}{Original ID type (e.g. "UNIPROT", "ENSEMBL")}

\item{toType}{Target ID type (e.g. "SYMBOL", "ENTREZID")}

\item{org_db}{OrgDb object (default: org.Hs.eg.db)}

\item{gene_col}{Column name containing gene IDs (default: "Gene_Symbols")}

\item{remove_na}{Remove unmapped genes (default: TRUE)}

\item{remove_duplicates}{Remove duplicated mapped IDs (default: TRUE)}
}
\value{
Updated data frame with converted gene IDs
}
\description{
Convert gene identifiers in a DEG table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_stratified_analysis_sc.R
\name{sex_stratified_analysis_sc}
\alias{sex_stratified_analysis_sc}
\title{Perform sex-stratified differential expression analysis for single-cell Seurat object}
\usage{
sex_stratified_analysis_sc(
  seurat_obj,
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  test.use = "wilcox",
  min_samples = 3
)
}
\arguments{
\item{seurat_obj}{A Seurat object containing single-cell RNA-seq data.}

\item{sex}{Character. Name of the column in metadata indicating sex.}

\item{phenotype}{Character. Name of the column indicating phenotype/condition.}

\item{celltype_col}{Character. Name of the column indicating cell type.}

\item{sex_labels_vector}{A character vector of length two defining the labels for sex (e.g., c("F", "M")). The first element must correspond to "female" and the second to "male".}

\item{phenotype_labels_vector}{Character vector of length 2 specifying the reference and test phenotypes (e.g., c("WT", "TG")).}

\item{test.use}{Statistical test to use in FindMarkers.}

\item{min_samples}{Integer. Minimum number of cells per group (default 3).}
}
\value{
A list with two elements: `male_DEGs` and `female_DEGs`, each containing a list of DEGs per cell type.
}
\description{
Perform sex-stratified differential expression analysis for single-cell Seurat object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_stratified_analysis_bulk.R
\name{sex_stratified_analysis_bulk}
\alias{sex_stratified_analysis_bulk}
\title{Perform differential expression analysis within each sex}
\usage{
sex_stratified_analysis_bulk(
  expression_data,
  sex,
  phenotype,
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  min_samples = 3,
  method = c("limma", "deseq2", "edger")
)
}
\arguments{
\item{expression_data}{A numeric matrix of expression data with features in rows
and samples in columns. For \code{method = "limma"}, this can be a normalized
log-expression matrix or a raw count matrix. For \code{method = "deseq2"} or
\code{method = "edger"}, this must be a raw (non-negative integer) count matrix.}

\item{sex}{A vector indicating sex for each sample.}

\item{phenotype}{A vector indicating phenotype labels for each sample.}

\item{sex_labels_vector}{A character vector of length two defining the labels for sex (e.g., c("F", "M")). The first element must correspond to "female" and the second to "male".}

\item{phenotype_labels_vector}{Character vector specifying phenotype levels. The
first element is the reference, the second the comparison; contrast = 2 - 1
(e.g. c("WT","TG")).}

\item{min_samples}{Integer. Minimum number of samples per group (default 3).}

\item{method}{Character. Differential expression method to use. One of
\code{"limma"} (default), \code{"deseq2"}, or \code{"edger"}.}
}
\value{
A list with three elements:
  \describe{
    \item{male_DEGs}{A data.frame of DE results for males.}
    \item{female_DEGs}{A data.frame of DE results for females.}
    \item{validation}{Output from \code{validate_input_bulk()}.}
  }
  Regardless of the method, each DEG data.frame contains the standardized
  columns \code{avg_log2FC}, \code{p_val}, \code{p_val_adj}, and \code{gene},
  with gene names as rownames. Additional method-specific columns (e.g.
  \code{baseMean} for DESeq2, \code{logCPM} for edgeR, \code{AveExpr} for limma)
  are retained for reference.
}
\description{
This function identifies differentially expressed genes between phenotype groups
separately for each sex using limma (default), DESeq2, or edgeR.
}
\details{
For each sex, the function subsets the data and fits the appropriate
  model:
  \itemize{
   \item \strong{limma}: Fits a linear model via \code{lmFit} + \code{eBayes}
      on the input matrix directly. Suitable for microarray, proteomics, or
      pre-normalized RNA-seq data. If raw RNA-seq counts are provided, the
      function automatically detects integer matrices and applies
      \code{limma-voom} internally with a warning. For full control, consider
      using \code{"edger"} or \code{"deseq2"} instead, or apply \code{voom}
      externally and pass the transformed data with
      \code{method = "limma"}.
    \item \strong{DESeq2}: Fits a negative binomial GLM via
      \code{DESeq2::DESeq()}. Requires raw integer counts. Performs internal
      normalization (median-of-ratios) and dispersion estimation.
    \item \strong{edgeR}: Fits a negative binomial GLM via
      \code{edgeR::glmQLFit()} + \code{edgeR::glmQLFTest()}. Requires raw
      integer counts. Performs TMM normalization internally.
  }
  Positive logFC values indicate higher expression in the test phenotype
  (second element of \code{phenotype_labels_vector}) compared to the reference
  (first element).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex_stratified_analysis_sc.R
\name{validate_input_sc}
\alias{validate_input_sc}
\title{Validate single-cell input data for sex-stratified analysis}
\usage{
validate_input_sc(
  seurat_obj,
  sex = "sex",
  phenotype = "status",
  celltype_col = "cell_type",
  sex_labels_vector = c("F", "M"),
  phenotype_labels_vector = c("WT", "TG"),
  min_cells = 3
)
}
\arguments{
\item{seurat_obj}{Seurat object}

\item{sex}{Column name for sex}

\item{phenotype}{Column name for phenotype}

\item{celltype_col}{Column name for cell type}

\item{sex_labels_vector}{Expected sex labels (length 2)}

\item{phenotype_labels_vector}{Expected phenotype labels (length 2)}

\item{min_cells}{Minimum number of cells per group}
}
\value{
A list with validation status, ratios, imbalance flag, and group counts
}
\description{
Validate single-cell input data for sex-stratified analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{call_sr}
\alias{call_sr}
\title{Internal function \code{call_sr}}
\usage{
call_sr(from, to, cost, node_names, node_prizes)
}
\arguments{
\item{from}{A \code{CharacterVector} that corresponds to \code{head} nodes of the edges.}

\item{to}{A \code{CharacterVector} that corresponds the \code{tail} nodes of the edges.}

\item{cost}{A \code{NumericVector} which represents the edge weights.}

\item{node_names}{A \code{CharacterVector} demonstrates the names of the nodes.}

\item{node_prizes}{A \code{NumericVector} which corresponds to the node prizes.}
}
\description{
This function is internally used to solve the PCST.
}
\author{
Murodzhon Akhmedov
}
\keyword{internal}
